#ifndef ELF2_H
#define ELF2_H

#include "elf.h"
#include "tms9918.h"
#include "pixie.h"
#include "mc6847.h"
#include "mc6845.h"
#include "i8275.h"
#include "ledmodule.h"
#include "printer.h"
#include "elfconfiguration.h"

class Elf2Screen : public Panel
{
public:
	Elf2Screen(wxWindow *parent, const wxSize& size);
	~Elf2Screen();

	void init();
	void onPaint(wxPaintEvent&event);
	void onMouseRelease(wxMouseEvent& event);

private:
};

class Elf2 : public wxFrame, public MainElf 
{
public:
	Elf2(const wxString& title, const wxPoint& pos, const wxSize& size, double clock, ElfConfiguration conf);
	Elf2() {};
	~Elf2();

	void onClose(wxCloseEvent&WXUNUSED(event));
	void charEvent(int keycode);
	bool keyDownPressed(int keycode);
	bool keyUpReleased(int keycode);

	void onRun();
	void onButtonRelease(wxCommandEvent& event);
	void onButtonPress(wxCommandEvent& event);
	void onInButtonPress();
	void onInButtonRelease();
	void onHexKeyDown(int keycode);
	void onHexKeyUp(int keycode);

	void configureComputer();
	void reDefineKeysA(int *, int *, int *);
	void reDefineKeysB(int *, int *, int *);
	void initComputer();
	Byte ef(int flag);
	Byte ef4();
	Byte in(Byte port, Word address);
	Byte getData();
	void out(Byte port, Word address, Byte value);
	void cycle(int type);
	void showData(Byte value);

	void autoBoot();
	void switchQ(int value);
	int getMpButtonState();
	void onRunButton();
	void onMpButton();
	void onLoadButton();
	void onNumberKeyDown(wxCommandEvent& event);
	void onNumberKeyUp(wxCommandEvent& event);

	void startComputer();
	Byte readMem(Word addr);
	void writeMem(Word addr, Byte value, bool writeRom);
	void cpuInstruction();
	void configureElfExtensions();
	void moveWindows();
	void updateTitle(wxString Title);
	void setForceUpperCase(bool status);
	void onReset();
	void sleepComputer(long ms);

	Byte getTmsMemory(int address) {return tmsPointer->getTmsMemory(address);};
	void setTmsMemory(int address, Byte value) {tmsPointer->setTmsMemory(address, value);};
	Byte read8275CharRom(Word addr);
	void write8275CharRom(Word addr, Byte value);
	Byte read6845CharRom(Word addr);
	void write6845CharRom(Word addr, Byte value);
	Byte read6847CharRom(Word addr);
	void write6847CharRom(Word addr, Byte value);
	int readDirect6847(Word addr); 
	Word get6847RamMask();
	void writeDirect6847(Word addr, int value); 
	void ledTimeout();
	void setLedMs(long ms);
	Byte getKey(Byte vtOut);
	void activateMainWindow();

private:
	class Elf2Screen *elf2ScreenPointer;

	Tms9918 *tmsPointer;
	Pixie *pixiePointer;
	MC6845 *mc6845Pointer;
	mc6847 *mc6847Pointer;
	i8275 *i8275Pointer;

	Byte switches_;
	int runButtonState_;
	int mpButtonState_;
	int loadButtonState_;
	Byte ef4State_;
	Byte ef3State_;

	int hexKeyDefA_[16];
	int keyDefGameHexA_[5];
	int keyDefGameValueA_[5];
	int keyDefGameHexB_[5];
	int keyDefGameValueB_[5];

	double elfClockSpeed_;

	long offset_;

	DECLARE_EVENT_TABLE()
};

#endif  // ELF2_H
