/*
 *******************************************************************
 *** This software is copyright 2008 by Marcel van Tongeren      ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 *******************************************************************
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#ifdef __WXMSW__
#include "wx/msw/private.h"
#endif

#if !wxUSE_COMBOCTRL
    #error "Please set wxUSE_COMBOCTRL to 1 and rebuild the library."
#endif

#if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__)
#include "app_icon.xpm"
#endif

#include "main.h"
#include "elf2khex.h"

Elf2KHexScreen::Elf2KHexScreen(wxWindow *parent, const wxSize& size)
: Panel(parent, size)
{
}

Elf2KHexScreen::~Elf2KHexScreen()
{
	for (int i=0;i<16;i++)
	{
		delete buttonPointer[i];
	}
	delete runButtonPointer;
	delete mpButtonPointer;
	delete loadButtonPointer;
	delete resetButtonPointer;
	delete inButtonPointer;

	delete resetUpBitmapPointer;
	delete resetDownBitmapPointer;
	delete loadUpBitmapPointer;
	delete loadDownBitmapPointer;
	delete mpUpBitmapPointer;
	delete mpDownBitmapPointer;
	delete runUpBitmapPointer;
	delete runDownBitmapPointer;
	delete inUpBitmapPointer;
	delete inDownBitmapPointer;
}

void Elf2KHexScreen::init()
{
	keyStart_ = 0;
	keyEnd_ = 0;
	lastKey_ = 0;
	forceUpperCase_ = p_Main->getUpperCase(ELF2K);

	wxClientDC dc(this);
	wxString buttonText;
	int x, y;

	dc.SetBackground(wxBrush(wxColour(255,255,255)));
	dc.DrawRectangle(0, 0, 186, 126);

	resetUpBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/resetButtonUp.png", wxBITMAP_TYPE_PNG);
	resetDownBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/resetButtonDown.png", wxBITMAP_TYPE_PNG);
	loadUpBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/loadButtonUp.png", wxBITMAP_TYPE_PNG);
	loadDownBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/loadButtonDown.png", wxBITMAP_TYPE_PNG);
	mpUpBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/mpButtonUp.png", wxBITMAP_TYPE_PNG);
	mpDownBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/mpButtonDown.png", wxBITMAP_TYPE_PNG);
	runUpBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/runButtonUp.png", wxBITMAP_TYPE_PNG);
	runDownBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/runButtonDown.png", wxBITMAP_TYPE_PNG);
	inUpBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/inButtonUp.png", wxBITMAP_TYPE_PNG);
	inDownBitmapPointer = new wxBitmap(p_Main->getApplicationDir() + "images/inButtonDown.png", wxBITMAP_TYPE_PNG);

	wxSize buttonSize;
	int offSetX, offSetY;

	#if defined(__WXGTK__)
	buttonSize.x = 30;
	buttonSize.y = 30;
	offSetX = -4;
	offSetY = -4;
#else
	buttonSize.x = 30;
	buttonSize.y = 30;
	offSetX = 0;
	offSetY = 0;
#endif

#if wxCHECK_VERSION(2, 9, 0)
	runButtonPointer = new wxButton(this, 20, wxEmptyString, wxPoint(153+offSetX, 63+offSetY), buttonSize, wxBU_EXACTFIT);
	runButtonPointer->SetBitmap(*runUpBitmapPointer);
	mpButtonPointer = new wxButton(this, 21, wxEmptyString, wxPoint(123+offSetX, 63+offSetY), buttonSize, wxBU_EXACTFIT);
	mpButtonPointer->SetBitmap(*mpUpBitmapPointer);
	loadButtonPointer = new wxButton(this, 22, wxEmptyString, wxPoint(153+offSetX, 33+offSetY), buttonSize, wxBU_EXACTFIT);
	loadButtonPointer->SetBitmap(*loadUpBitmapPointer);
	resetButtonPointer = new wxButton(this, 23, wxEmptyString, wxPoint(153+offSetX, 3+offSetY), buttonSize, wxBU_EXACTFIT);
	resetButtonPointer->SetBitmap(*resetUpBitmapPointer);
	inButtonPointer = new wxButton(this, 24, wxEmptyString, wxPoint(123+offSetX, 93+offSetY), buttonSize, wxBU_EXACTFIT);
	inButtonPointer->SetBitmap(*inUpBitmapPointer);
#else
	runButtonPointer = new wxBitmapButton(this, 20, *runUpBitmapPointer, wxPoint(153+offSetX, 63+offSetY), buttonSize, wxBU_EXACTFIT);
	mpButtonPointer = new wxBitmapButton(this, 21, *mpUpBitmapPointer, wxPoint(123+offSetX, 63+offSetY), buttonSize, wxBU_EXACTFIT);
	loadButtonPointer = new wxBitmapButton(this, 22, *loadUpBitmapPointer, wxPoint(153+offSetX, 33+offSetY), buttonSize, wxBU_EXACTFIT);
	resetButtonPointer = new wxBitmapButton(this, 23, *resetUpBitmapPointer, wxPoint(153+offSetX, 3+offSetY), buttonSize, wxBU_EXACTFIT);
	inButtonPointer = new wxBitmapButton(this, 24, *inUpBitmapPointer, wxPoint(123+offSetX, 93+offSetY), buttonSize, wxBU_EXACTFIT);
#endif

	for (int i=0;i<16;i++)
	{
		buttonText.Printf("%01X", i);
		x = 3+(i&0x3)*30;
		y = 93 -(int)i/4*30;
		buttonPointer[i] = new wxButton(this, i, buttonText, wxPoint(x+offSetX, y+offSetY), buttonSize);
	}
	this->connectKeyEvent(this);
}

void Elf2KHexScreen::onPaint(wxPaintEvent&WXUNUSED(event))
{
	wxPaintDC dc(this);
	dc.SetBackground(wxBrush(wxColour(255,255,255)));
	dc.DrawRectangle(0, 0, 186, 126);
}

void Elf2KHexScreen::resetUp()
{
#if wxCHECK_VERSION(2, 9, 0)
	resetButtonPointer->SetBitmap(*resetUpBitmapPointer);
#else
	resetButtonPointer->SetBitmapLabel(*resetUpBitmapPointer);
#endif
}

void Elf2KHexScreen::resetDown()
{
#if wxCHECK_VERSION(2, 9, 0)
	resetButtonPointer->SetBitmap(*resetDownBitmapPointer);
#else
	resetButtonPointer->SetBitmapLabel(*resetDownBitmapPointer);
#endif
}

void Elf2KHexScreen::inUp()
{
#if wxCHECK_VERSION(2, 9, 0)
	inButtonPointer->SetBitmap(*inUpBitmapPointer);
#else
	inButtonPointer->SetBitmapLabel(*inUpBitmapPointer);
#endif
}

void Elf2KHexScreen::inDown()
{
#if wxCHECK_VERSION(2, 9, 0)
	inButtonPointer->SetBitmap(*inDownBitmapPointer);
#else
	inButtonPointer->SetBitmapLabel(*inDownBitmapPointer);
#endif
}

void Elf2KHexScreen::runUp()
{
#if wxCHECK_VERSION(2, 9, 0)
	runButtonPointer->SetBitmap(*runUpBitmapPointer);
#else
	runButtonPointer->SetBitmapLabel(*runUpBitmapPointer);
#endif
}

void Elf2KHexScreen::runDown()
{
#if wxCHECK_VERSION(2, 9, 0)
	runButtonPointer->SetBitmap(*runDownBitmapPointer);
#else
	runButtonPointer->SetBitmapLabel(*runDownBitmapPointer);
#endif
}

void Elf2KHexScreen::mpUp()
{
#if wxCHECK_VERSION(2, 9, 0)
	mpButtonPointer->SetBitmap(*mpUpBitmapPointer);
#else
	mpButtonPointer->SetBitmapLabel(*mpUpBitmapPointer);
#endif
}

void Elf2KHexScreen::mpDown()
{
#if wxCHECK_VERSION(2, 9, 0)
	mpButtonPointer->SetBitmap(*mpDownBitmapPointer);
#else
	mpButtonPointer->SetBitmapLabel(*mpDownBitmapPointer);
#endif
}

void Elf2KHexScreen::loadUp()
{
#if wxCHECK_VERSION(2, 9, 0)
	loadButtonPointer->SetBitmap(*loadUpBitmapPointer);
#else
	loadButtonPointer->SetBitmapLabel(*loadUpBitmapPointer);
#endif
}

void Elf2KHexScreen::loadDown()
{
#if wxCHECK_VERSION(2, 9, 0)
	loadButtonPointer->SetBitmap(*loadDownBitmapPointer);
#else
	loadButtonPointer->SetBitmapLabel(*loadDownBitmapPointer);
#endif
}

BEGIN_EVENT_TABLE(Elf2Khex, wxFrame)
	EVT_CLOSE (Elf2Khex::onClose)
	EVT_BUTTON(0, Elf2Khex::onNumberKey)
	EVT_BUTTON(1, Elf2Khex::onNumberKey)
	EVT_BUTTON(2, Elf2Khex::onNumberKey)
	EVT_BUTTON(3, Elf2Khex::onNumberKey)
	EVT_BUTTON(4, Elf2Khex::onNumberKey)
	EVT_BUTTON(5, Elf2Khex::onNumberKey)
	EVT_BUTTON(6, Elf2Khex::onNumberKey)
	EVT_BUTTON(7, Elf2Khex::onNumberKey)
	EVT_BUTTON(8, Elf2Khex::onNumberKey)
	EVT_BUTTON(9, Elf2Khex::onNumberKey)
	EVT_BUTTON(10, Elf2Khex::onNumberKey)
	EVT_BUTTON(11, Elf2Khex::onNumberKey)
	EVT_BUTTON(12, Elf2Khex::onNumberKey)
	EVT_BUTTON(13, Elf2Khex::onNumberKey)
	EVT_BUTTON(14, Elf2Khex::onNumberKey)
	EVT_BUTTON(15, Elf2Khex::onNumberKey)
	EVT_BUTTON(20, Elf2Khex::onRunButton)
	EVT_BUTTON(21, Elf2Khex::onMpButton)
	EVT_BUTTON(22, Elf2Khex::onLoadButton)
	EVT_BUTTON(23, Elf2Khex::onResetButton)
	EVT_BUTTON(24, Elf2Khex::onInButton)
	EVT_TIMER(950, Elf2Khex::onInButtonTimer)
END_EVENT_TABLE()

Elf2Khex::Elf2Khex(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
	SetIcon(wxICON(app_icon));

	elf2KHexScreenPointer = new Elf2KHexScreen(this, size);
	elf2KHexScreenPointer->init();

	keypadValue_ = 0;
	nextNybble_ = 'H';
	this->SetClientSize(size);

	inButtonTimerPointer = new wxTimer(this, 950);
}

Elf2Khex::~Elf2Khex()
{
	delete elf2KHexScreenPointer;

	inButtonTimerPointer->Stop();
	delete inButtonTimerPointer;
}

void Elf2Khex::onClose(wxCloseEvent&WXUNUSED(event))
{
	p_Computer->removeElfHex();
	Destroy();
}

void Elf2Khex::onNumberKey(wxCommandEvent&event)
{
	wxClientDC dc(this);

	int i = event.GetId();
	keypadValue_= (nextNybble_ == 'H')?(keypadValue_&15)+(i<<4):(keypadValue_&0xf0)+i;
	if (nextNybble_ == 'H')
	{
		nextNybble_ = 'L';
	}
	else
	{
		nextNybble_ = 'H';
	}
	elf2KHexScreenPointer->inDown();
}

Byte Elf2Khex::getData()
{
	return keypadValue_;
}

void Elf2Khex::onInButton(wxCommandEvent&WXUNUSED(event))
{
	inButtonTimerPointer->Start(1000, wxTIMER_ONE_SHOT );
	p_Computer->onInButtonPress(keypadValue_);
}

void Elf2Khex::onInButtonTimer(wxTimerEvent&WXUNUSED(event))
{
	p_Computer->onInButtonRelease();
}


void Elf2Khex::onRunButton(wxCommandEvent&WXUNUSED(event))
{
	p_Computer->onRun();
}

void Elf2Khex::onLoadButton(wxCommandEvent&WXUNUSED(event))
{
	p_Computer->onLoadButton();
}

void Elf2Khex::onMpButton(wxCommandEvent&WXUNUSED(event))
{
	p_Computer->onMpButton();
}

void Elf2Khex::setRunButtonUp(int up)
{
	if (up == 1)
		elf2KHexScreenPointer->runDown();
	else
		elf2KHexScreenPointer->runUp();
}

void Elf2Khex::setInButtonUp(bool up)
{
	if (up)
		elf2KHexScreenPointer->inUp();
	else
		elf2KHexScreenPointer->inDown();
}

void Elf2Khex::setLoadButtonUp(int up)
{
	if (up)
		elf2KHexScreenPointer->loadUp();
	else
		elf2KHexScreenPointer->loadDown();
}

void Elf2Khex::setMpButtonUp(int up)
{
	if (up)
		elf2KHexScreenPointer->mpUp();
	else
		elf2KHexScreenPointer->mpDown();
}

void Elf2Khex::onResetButton(wxCommandEvent&WXUNUSED(event))
{
	p_Computer->onReset();
}

Byte Elf2Khex::getKey(Byte vtOut)
{
	return elf2KHexScreenPointer->getKey(vtOut);
}

