/*
 *******************************************************************
 *** This software is copyright 2008 by Marcel van Tongeren      ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 *******************************************************************
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#ifdef __WXMSW__
#include "wx/msw/private.h"
#endif

#if !wxUSE_COMBOCTRL
    #error "Please set wxUSE_COMBOCTRL to 1 and rebuild the library."
#endif

#include "wx/xrc/xmlres.h"
#include "wx/spinctrl.h"
#include "wx/tglbtn.h"

#include "main.h"
#include "guicomx.h"
#include "printer.h"
#include "guieprom.h"

DEFINE_EVENT_TYPE(STATUS_LED_ON)
DEFINE_EVENT_TYPE(STATUS_LED_OFF)
DEFINE_EVENT_TYPE(EXP_LED_ON)
DEFINE_EVENT_TYPE(EXP_LED_OFF)
DEFINE_EVENT_TYPE(STATUS_BAR_1870)

BEGIN_EVENT_TABLE(GuiComx, GuiElf)

	EVT_TEXT(XRCID("MainRomComx"), GuiMain::onMainRom1Text)
	EVT_COMBOBOX(XRCID("MainRomComx"), GuiMain::onMainRom1Text)
	EVT_BUTTON(XRCID("RomButtonComx"), GuiMain::onMainRom1)

	EVT_TEXT(XRCID("ExpRomComx"), GuiComx::onExpansionRomText)
	EVT_COMBOBOX(XRCID("ExpRomComx"), GuiComx::onExpansionRomText)
	EVT_BUTTON(XRCID("ExpRomButtonComx"), GuiComx::onExpansionRom)

	EVT_TEXT(XRCID("Cart1RomComx"), GuiComx::onCard1RomText)
	EVT_COMBOBOX(XRCID("Cart1RomComx"), GuiComx::onCard1RomText)
	EVT_BUTTON(XRCID("Cart1RomButtonComx"), GuiComx::onCard1Rom)

	EVT_TEXT(XRCID("Cart2RomComx"), GuiComx::onCard2RomText)
	EVT_COMBOBOX(XRCID("Cart2RomComx"), GuiComx::onCard2RomText)
	EVT_BUTTON(XRCID("Cart2RomButtonComx"), GuiComx::onCard2Rom)

	EVT_TEXT(XRCID("Cart3RomComx"), GuiComx::onCard3RomText)
	EVT_COMBOBOX(XRCID("Cart3RomComx"), GuiComx::onCard3RomText)
	EVT_BUTTON(XRCID("Cart3RomButtonComx"), GuiComx::onCard3Rom)

	EVT_TEXT(XRCID("Cart4RomComx"), GuiComx::onCard4RomText)
	EVT_COMBOBOX(XRCID("Cart4RomComx"), GuiComx::onCard4RomText)
	EVT_BUTTON(XRCID("Cart4RomButtonComx"), GuiComx::onCard4Rom)

	EVT_BUTTON(XRCID("Disk1ButtonComx"), GuiComx::onComxDisk1)
	EVT_TEXT(XRCID("Disk1FileComx"), GuiComx::onComxDiskText1)
	EVT_BUTTON(XRCID("EjectDisk1Comx"), GuiComx::onComxDiskEject1)
	EVT_BUTTON(XRCID("Disk2ButtonComx"), GuiComx::onComxDisk2)
	EVT_TEXT(XRCID("Disk2FileComx"), GuiComx::onComxDiskText2)
	EVT_BUTTON(XRCID("EjectDisk2Comx"), GuiComx::onComxDiskEject2)
	EVT_BUTTON(XRCID("PrintFileButtonComx"), GuiMain::onPrintFile)
	EVT_TEXT(XRCID("PrintFileComx"), GuiComx::onComxPrintFileText)
	EVT_CHOICE(XRCID("PrintModeComx"), GuiComx::onComxPrintMode)
	EVT_BUTTON(XRCID("PrintButtonComx"), GuiComx::onComxPrintButton)

	EVT_BUTTON(XRCID("ScreenDumpFileButtonComx"), GuiMain::onScreenDumpFile)
	EVT_TEXT(XRCID("ScreenDumpFileComx"), GuiMain::onScreenDumpFileText)
	EVT_COMBOBOX(XRCID("ScreenDumpFileComx"), GuiMain::onScreenDumpFileText)

	EVT_BUTTON(XRCID("ScreenDumpF5Comx"), GuiMain::onScreenDump)
	EVT_BUTTON(XRCID("CasButtonComx"), GuiMain::onCassette)
	EVT_BUTTON(XRCID("EjectCasComx"), GuiMain::onCassetteEject)
	EVT_TEXT(XRCID("WavFileComx"), GuiMain::onCassetteText)
	EVT_BUTTON(XRCID("KeyFileButtonComx"), GuiMain::onKeyFile)
	EVT_TEXT(XRCID("KeyFileComx"), GuiMain::onKeyFileText)
	EVT_BUTTON(XRCID("EjectKeyFileComx"), GuiMain::onKeyFileEject)
	EVT_CHOICE(XRCID("VidModeComx"), GuiComx::onComxVideoMode)
	EVT_SPIN_UP(XRCID("ZoomSpinComx"), GuiMain::onZoomUp)
	EVT_SPIN_DOWN(XRCID("ZoomSpinComx"), GuiMain::onZoomDown)
	EVT_TEXT(XRCID("ZoomValueComx"), GuiMain::onZoomValue)
	EVT_BUTTON(XRCID("FullScreenF3Comx"), GuiMain::onFullScreen)
	EVT_COMMAND_SCROLL_THUMBTRACK(XRCID("VolumeComx"), GuiMain::onVolume)
	EVT_COMMAND_SCROLL_CHANGED(XRCID("VolumeComx"), GuiMain::onVolume)
	EVT_CHECKBOX(XRCID("InterlaceComx"), GuiMain::onInterlace)
	EVT_CHECKBOX(XRCID("ExpRamComx"), GuiComx::onComxExpansionRam)
	EVT_SPINCTRL(XRCID("ExpRamSlotComx"), GuiComx::onComxExpansionRamSlot)
	EVT_CHECKBOX(XRCID("UseLocationComx"), GuiMain::onUseLocation)

	EVT_TEXT(XRCID("SaveStartComx"), GuiMain::onSaveStart)
	EVT_TEXT(XRCID("SaveEndComx"), GuiMain::onSaveEnd)
	EVT_TEXT(XRCID("SaveExecComx"), GuiMain::onSaveExec)

	EVT_BUTTON(XRCID("SaveButtonComx"), GuiMain::onSaveButton)
	EVT_BUTTON(XRCID("LoadButtonComx"), GuiMain::onLoadButton)
	EVT_BUTTON(XRCID("RunButtonComx"), GuiMain::onLoadRunButton)
	EVT_BUTTON(XRCID("DsaveButtonComx"), GuiMain::onDataSaveButton)

	EVT_CHECKBOX(XRCID("TurboComx"), GuiMain::onTurbo)
	EVT_TEXT(XRCID("TurboClockComx"), GuiMain::onTurboClock)
	EVT_CHECKBOX(XRCID("AutoCasLoadComx"), GuiMain::onAutoLoad)
	EVT_BUTTON(XRCID("CasLoadComx"), GuiMain::onCassetteLoad)
	EVT_BUTTON(XRCID("CasSaveComx"), GuiMain::onCassetteSave)
	EVT_BUTTON(XRCID("CasStopComx"), GuiMain::onCassetteStop)
	EVT_BUTTON(XRCID("RealCasLoadComx"), GuiMain::onRealCas)
	EVT_TOGGLEBUTTON(XRCID("ComxWavFile"), GuiMain::onWavFile)
	EVT_BUTTON(XRCID("ColoursComx"), Main::onColoursDef)

	EVT_COMMAND(wxID_ANY, OPEN_COMX_PRINTER_WINDOW, GuiComx::openComxPrinterFrame)
	EVT_COMMAND(wxID_ANY, STATUS_LED_ON, GuiComx::statusLedOn)
	EVT_COMMAND(wxID_ANY, STATUS_LED_OFF, GuiComx::statusLedOff)
	EVT_COMMAND(wxID_ANY, STATUS_BAR_1870, GuiComx::v1870BarSize)
	EVT_COMMAND(wxID_ANY, EXP_LED_ON, GuiComx::expLedOn)
	EVT_COMMAND(wxID_ANY, EXP_LED_OFF, GuiComx::expLedOff)

	EVT_BUTTON(XRCID("EpromComx"), GuiComx::onEpromDialog)

	END_EVENT_TABLE()

GuiComx::GuiComx(const wxString& title, const wxPoint& pos, const wxSize& size, Mode mode, wxString dataDir)
: GuiElf(title, pos, size, mode, dataDir)
{
	conf[COMX].loadFileNameFull_ = "";
	conf[COMX].loadFileName_ = "";

	conf[COMX].pLoadSaveName_[0] = 'C';
	conf[COMX].pLoadSaveName_[1] = 'O';
	conf[COMX].pLoadSaveName_[2] = 'M';
	conf[COMX].pLoadSaveName_[3] = 'X';

	conf[COMX].defus_ = 0x4281;
	conf[COMX].eop_ = 0x4283;
	conf[COMX].string_ = 0x4292;
	conf[COMX].arrayValue_ = 0x4294;
	conf[COMX].eod_ = 0x4299;
	conf[COMX].basicRamAddress_ = 0x4400;

	conf[COMX].saveStartString_ = "";
	conf[COMX].saveEndString_ = "";
	conf[COMX].saveExecString_ = "";
}

GuiComx::~GuiComx()
{
}

void GuiComx::readComxConfig()
{
	selectedComputer_ = COMX;

	conf[COMX].romDir_[MAINROM1] = configPointer->Read("Dir/Comx/Main_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].romDir_[EXPROM] = configPointer->Read("Dir/Comx/Expansion_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].romDir_[CARTROM1] = configPointer->Read("Dir/Comx/Card_1_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].romDir_[CARTROM2] = configPointer->Read("Dir/Comx/Card_2_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].romDir_[CARTROM3] = configPointer->Read("Dir/Comx/Card_3_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].romDir_[CARTROM4] = configPointer->Read("Dir/Comx/Card_4_Rom_File", dataDir_ + "Comx" + pathSeparator_);
	Pl80Data_[0] = configPointer->Read("Dir/Comx/PL80_Rom_Files", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].ramDir_ = configPointer->Read("Dir/Comx/Software_File", dataDir_ + "Comx" + pathSeparator_);
	floppyDir_[0] = configPointer->Read("Dir/Comx/Floppy_1_File", dataDir_ + "Comx" + pathSeparator_ + "Disks" + pathSeparator_);
	floppyDir_[1] = configPointer->Read("Dir/Comx/Floppy_2_File", dataDir_ + "Comx" + pathSeparator_ + "Disks" + pathSeparator_);
	conf[COMX].keyFileDir_ = configPointer->Read("Dir/Comx/Key_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].printFileDir_ = configPointer->Read("Dir/Comx/Print_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].screenDumpFileDir_ = configPointer->Read("Dir/Comx/Video_Dump_File", dataDir_ + "Comx" + pathSeparator_);
	conf[COMX].wavFileDir_ = configPointer->Read("Dir/Comx/Wav_File", dataDir_ + "Comx" + pathSeparator_);

	conf[COMX].rom_[MAINROM1] = configPointer->Read("/Comx/Main_Rom_File", "comx35.1.1.bin");
	conf[COMX].rom_[EXPROM] = configPointer->Read("/Comx/Expansion_Rom_File", "f&m.expansion.3.2.bin");
	conf[COMX].rom_[CARTROM1] = configPointer->Read("/Comx/Card_1_Rom_File", "fdc.bin");
	conf[COMX].rom_[CARTROM2] = configPointer->Read("/Comx/Card_2_Rom_File", "f&m.eprom.board.1.1.bin");
	conf[COMX].rom_[CARTROM3] = configPointer->Read("/Comx/Card_3_Rom_File", "f&m.printer.1.2.bin");
	conf[COMX].rom_[CARTROM4] = configPointer->Read("/Comx/Card_4_Rom_File", "80column.1.1.bin");
	Pl80Data_[1] = configPointer->Read("/Comx/PL80_Rom_File", "pl80.bin");
	Pl80Data_[2] = configPointer->Read("/Comx/PL80_Extension_Rom_File", "pl80.it.em.ou.bin");
	floppy_[0] = configPointer->Read("/Comx/Floppy_1_File", "dos.1.4+f&m.disk.tools.img");
	floppy_[1] = configPointer->Read("/Comx/Floppy_2_File", "f&m-heijmans.sw.img");
	conf[COMX].keyFile_ = configPointer->Read("/Comx/Key_File", "");
	conf[COMX].printFile_ = configPointer->Read("/Comx/Print_File", "printerout.txt");
	conf[COMX].screenDumpFile_ = configPointer->Read("/Comx/Video_Dump_File", "screendump.png");
	conf[COMX].wavFile_ = configPointer->Read("/Comx/Wav_File", "");

	wxString defaultZoom;
	defaultZoom.Printf("%2.2f", 2.0);
	conf[COMX].zoom_ = configPointer->Read("/Comx/Zoom", defaultZoom);
	wxString defaultClock;
	defaultClock.Printf("%1.3f", 2.813);
	comxPalClock_ = configPointer->Read("/Comx/Clock_Speed_When_Using_Pal", defaultClock);
	defaultClock.Printf("%1.3f", 2.835);
	comxNtscClock_ = configPointer->Read("/Comx/Clock_Speed_When_Using_Ntsc", defaultClock);
	conf[COMX].videoMode_ = configPointer->Read("/Comx/Video_Mode", 0l);
	if (conf[COMX].videoMode_ == PAL)
		conf[COMX].clock_ = comxPalClock_;
	else
		conf[COMX].clock_ = comxNtscClock_;

	conf[COMX].turboClock_ = configPointer->Read("/Comx/Turbo_Clock_Speed", "15");
	conf[COMX].volume_ = configPointer->Read("/Comx/Volume", 25l);
	expansionRamSlot_ = configPointer->Read("/Comx/Ram_Card_Slot", 4l);
	comxPrintMode_ = configPointer->Read("/Comx/Print_Mode", 1l);
	configPointer->Read("/Comx/Use_Ram_Card", &useExpansionRam_, false);
	configPointer->Read("/Comx/Expansion_Rom_Loaded", &expansionRomLoaded_, true);
	configPointer->Read("/Comx/Enable_Turbo_Cassette", &conf[COMX].turbo_, true);
	configPointer->Read("/Comx/Enable_Auto_Cassette", &conf[COMX].autoCassetteLoad_, true);
	configPointer->Read("/Comx/Enable_Real_Cassette", &conf[COMX].realCassetteLoad_, false);
	configPointer->Read("/Comx/Enable_80_Column_Interlace", &conf[COMX].interlace_, true);
	configPointer->Read("/Comx/Disk_Rom_Loaded", &diskRomLoaded_, true);
	conf[COMX].useLoadLocation_ = false;
	conf[COMX].mainX_ = configPointer->Read("/Comx/Window_Position_X", mainWindowX_+windowInfo.mainwX);
	conf[COMX].mainY_ = configPointer->Read("/Comx/Window_Position_Y", mainWindowY_);
	conf[COMX].mc6845X_ = configPointer->Read("/Comx/Window_Position_MC6845_X", mainWindowX_);
	conf[COMX].mc6845Y_ = configPointer->Read("/Comx/Window_Position_MC6845_Y", mainWindowY_+windowInfo.mainwY);
	conf[COMX].sizeX_ = configPointer->Read("/Comx/Window_Size_X", 480);
	conf[COMX].sizeY_ = configPointer->Read("/Comx/Window_Size_Y", 432);
	conf[COMX].sizeNTSCX_ = configPointer->Read("/Comx/Window_Size_Ntsc_X", 480);
	conf[COMX].sizeNTSCY_ = configPointer->Read("/Comx/Window_Size_Ntsc_Y", 384);
	conf[COMX].sizeX6845_ = configPointer->Read("/Comx/Window_Size_6845_X", 640);
	conf[COMX].sizeY6845_ = configPointer->Read("/Comx/Window_Size_6845_Y", 432);

	setRealCas(COMX);

	if (mode_.gui)
	{
		XRCCTRL(*this, "TurboClockComx", wxTextCtrl)->SetValue(conf[COMX].turboClock_);
		XRCCTRL(*this, "VolumeComx", wxSlider)->SetValue(conf[COMX].volume_);

		XRCCTRL(*this, "MainRomComx", wxComboBox)->SetValue(conf[COMX].rom_[MAINROM1]);
		XRCCTRL(*this, "ExpRomComx", wxComboBox)->SetValue(conf[COMX].rom_[EXPROM]);
		XRCCTRL(*this, "Cart1RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM1]);
		XRCCTRL(*this, "Cart2RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM2]);
		XRCCTRL(*this, "Cart3RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM3]);
		XRCCTRL(*this, "Cart4RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM4]);
		XRCCTRL(*this, "Disk1FileComx", wxTextCtrl)->SetValue(floppy_[0]);
		XRCCTRL(*this, "Disk2FileComx", wxTextCtrl)->SetValue(floppy_[1]);
		XRCCTRL(*this, "KeyFileComx", wxTextCtrl)->SetValue(conf[COMX].keyFile_);
		XRCCTRL(*this, "PrintFileComx", wxTextCtrl)->SetValue(conf[COMX].printFile_);
		XRCCTRL(*this, "ScreenDumpFileComx", wxComboBox)->SetValue(conf[COMX].screenDumpFile_);

		XRCCTRL(*this, "TurboComx", wxCheckBox)->SetValue(conf[COMX].turbo_);
		turboGui("Comx");
		XRCCTRL(*this, "AutoCasLoadComx", wxCheckBox)->SetValue(conf[COMX].autoCassetteLoad_);

		XRCCTRL(*this, "InterlaceComx", wxCheckBox)->SetValue(conf[COMX].interlace_);

		XRCCTRL(*this, "ExpRamComx", wxCheckBox)->SetValue(useExpansionRam_);
		XRCCTRL(*this, "ExpRamSlotComx", wxSpinCtrl)->SetValue(expansionRamSlot_);

		XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(expansionRomLoaded_ );
		XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(expansionRomLoaded_ );
		XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(expansionRomLoaded_ );
		XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(expansionRomLoaded_ );
		XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(expansionRomLoaded_ );
		XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(expansionRomLoaded_ );

		XRCCTRL(*this, "ExpRamSlotComx", wxSpinCtrl)->Enable(useExpansionRam_);
		if (useExpansionRam_)
			setComxExpRamSlot();
		XRCCTRL(*this, "Disk1ButtonComx", wxButton)->Enable(diskRomLoaded_);
		XRCCTRL(*this, "Disk1FileComx", wxTextCtrl)->Enable(diskRomLoaded_);
		XRCCTRL(*this, "EjectDisk1Comx", wxButton)->Enable(diskRomLoaded_ );
		XRCCTRL(*this, "Disk2ButtonComx", wxButton)->Enable(diskRomLoaded_);
		XRCCTRL(*this, "Disk2FileComx", wxTextCtrl)->Enable(diskRomLoaded_);
		XRCCTRL(*this, "EjectDisk2Comx", wxButton)->Enable(diskRomLoaded_ );

		XRCCTRL(*this, "WavFileComx", wxTextCtrl)->SetValue(conf[COMX].wavFile_);
		XRCCTRL(*this, "VidModeComx", wxChoice)->SetSelection(conf[COMX].videoMode_);
		if (conf[COMX].videoMode_ == PAL)
			clockTextCtrl[COMX]->ChangeValue(comxPalClock_);
		else
			clockTextCtrl[COMX]->ChangeValue(comxNtscClock_);

		XRCCTRL(*this, "ZoomValueComx", wxTextCtrl)->ChangeValue(conf[COMX].zoom_);
		XRCCTRL(*this, "PrintModeComx", wxChoice)->SetSelection(comxPrintMode_);
		setComxPrintMode();

		XRCCTRL(*this, "UseLocationComx", wxCheckBox)->SetValue(conf[COMX].useLoadLocation_);
	}
}

void GuiComx::writeComxConfig()
{
	if (useExpansionRam_)
		resetCardText();

	configPointer->Write("Dir/Comx/Main_Rom_File", conf[COMX].romDir_[MAINROM1]);
	configPointer->Write("Dir/Comx/Expansion_Rom_File", conf[COMX].romDir_[EXPROM]);
	configPointer->Write("Dir/Comx/Card_1_Rom_File", conf[COMX].romDir_[CARTROM1]);
	configPointer->Write("Dir/Comx/Card_2_Rom_File", conf[COMX].romDir_[CARTROM2]);
	configPointer->Write("Dir/Comx/Card_3_Rom_File", conf[COMX].romDir_[CARTROM3]);
	configPointer->Write("Dir/Comx/Card_4_Rom_File", conf[COMX].romDir_[CARTROM4]);
	configPointer->Write("Dir/Comx/PL80_Rom_Files", Pl80Data_[0]);
	configPointer->Write("Dir/Comx/Software_File", conf[COMX].ramDir_);
	configPointer->Write("Dir/Comx/Floppy_1_File", floppyDir_[0]);
	configPointer->Write("Dir/Comx/Floppy_2_File", floppyDir_[1]);
	configPointer->Write("Dir/Comx/Key_File", conf[COMX].keyFileDir_);
	configPointer->Write("Dir/Comx/Print_File", conf[COMX].printFileDir_);
	configPointer->Write("Dir/Comx/Video_Dump_File", conf[COMX].screenDumpFileDir_);
	configPointer->Write("Dir/Comx/Wav_File", conf[COMX].wavFileDir_);

	configPointer->Write("/Comx/Main_Rom_File", conf[COMX].rom_[MAINROM1]);
	configPointer->Write("/Comx/Expansion_Rom_File", conf[COMX].rom_[EXPROM]);
	configPointer->Write("/Comx/Card_1_Rom_File", conf[COMX].rom_[CARTROM1]);
	configPointer->Write("/Comx/Card_2_Rom_File", conf[COMX].rom_[CARTROM2]);
	configPointer->Write("/Comx/Card_3_Rom_File", conf[COMX].rom_[CARTROM3]);
	configPointer->Write("/Comx/Card_4_Rom_File", conf[COMX].rom_[CARTROM4]);
	configPointer->Write("/Comx/PL80_Rom_File", Pl80Data_[1]);
	configPointer->Write("/Comx/PL80_Extension_Rom_File", Pl80Data_[2]);
	configPointer->Write("/Comx/Floppy_1_File", floppy_[0]);
	configPointer->Write("/Comx/Floppy_2_File", floppy_[1]);
	configPointer->Write("/Comx/Key_File", conf[COMX].keyFile_);
	configPointer->Write("/Comx/Print_File", conf[COMX].printFile_);
	configPointer->Write("/Comx/Video_Dump_File", conf[COMX].screenDumpFile_);
	configPointer->Write("/Comx/Wav_File", conf[COMX].wavFile_);

	configPointer->Write("/Comx/Clock_Speed_When_Using_Pal", comxPalClock_);
	configPointer->Write("/Comx/Clock_Speed_When_Using_Ntsc", comxNtscClock_);
	configPointer->Write("/Comx/Turbo_Clock_Speed", conf[COMX].turboClock_);
	configPointer->Write("/Comx/Volume", conf[COMX].volume_);
	configPointer->Write("/Comx/Enable_Turbo_Cassette", conf[COMX].turbo_);
	configPointer->Write("/Comx/Video_Mode", conf[COMX].videoMode_);
	configPointer->Write("/Comx/Zoom", conf[COMX].zoom_);
	configPointer->Write("/Comx/Enable_Auto_Cassette", conf[COMX].autoCassetteLoad_);
	configPointer->Write("/Comx/Enable_Real_Cassette", conf[COMX].realCassetteLoad_);
	configPointer->Write("/Comx/Enable_80_Column_Interlace", conf[COMX].interlace_);
	configPointer->Write("/Comx/Use_Ram_Card", useExpansionRam_);
	configPointer->Write("/Comx/Ram_Card_Slot", expansionRamSlot_);
	configPointer->Write("/Comx/Expansion_Rom_Loaded", expansionRomLoaded_);
	configPointer->Write("/Comx/Disk_Rom_Loaded", diskRomLoaded_);

	configPointer->Write("/Comx/Print_Mode", comxPrintMode_);

	if (conf[COMX].mainX_ > 0)
		configPointer->Write("/Comx/Window_Position_X", conf[COMX].mainX_);
	if (conf[COMX].mainY_ > 0)
		configPointer->Write("/Comx/Window_Position_Y", conf[COMX].mainY_);
	if (conf[COMX].mc6845X_ > 0)
		configPointer->Write("/Comx/Window_Position_MC6845_X", conf[COMX].mc6845X_);
	if (conf[COMX].mc6845Y_ > 0)
		configPointer->Write("/Comx/Window_Position_MC6845_Y", conf[COMX].mc6845Y_);
	configPointer->Write("/Comx/Window_Size_X", conf[COMX].sizeX_);
	configPointer->Write("/Comx/Window_Size_Y", conf[COMX].sizeY_);
	configPointer->Write("/Comx/Window_Size_Ntsc_X", conf[COMX].sizeNTSCX_);
	configPointer->Write("/Comx/Window_Size_Ntsc_Y", conf[COMX].sizeNTSCY_);
	configPointer->Write("/Comx/Window_Size_6845_X", conf[COMX].sizeX6845_);
	configPointer->Write("/Comx/Window_Size_6845_Y", conf[COMX].sizeY6845_);
}

void GuiComx::onExpansionRom(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Expansion ROM file to load",
                               conf[COMX].romDir_[EXPROM], XRCCTRL(*this, "ExpRomComx", wxComboBox)->GetValue(),
                               "",
                               wxString::Format
                              (
                                    "Binary File|*.bin;*.rom;*.ram;*.cos|Intel Hex File|*.hex|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
	{
		expansionRomLoaded_ = false;
		XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(false);
		XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(false);
		XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(false);
		XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(false);
		XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(false);
		XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(false);
		return;
	}

	expansionRomLoaded_ = true;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	conf[COMX].romDir_[EXPROM] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	conf[COMX].rom_[EXPROM] = FullPath.GetFullName();

	XRCCTRL(*this, "ExpRomComx", wxComboBox)->SetValue(conf[COMX].rom_[EXPROM]);
	if ((expansionRamSlot_ != 2) || !useExpansionRam_)
	{
		XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(true);
		XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(true);
	}
	if ((expansionRamSlot_ != 3) || !useExpansionRam_)
	{
		XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(true);
		XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(true);
	}
	if ((expansionRamSlot_ != 4) || !useExpansionRam_)
	{
		XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(true);
		XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(true);
	}
}

void GuiComx::onExpansionRomText(wxCommandEvent& event)
{
	conf[selectedComputer_].rom_[EXPROM] = event.GetString();
}

void GuiComx::onCard1Rom(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Cartridge 1 ROM file to load",
                               conf[COMX].romDir_[CARTROM1], XRCCTRL(*this, "Cart1RomComx", wxComboBox)->GetValue(),
                               "",
                               wxString::Format
                              (
                                    "Binary File|*.bin;*.rom;*.ram;*.cos|Intel Hex File|*.hex|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	conf[COMX].romDir_[CARTROM1] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	conf[COMX].rom_[CARTROM1] = FullPath.GetFullName();

	XRCCTRL(*this, "Cart1RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM1]);
}

void GuiComx::onCard1RomText(wxCommandEvent& event)
{
	conf[selectedComputer_].rom_[CARTROM1] = event.GetString();
}

void GuiComx::onCard2Rom(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Cartridge 2 ROM file to load",
                               conf[COMX].romDir_[CARTROM2], XRCCTRL(*this, "Cart2RomComx", wxComboBox)->GetValue(),
                               "",
                               wxString::Format
                              (
                                    "Binary File|*.bin;*.rom;*.ram;*.cos|Intel Hex File|*.hex|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	conf[COMX].romDir_[CARTROM2] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	conf[COMX].rom_[CARTROM2] = FullPath.GetFullName();

	XRCCTRL(*this, "Cart2RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM2]);
}

void GuiComx::onCard2RomText(wxCommandEvent& event)
{
	conf[selectedComputer_].rom_[CARTROM2] = event.GetString();
}

void GuiComx::onCard3Rom(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Cartridge 3 ROM file to load",
                               conf[COMX].romDir_[CARTROM3], XRCCTRL(*this, "Cart3RomComx", wxComboBox)->GetValue(),
                               "",
                               wxString::Format
                              (
                                    "Binary File|*.bin;*.rom;*.ram;*.cos|Intel Hex File|*.hex|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	conf[COMX].romDir_[CARTROM3] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	conf[COMX].rom_[CARTROM3] = FullPath.GetFullName();

	XRCCTRL(*this, "Cart3RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM3]);
}

void GuiComx::onCard3RomText(wxCommandEvent& event)
{
	conf[selectedComputer_].rom_[CARTROM3] = event.GetString();
}

void GuiComx::onCard4Rom(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Cartridge 4 ROM file to load",
                               conf[COMX].romDir_[CARTROM4], XRCCTRL(*this, "Cart4RomComx", wxComboBox)->GetValue(),
                               "",
                               wxString::Format
                              (
                                    "Binary File|*.bin;*.rom;*.ram;*.cos|Intel Hex File|*.hex|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	conf[COMX].romDir_[CARTROM4] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	conf[COMX].rom_[CARTROM4] = FullPath.GetFullName();

	XRCCTRL(*this, "Cart4RomComx", wxComboBox)->SetValue(conf[COMX].rom_[CARTROM4]);
}

void GuiComx::onCard4RomText(wxCommandEvent& event)
{
	conf[selectedComputer_].rom_[CARTROM4] = event.GetString();
}

void GuiComx::onComxDisk1(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Disk 1 file to load",
                               floppyDir_[0], floppy_[0],
                               "img",
                               wxString::Format
                              (
                                    "Disk Image (*.img)|*.img|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	floppyDir_[0] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	floppy_[0] = FullPath.GetFullName();

	if (mode_.gui)
		XRCCTRL(*this, "Disk1FileComx", wxTextCtrl)->SetValue(floppy_[0]);
}

void GuiComx::onComxDiskText1(wxCommandEvent&event)
{
	floppy_[0] = event.GetString();
	if ((runningComputer_ == COMX) && diskRomLoaded_)
	{
		if (floppy_[0].Len() == 0)
			p_Comx->setDiskName(1, "");
		else
			p_Comx->setDiskName(1, floppyDir_[0] + floppy_[0]);
	}
}

void GuiComx::onComxDiskEject1(wxCommandEvent& WXUNUSED(event) )
{
	floppy_[0] = "";
	if (mode_.gui)
		XRCCTRL(*this, "Disk1FileComx", wxTextCtrl)->SetValue(floppy_[0]);
}

void GuiComx::onComxDisk2(wxCommandEvent& WXUNUSED(event) )
{
	wxString fileName;

	fileName = wxFileSelector( "Select the Disk 2 file to load",
                               floppyDir_[1], floppy_[1],
                               "img",
                               wxString::Format
                              (
                                    "Disk Image (*.img)|*.img|All files (%s)|%s",
                                    wxFileSelectorDefaultWildcardStr,
                                    wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_OPEN|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );
	if (!fileName)
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	floppyDir_[1] = FullPath.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR, wxPATH_NATIVE);
	floppy_[1] = FullPath.GetFullName();

	if (mode_.gui)
		XRCCTRL(*this, "Disk2FileComx", wxTextCtrl)->SetValue(floppy_[1]);
}

void GuiComx::onComxDiskText2(wxCommandEvent&event)
{
	floppy_[1] = event.GetString();
	if ((runningComputer_ == COMX) && diskRomLoaded_)
	{
		if (floppy_[1].Len() == 0)
			p_Comx->setDiskName(2, "");
		else
			p_Comx->setDiskName(2, floppyDir_[1] + floppy_[1]);
	}
}

void GuiComx::onComxDiskEject2(wxCommandEvent& WXUNUSED(event) )
{
	floppy_[1] = "";
	if (mode_.gui)
		XRCCTRL(*this, "Disk2FileComx", wxTextCtrl)->SetValue(floppy_[1]);
}

void GuiComx::onComxPrintFileText(wxCommandEvent&WXUNUSED(event))
{
	conf[COMX].printFile_ = XRCCTRL(*this, "PrintFileComx", wxTextCtrl)->GetValue();

	if (runningComputer_ == COMX)
	{
		if (p_Comx->isPrintRomLoaded(COMXPARALLEL))
		{
			if (conf[COMX].printFile_.Len() == 0)
				p_PrinterParallel->setPrintfileName("");
			else
				p_PrinterParallel->setPrintfileName(conf[COMX].printFileDir_ + conf[COMX].printFile_);
		}
		if (p_Comx->isPrintRomLoaded(COMXSERIAL))
		{
			if (conf[COMX].printFile_.Len() == 0)
				p_PrinterSerial->setPrintfileName("");
			else
				p_PrinterSerial->setPrintfileName(conf[COMX].printFileDir_ + conf[COMX].printFile_);
		}
		if (p_Comx->isPrintRomLoaded(COMXTHERMAL))
		{
			if (conf[COMX].printFile_.Len() == 0)
				p_PrinterThermal->setPrintfileName("");
			else
				p_PrinterThermal->setPrintfileName(conf[COMX].printFileDir_ + conf[COMX].printFile_);
		}
	}
}

void GuiComx::onComxPrintMode(wxCommandEvent&event)
{
	comxPrintMode_ = event.GetSelection();
	setComxPrintMode();

	if (runningComputer_ == COMX)
	{
		if (p_Comx->isPrintRomLoaded(COMXPARALLEL))
		{
			p_PrinterParallel->setComxPrintMode(comxPrintMode_);
		}
		if (p_Comx->isPrintRomLoaded(COMXSERIAL))
		{
			p_PrinterSerial->setComxPrintMode(comxPrintMode_);
		}
	}
}

void GuiComx::onComxPrintButton(wxCommandEvent&WXUNUSED(event))
{
	onComxF4();
}

void GuiComx::onComxVideoMode(wxCommandEvent&event)
{
	conf[COMX].videoMode_ = event.GetSelection();
	switch(conf[COMX].videoMode_)
	{
		case PAL:
			clockTextCtrl[COMX]->ChangeValue(comxPalClock_);
		break;

		case NTSC:
			clockTextCtrl[COMX]->ChangeValue(comxNtscClock_);
		break;
	}
}

void GuiComx::onComxExpansionRam(wxCommandEvent&event)
{
	useExpansionRam_ = event.IsChecked();
	XRCCTRL(*this, "ExpRamSlotComx", wxSpinCtrl)->Enable(useExpansionRam_);
	if (useExpansionRam_)
		setComxExpRamSlot();
	else
		resetCardText();
}

void GuiComx::onComxExpansionRamSlot(wxSpinEvent&event)
{
	resetCardText();

	expansionRamSlot_ = event.GetPosition();

	setComxExpRamSlot();
}

void GuiComx::statusLedOnEvent()
{
	if (wxIsMainThread())
	{
		statusLedOnDirect();
		return;
	}
	wxCommandEvent event(STATUS_LED_ON, 801);
	event.SetEventObject(this);
	wxPostEvent(this, event);
	while(!isComxStatusLedOn_)
		p_Computer->sleepComputer(1);
}

void GuiComx::statusLedOffEvent()
{
	if (wxIsMainThread())
	{
		statusLedOffDirect();
		return;
	}
	wxCommandEvent event(STATUS_LED_OFF, 802);
	event.SetEventObject(this);
	wxPostEvent(this, event);
	while(isComxStatusLedOn_)
		p_Computer->sleepComputer(1);
}

void GuiComx::expLedOnEvent()
{
	if (wxIsMainThread())
	{
		expLedOnDirect();
		return;
	}
	if (isComxExpLedOn_)  return;
	wxCommandEvent event(EXP_LED_ON, 805);
	event.SetEventObject(this);
	wxPostEvent(this, event);
	while(!isComxExpLedOn_)
		p_Computer->sleepComputer(1);
}

void GuiComx::expLedOffEvent()
{
	if (wxIsMainThread())
	{
		expLedOffDirect();
		return;
	}
	if (!isComxExpLedOn_)  return;
	wxCommandEvent event(EXP_LED_OFF, 806);
	event.SetEventObject(this);
	wxPostEvent(this, event);
	while(isComxExpLedOn_)
		p_Computer->sleepComputer(1);
}

void GuiComx::v1870BarSizeEvent()
{
	wxCommandEvent event(STATUS_BAR_1870, 803);
	event.SetEventObject(this);
	wxPostEvent(this, event);
}

wxString GuiComx::getFloppyDir(int drive)
{
	return floppyDir_[drive];
}

wxString GuiComx::getFloppyFile(int drive)
{
	return floppy_[drive];
}

wxString GuiComx::getPL80Data(int item)
{
	return Pl80Data_[item];
}

void GuiComx::setPL80Data(int item, wxString data)
{
	Pl80Data_[item] = data;
}

void GuiComx::enableComxGui(bool status)
{
	if (useExpansionRam_)
	{
		XRCCTRL(*this, "ExpRamSlotComx", wxSpinCtrl)->Enable(status);
		switch(expansionRamSlot_)
		{
			case 1:
				XRCCTRL(*this, "Cart1RomComx", wxComboBox)->Enable(false);
				XRCCTRL(*this, "Cart1RomButtonComx", wxButton)->Enable(false);
			break;

			case 2:
				XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(false);
				XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(false);
			break;

			case 3:
				XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(false);
				XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(false);
			break;

			case 4:
				XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(false);
				XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(false);
			break;
		}
	}
}

void GuiComx::statusLedOn(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateStatusLed(true);
	isComxStatusLedOn_ = true;
}

void GuiComx::statusLedOnDirect()
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateStatusLed(true);
	isComxStatusLedOn_ = true;
}

void GuiComx::statusLedOff(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateStatusLed(false);
	isComxStatusLedOn_ = false;
}

void GuiComx::statusLedOffDirect()
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateStatusLed(false);
	isComxStatusLedOn_ = false;
}

void GuiComx::expLedOn(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateExpansionLed(true);
	isComxExpLedOn_ = true;
}

void GuiComx::expLedOnDirect()
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateExpansionLed(true);
	isComxExpLedOn_ = true;
}

void GuiComx::expLedOff(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateExpansionLed(false);
	isComxExpLedOn_ = false;
}

void GuiComx::expLedOffDirect()
{
	if (runningComputer_ != COMX)  return;
	p_Comx->updateExpansionLed(false);
	isComxExpLedOn_ = false;
}

void GuiComx::v1870BarSize(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ == COMX)
		p_Comx->v1870BarSize();
	if (runningComputer_ == CIDELSA)
		p_Cidelsa->v1870BarSize();
}

void GuiComx::setComxExpRamSlot()
{
	if (!mode_.gui)  return;
	switch(expansionRamSlot_)
	{
		case 1:
			saveCardText_ = XRCCTRL(*this, "Cart1RomComx", wxComboBox)->GetValue();
			XRCCTRL(*this, "Cart1RomComx", wxComboBox)->SetValue("Ram Card");
			XRCCTRL(*this, "Cart1RomComx", wxComboBox)->Enable(false);
			XRCCTRL(*this, "Cart1RomButtonComx", wxButton)->Enable(false);
		break;

		case 2:
			saveCardText_ = XRCCTRL(*this, "Cart2RomComx", wxComboBox)->GetValue();
			XRCCTRL(*this, "Cart2RomComx", wxComboBox)->SetValue("Ram Card");
			XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(false);
			XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(false);
		break;

		case 3:
			saveCardText_ = XRCCTRL(*this, "Cart3RomComx", wxComboBox)->GetValue();
			XRCCTRL(*this, "Cart3RomComx", wxComboBox)->SetValue("Ram Card");
			XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(false);
			XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(false);
		break;

		case 4:
			saveCardText_ = XRCCTRL(*this, "Cart4RomComx", wxComboBox)->GetValue();
			XRCCTRL(*this, "Cart4RomComx", wxComboBox)->SetValue("Ram Card");
			XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(false);
			XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(false);
		break;
	}
}

void GuiComx::resetCardText()
{
	if (!mode_.gui)  return;
	switch(expansionRamSlot_)
	{
		case 1:
			XRCCTRL(*this, "Cart1RomComx", wxComboBox)->SetValue(saveCardText_);
			XRCCTRL(*this, "Cart1RomComx", wxComboBox)->Enable(true);
			XRCCTRL(*this, "Cart1RomButtonComx", wxButton)->Enable(true);
		break;

		case 2:
			XRCCTRL(*this, "Cart2RomComx", wxComboBox)->SetValue(saveCardText_);
			if (expansionRomLoaded_)
			{
				XRCCTRL(*this, "Cart2RomComx", wxComboBox)->Enable(true);
				XRCCTRL(*this, "Cart2RomButtonComx", wxButton)->Enable(true);
			}
		break;

		case 3:
			XRCCTRL(*this, "Cart3RomComx", wxComboBox)->SetValue(saveCardText_);
			if (expansionRomLoaded_)
			{
				XRCCTRL(*this, "Cart3RomComx", wxComboBox)->Enable(true);
				XRCCTRL(*this, "Cart3RomButtonComx", wxButton)->Enable(true);
			}
		break;

		case 4:
			XRCCTRL(*this, "Cart4RomComx", wxComboBox)->SetValue(saveCardText_);
			if (expansionRomLoaded_)
			{
				XRCCTRL(*this, "Cart4RomComx", wxComboBox)->Enable(true);
				XRCCTRL(*this, "Cart4RomButtonComx", wxButton)->Enable(true);
			}
		break;
	}
}

bool GuiComx::isSaving()
{
	if (runningComputer_ == COMX)
		return p_Comx->isSaving();
	else
		return false;
}

void GuiComx::setComxPrintMode()
{
	if (!mode_.gui)  return;
	if (comxPrintMode_ == PRINTFILE)
	{
		XRCCTRL(*this, "PrintFileButtonComx", wxButton)->Enable(true);
		XRCCTRL(*this, "PrintFileComx", wxTextCtrl)->Enable(true);
	}
	else
	{
		XRCCTRL(*this, "PrintFileButtonComx", wxButton)->Enable(false);
		XRCCTRL(*this, "PrintFileComx", wxTextCtrl)->Enable(false);
	}
}

void GuiComx::openComxPrinterFrame(wxCommandEvent&WXUNUSED(event))
{
	if (runningComputer_ != COMX)  return;
	onComxF4();
}

void GuiComx::onComxF4()
{
	if (runningComputer_ == COMX)
	{
		p_Comx->onComxF4();
		if (comxPrintMode_ == PRINTFILE)
		{
			comxPrintMode_ = COMXPRINTPLOTTER;
			if (mode_.gui)  
				XRCCTRL(*this, "PrintModeComx", wxChoice)->SetSelection(comxPrintMode_);
			setComxPrintMode();
		}
	}
}

void GuiComx::setFandMBasicGui()
{
	if (!mode_.gui)  return;
	if (p_Comx->isFAndMBasicRunning())
	{
		XRCCTRL(*this, "RunButtonComx", wxButton)->SetLabel("F&&M RUN");
		XRCCTRL(*this, "SaveButtonComx", wxButton)->SetLabel("F&&M SAVE");
		XRCCTRL(*this, "LoadButtonComx", wxButton)->SetLabel("F&&M LOAD");
	}
	else
	{
		XRCCTRL(*this, "RunButtonComx", wxButton)->SetLabel("RUN");
		XRCCTRL(*this, "SaveButtonComx", wxButton)->SetLabel("SAVE");
		XRCCTRL(*this, "LoadButtonComx", wxButton)->SetLabel("LOAD");
	}
}

void GuiComx::enableDiskRomGui(bool DiskRom)
{
	diskRomLoaded_ = DiskRom;
	if (!mode_.gui)  return;
	XRCCTRL(*this, "Disk1FileComx", wxTextCtrl)->Enable(diskRomLoaded_);
	XRCCTRL(*this, "Disk1ButtonComx", wxButton)->Enable(diskRomLoaded_);
	XRCCTRL(*this, "EjectDisk1Comx", wxButton)->Enable(diskRomLoaded_);
	XRCCTRL(*this, "Disk2FileComx", wxTextCtrl)->Enable(diskRomLoaded_);
	XRCCTRL(*this, "Disk2ButtonComx", wxButton)->Enable(diskRomLoaded_);
	XRCCTRL(*this, "EjectDisk2Comx", wxButton)->Enable(diskRomLoaded_);
}

void GuiComx::setExtRomStatus(bool expansionRomLoaded)
{
	expansionRomLoaded_ = expansionRomLoaded;
}

void GuiComx::setComxPrintMode(int mode)
{
	comxPrintMode_ = mode;
}

int GuiComx::getComxPrintMode()
{
	return comxPrintMode_;
}

void GuiComx::onEpromDialog(wxCommandEvent&WXUNUSED(event))
{
	EpromDialog EpromDialog(this);
 	EpromDialog.ShowModal();
}

