/*
 *******************************************************************
 *** This software is copyright 2008 by Marcel van Tongeren      ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 *******************************************************************
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#ifdef __WXMSW__
#include "wx/msw/private.h"
#endif

#if !wxUSE_COMBOCTRL
    #error "Please set wxUSE_COMBOCTRL to 1 and rebuild the library."
#endif

#include "wx/xrc/xmlres.h"
#include "wx/spinctrl.h"

#include "main.h"
#include "guipecom.h"
#include "psave.h"

BEGIN_EVENT_TABLE(GuiPecom, GuiMain)

	EVT_TEXT(XRCID("MainRomPecom"), GuiMain::onMainRom1Text)
	EVT_COMBOBOX(XRCID("MainRomPecom"), GuiMain::onMainRom1Text)
	EVT_BUTTON(XRCID("RomButtonPecom"), GuiMain::onMainRom1)

	EVT_BUTTON(XRCID("ScreenDumpFileButtonPecom"), GuiMain::onScreenDumpFile)
	EVT_TEXT(XRCID("ScreenDumpFilePecom"), GuiMain::onScreenDumpFileText)
	EVT_COMBOBOX(XRCID("ScreenDumpFilePecom"), GuiMain::onScreenDumpFileText)
	EVT_BUTTON(XRCID("ScreenDumpF5Pecom"), GuiMain::onScreenDump)

	EVT_SPIN_UP(XRCID("ZoomSpinPecom"), GuiMain::onZoomUp)
	EVT_SPIN_DOWN(XRCID("ZoomSpinPecom"), GuiMain::onZoomDown)
	EVT_TEXT(XRCID("ZoomValuePecom"), GuiMain::onZoomValue)
	EVT_BUTTON(XRCID("FullScreenF3Pecom"), GuiMain::onFullScreen)
	EVT_COMMAND_SCROLL_THUMBTRACK(XRCID("VolumePecom"), GuiMain::onVolume) 
	EVT_COMMAND_SCROLL_CHANGED(XRCID("VolumePecom"), GuiMain::onVolume) 
	EVT_BUTTON(XRCID("CasButtonPecom"), GuiMain::onCassette)
	EVT_BUTTON(XRCID("EjectCasPecom"), GuiMain::onCassetteEject)
	EVT_TEXT(XRCID("WavFilePecom"), GuiMain::onCassetteText)
	EVT_BUTTON(XRCID("RealCasLoadPecom"), GuiMain::onRealCas)
	EVT_BUTTON(XRCID("CasLoadPecom"), GuiMain::onCassetteLoad)
	EVT_BUTTON(XRCID("CasSavePecom"), GuiMain::onCassetteSave)
	EVT_BUTTON(XRCID("CasStopPecom"), GuiMain::onCassetteStop)
	EVT_CHECKBOX(XRCID("TurboPecom"), GuiMain::onTurbo)
	EVT_TEXT(XRCID("TurboClockPecom"), GuiMain::onTurboClock)
	EVT_CHECKBOX(XRCID("AutoCasLoadPecom"), GuiMain::onAutoLoad)
	EVT_BUTTON(XRCID("PrintFileButtonPecom"), GuiMain::onPrintFile)
	EVT_TEXT(XRCID("PrintFilePecom"), GuiMain::onPrintFileText)
	EVT_BUTTON(XRCID("PrintButtonPecom"), GuiMain::onPrintButton)
	EVT_CHOICE(XRCID("PrintModePecom"), GuiMain::onPrintMode)
	EVT_BUTTON(XRCID("KeyFileButtonPecom"), GuiMain::onKeyFile)
	EVT_TEXT(XRCID("KeyFilePecom"), GuiMain::onKeyFileText)
	EVT_BUTTON(XRCID("EjectKeyFilePecom"), GuiMain::onKeyFileEject)
	EVT_CHECKBOX(XRCID("UseLocationPecom"), GuiMain::onUseLocation)

	EVT_TEXT(XRCID("SaveStartPecom"), GuiMain::onSaveStart)
	EVT_TEXT(XRCID("SaveEndPecom"), GuiMain::onSaveEnd)
	EVT_TEXT(XRCID("SaveExecPecom"), GuiMain::onSaveExec)

	EVT_BUTTON(XRCID("SaveButtonPecom"), GuiMain::onSaveButton) 
	EVT_BUTTON(XRCID("LoadButtonPecom"), GuiMain::onLoadButton)
	EVT_BUTTON(XRCID("RunButtonPecom"), GuiMain::onLoadRunButton)
	EVT_BUTTON(XRCID("DsaveButtonPecom"), GuiMain::onDataSaveButton)
	EVT_COMMAND(wxID_ANY, OPEN_PRINTER_WINDOW, GuiMain::openPrinterFrame) 
	EVT_BUTTON(XRCID("ColoursPecom"), Main::onColoursDef)

END_EVENT_TABLE()

GuiPecom::GuiPecom(const wxString& title, const wxPoint& pos, const wxSize& size, Mode mode, wxString dataDir)
: GuiMain(title, pos, size, mode, dataDir)
{
	conf[PECOM].loadFileNameFull_ = "";
	conf[PECOM].loadFileName_ = "";

	conf[PECOM].pLoadSaveName_[0] = 'P';
	conf[PECOM].pLoadSaveName_[1] = '-';
	conf[PECOM].pLoadSaveName_[2] = '6';
	conf[PECOM].pLoadSaveName_[3] = '4';

	conf[PECOM].defus_ = 0x81;
	conf[PECOM].eop_ = 0x83;
	conf[PECOM].string_ = 0x92;
	conf[PECOM].arrayValue_ = 0x94;
	conf[PECOM].eod_ = 0x99;
	conf[PECOM].basicRamAddress_ = 0x200;

	conf[PECOM].saveStartString_ = "";
	conf[PECOM].saveEndString_ = "";
	conf[PECOM].saveExecString_ = "";
}

void GuiPecom::readPecomConfig()
{
	selectedComputer_ = PECOM;

	conf[PECOM].romDir_[MAINROM1] = configPointer->Read("Dir/Pecom/Main_Rom_File", dataDir_ + "Pecom" + pathSeparator_);
	conf[PECOM].ramDir_ = configPointer->Read("Dir/Pecom/Software_File", dataDir_ + "Pecom" + pathSeparator_); 
	conf[PECOM].keyFileDir_ = configPointer->Read("Dir/Pecom/Key_File", dataDir_ + "Pecom" + pathSeparator_);
	conf[PECOM].printFileDir_ = configPointer->Read("Dir/Pecom/Print_File", dataDir_ + "Pecom" + pathSeparator_); 
	conf[PECOM].screenDumpFileDir_ = configPointer->Read("Dir/Pecom/Video_Dump_File", dataDir_ + "Pecom" + pathSeparator_); 
	conf[PECOM].wavFileDir_ = configPointer->Read("Dir/Pecom/Wav_File", dataDir_ + "Pecom" + pathSeparator_); 

	conf[PECOM].rom_[MAINROM1] = configPointer->Read("/Pecom/Main_Rom_File", "pecom64.v4.bin");
	conf[PECOM].keyFile_ = configPointer->Read("/Pecom/Key_File", "");
	conf[PECOM].printFile_ = configPointer->Read("/Pecom/Print_File", "printerout.txt");
	conf[PECOM].screenDumpFile_ = configPointer->Read("/Pecom/Video_Dump_File", "screendump.png");
	conf[PECOM].wavFile_ = configPointer->Read("/Pecom/Wav_File", "");

	conf[PECOM].volume_ = configPointer->Read("/Pecom/Volume", 25l);
	conf[PECOM].turboClock_ = configPointer->Read("/Pecom/Turbo_Clock_Speed", "15"); 
	conf[PECOM].mainX_ = configPointer->Read("/Pecom/Window_Position_X", mainWindowX_+windowInfo.mainwX); 
	conf[PECOM].mainY_ = configPointer->Read("/Pecom/Window_Position_Y", mainWindowY_); 
	conf[PECOM].printMode_ = configPointer->Read("/Pecom/Print_Mode", 1l);
	conf[PECOM].useLoadLocation_ = false;

	configPointer->Read("/Pecom/Enable_Turbo_Cassette", &conf[PECOM].turbo_, true);
	configPointer->Read("/Pecom/Enable_Auto_Cassette", &conf[PECOM].autoCassetteLoad_, true);
	configPointer->Read("/Pecom/Enable_Real_Cassette", &conf[PECOM].realCassetteLoad_, false);

	wxString defaultZoom;
	defaultZoom.Printf("%2.2f", 2.0);
	conf[PECOM].zoom_ = configPointer->Read("/Pecom/Zoom", defaultZoom);
	wxString defaultClock;
	defaultClock.Printf("%1.3f", 2.813);
	conf[PECOM].clock_ = configPointer->Read("/Pecom/Clock_Speed", defaultClock);

	setRealCas(PECOM);

	if (mode_.gui)
	{
		XRCCTRL(*this, "MainRomPecom", wxComboBox)->SetValue(conf[PECOM].rom_[MAINROM1]);
		XRCCTRL(*this, "KeyFilePecom", wxTextCtrl)->SetValue(conf[PECOM].keyFile_);
		XRCCTRL(*this, "PrintFilePecom", wxTextCtrl)->SetValue(conf[PECOM].printFile_);
		XRCCTRL(*this, "ScreenDumpFilePecom", wxComboBox)->SetValue(conf[PECOM].screenDumpFile_);
		XRCCTRL(*this, "WavFilePecom", wxTextCtrl)->SetValue(conf[PECOM].wavFile_);

		XRCCTRL(*this, "ZoomValuePecom", wxTextCtrl)->ChangeValue(conf[PECOM].zoom_);
		XRCCTRL(*this, "VolumePecom", wxSlider)->SetValue(conf[PECOM].volume_);

		clockTextCtrl[PECOM]->ChangeValue(conf[PECOM].clock_);
		XRCCTRL(*this, "TurboPecom", wxCheckBox)->SetValue(conf[PECOM].turbo_);
		turboGui("Pecom");
		XRCCTRL(*this, "TurboClockPecom", wxTextCtrl)->SetValue(conf[PECOM].turboClock_);
		XRCCTRL(*this, "AutoCasLoadPecom", wxCheckBox)->SetValue(conf[PECOM].autoCassetteLoad_);

		XRCCTRL(*this, "PrintModePecom", wxChoice)->SetSelection(configPointer->Read("/Pecom/Print_Mode", 1l));
		setPrintMode();
		XRCCTRL(*this, "UseLocationPecom", wxCheckBox)->SetValue(false);
	}
}

void GuiPecom::writePecomConfig()
{
	configPointer->Write("Dir/Pecom/Main_Rom_File", conf[PECOM].romDir_[MAINROM1]);
	configPointer->Write("Dir/Pecom/Software_File", conf[PECOM].ramDir_);
	configPointer->Write("Dir/Pecom/Key_File", conf[PECOM].keyFileDir_);
	configPointer->Write("Dir/Pecom/Print_File", conf[PECOM].printFileDir_);
	configPointer->Write("Dir/Pecom/Video_Dump_File", conf[PECOM].screenDumpFileDir_);
	configPointer->Write("Dir/Pecom/Wav_File", conf[PECOM].wavFileDir_);

	configPointer->Write("/Pecom/Main_Rom_File", conf[PECOM].rom_[MAINROM1]);
	configPointer->Write("/Pecom/Key_File", conf[PECOM].keyFile_);
	configPointer->Write("/Pecom/Print_File", conf[PECOM].printFile_);
	configPointer->Write("/Pecom/Video_Dump_File", conf[PECOM].screenDumpFile_);
	configPointer->Write("/Pecom/Wav_File", conf[PECOM].wavFile_);

	configPointer->Write("/Pecom/Zoom", conf[PECOM].zoom_);
	configPointer->Write("/Pecom/Volume", conf[PECOM].volume_);

	if (conf[PECOM].mainX_ > 0)
		configPointer->Write("/Pecom/Window_Position_X", conf[PECOM].mainX_);
	if (conf[PECOM].mainY_ > 0)
		configPointer->Write("/Pecom/Window_Position_Y", conf[PECOM].mainY_);

	configPointer->Write("/Pecom/Clock_Speed", conf[PECOM].clock_);
	configPointer->Write("/Pecom/Enable_Turbo_Cassette", conf[PECOM].turbo_);
	configPointer->Write("/Pecom/Turbo_Clock_Speed", conf[PECOM].turboClock_); 
	configPointer->Write("/Pecom/Enable_Auto_Cassette", conf[PECOM].autoCassetteLoad_);
	configPointer->Write("/Pecom/Enable_Real_Cassette", conf[PECOM].realCassetteLoad_);
	configPointer->Write("/Pecom/Print_Mode", conf[PECOM].printMode_);
}
