#ifndef ELF_H
#define ELF_H

#include <wx/dcclient.h> 

#include "elf.h"
#include "tms9918.h"
#include "pixie.h"
#include "mc6847.h"
#include "mc6845.h"
#include "i8275.h"
#include "keypad.h"
#include "ledmodule.h"
#include "printer.h"
#include "elfconfiguration.h"

class ElfScreen : public Panel
{
public:
	ElfScreen(wxWindow *parent, const wxSize& size);
	~ElfScreen();

	void init();
	void onPaint(wxPaintEvent&event);
	void onMousePress(wxMouseEvent& event);
	void onMouseRelease(wxMouseEvent& event);
private:

};

class Elf : public wxFrame,  public MainElf
{
public:
	Elf(const wxString& title, const wxPoint& pos, const wxSize& size, double clock, ElfConfiguration conf);
	Elf() {};
	~Elf();

	void onClose(wxCloseEvent&WXUNUSED(event));
	bool keyDownPressed(int keycode);
	bool keyUpReleased(int keycode);
	void charEvent(int keycode);

	void onRun();
	void onInButtonPress();
	void onInButtonPress(Byte value);
	void onInButtonRelease();
	void onHexKeyDown(int keycode);
	void onHexDown(int hex);
	void onHexKeyUp(int keycode);

	void configureComputer();
	void reDefineKeysA(int *, int *, int *);
	void reDefineKeysB(int *, int *, int *);
	void initComputer();
	Byte ef(int flag);
	Byte ef4();
	Byte in(Byte port, Word address);
	Byte getData();
	void out(Byte port, Word address, Byte value);
	void showData(Byte value);
	void cycle(int type);

	void autoBoot();
	void switchQ(int value);
	int getMpButtonState();
	void onMpButton();
	void onLoadButton();
	void dataSwitch(int i);
	void efSwitch(int i);

	void startComputer();
	void removeElfLedModule() {ledModuleClosed_ = true;};
	void showModules(bool status);
	void removeElfHex() {hexKeypadClosed_ = true;};
	Byte readMem(Word addr);
	void writeMem(Word addr, Byte value, bool writeRom);
	void cpuInstruction();
	void configureElfExtensions();
	void moveWindows();
	void updateTitle(wxString Title);
	void setForceUpperCase(bool status);
	void onReset();
	void sleepComputer(long ms);

	Byte getTmsMemory(int address) {return tmsPointer->getTmsMemory(address);};
	void setTmsMemory(int address, Byte value) {tmsPointer->setTmsMemory(address, value);};
	Byte read8275CharRom(Word addr);
	void write8275CharRom(Word addr, Byte value);
	Byte read6845CharRom(Word addr);
	void write6845CharRom(Word addr, Byte value);
	Byte read6847CharRom(Word addr);
	void write6847CharRom(Word addr, Byte value);
	int readDirect6847(Word addr);
	Word get6847RamMask();
	void writeDirect6847(Word addr, int value);
	void ledTimeout();
	void setLedMs(long ms);
	Byte getKey(Byte vtOut);
	void activateMainWindow();

private:
	class ElfScreen *elfScreenPointer;

	Tms9918 *tmsPointer;
	Pixie *pixiePointer;
	MC6845 *mc6845Pointer;
	mc6847 *mc6847Pointer;
	i8275 *i8275Pointer;
	Keypad *keypadPointer;
	LedModule *ledModulePointer;

	int runButtonState_;
	int mpButtonState_;
	int loadButtonState_;
	int dataSwitchState_[8];
	int efSwitchState_[4];

	bool inPressed_;
	Byte ef3State_;
	Byte switches_;
	Byte data_;
	Word lastAddress_;

	bool ledModuleClosed_;
	bool hexKeypadClosed_;

	int hexKeyDefA_[16];
	int keyDefGameHexA_[5];
	int keyDefGameValueA_[5];
	int keyDefGameHexB_[5];
	int keyDefGameValueB_[5];

	double elfClockSpeed_;

	DECLARE_EVENT_TABLE()
};

#endif  // ELF_H
