#ifndef MAIN_H
#define MAIN_H

//#ifdef  WIN32
typedef unsigned char Byte;
typedef unsigned short Word;
//#endif

#include "wx/file.h"
#include "wx/html/helpctrl.h"
#include "wx/spinctrl.h"
#include "wx/choicebk.h"
#include "vector"
#include "wx/listctrl.h"

// code defining event

class wxErrorMsgEvent: public wxNotifyEvent
{
public:
    wxErrorMsgEvent( wxEventType commandType = wxEVT_NULL, int id = 0 );

    // accessors
    wxString GetMsg()
        { return message; }
	void SetMsg(wxString msg)
		{ message = msg;}
    // required for sending with wxPostEvent()
	wxEvent *Clone(void) const { return new wxErrorMsgEvent(*this); }

private:
    wxString   message;
};

DECLARE_EVENT_TYPE( wxEVT_ERROR_MSG, -1 )

typedef void (wxEvtHandler::*wxErrorMsgEventFunction)(wxErrorMsgEvent&);

#define EVT_ERROR_MSG(id, fn) \
    DECLARE_EVENT_TABLE_ENTRY( wxEVT_ERROR_MSG, id, -1, \
    (wxObjectEventFunction) (wxEventFunction) (wxCommandEventFunction) (wxNotifyEventFunction) \
    wxStaticCastEvent( wxErrorMsgEventFunction, & fn ), (wxObject *) NULL ),

class guiEvent: public wxThreadEvent
{
public:
    guiEvent( wxEventType commandType = wxEVT_NULL, int id = 0 )
	: wxThreadEvent( commandType, id )
	{
	}

    // accessors
    bool GetBoolValue()			{ return boolValue_;}
    Word GetWordValue1()		{ return wordValue1_;}
    Word GetWordValue2()		{ return wordValue2_;}
    Word GetWordValue3()		{ return wordValue3_;}
    Byte GetByteValue1()		{ return byteValue1_;}
    Byte GetByteValue2()		{ return byteValue2_;}
    wxString GetStringValue2()	{ return stringValue2_;}
    void SetBoolValue(bool boolValue)	{ boolValue_ = boolValue;}
    void SetWordValue1(Word wordValue)	{ wordValue1_ = wordValue;}
    void SetWordValue2(Word wordValue)	{ wordValue2_ = wordValue;}
    void SetWordValue3(Word wordValue)	{ wordValue3_ = wordValue;}
    void SetByteValue1(Byte byteValue)	{ byteValue1_ = byteValue;}
    void SetByteValue2(Byte byteValue)	{ byteValue2_ = byteValue;}
    void SetStringValue2(wxString stringValue2)	{ stringValue2_ = stringValue2;}

    // required for sending with wxPostEvent()
	wxEvent *Clone(void) const { return new guiEvent(*this); }

private:
	bool boolValue_;
	Word wordValue1_;
	Word wordValue2_;
	Word wordValue3_;
	Byte byteValue1_;
	Byte byteValue2_;
	wxString stringValue2_;
};

wxDEFINE_EVENT(GUI_MSG, guiEvent);

#define guiEventHandler(func) (&func)

#define EVT_GUI_MSG(id, func) \
    wx__DECLARE_EVT1(GUI_MSG, id, (&func))

#define SET_LOCATION 1
#define SET_SW_NAME 2
#define SET_TAPE_STATE 3
#define SET_TEXT_VALUE 4
#define SET_CHECK_BOX 5
#define PRINT_DEFAULT 6
#define PRINT_PARALLEL 7
#define PRINT_PARALLEL_FINISHED 8
#define PRINT_THERMAL 9
#define PRINT_THERMAL_FINISHED 10
#define PRINT_THERMAL_CYCLE 11
#define PRINT_SERIAL 12
#define PRINT_SERIAL_FINISHED 13
#define PRINT_PECOM 14
#define SET_FM_GUI 16
#define SET_SAVE_START 17
#define SET_SAVE_END 18
#define ENABLE_MEM_ACCESS 19
#define SET_VIDEO_FULLSCREEN 20
#define SET_VT_FULLSCREEN 21

#define OS_WINDOWS_2000 0
#define OS_WINDOWS_XP 1
#define OS_WINDOWS_VISTA 2
#define OS_WINDOWS_7 3
#define OS_LINUX_UBUNTU_11_04 10
#define OS_LINUX_UBUNTU_11_10 11
#define OS_LINUX_OPENSUSE_KDE 20
#define OS_LINUX_OPENSUSE_GNOME 21


class WindowInfo
{
public:
	int xBorder, yBorder;
	int xBorder2, yBorder2;
	int xPrint;
	int mainwX, mainwY;
	int RegularClockY, RegularClockX;
	int ChoiceClockY, ChoiceClockX;
	int operatingSystem;
};

class Mode
{
	public: 
	bool window_position_fixed;
	bool gui;
	bool verbose;
	bool portable;
	bool full_screen;
	bool run;
	bool load;
	bool update_check;
};

class FloatEdit : public wxTextCtrl 
{ 
public: 
	FloatEdit();
	FloatEdit(wxWindow *parent, wxWindowID id, const wxString& value = wxEmptyString, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize);
	virtual void OnChar(wxKeyEvent& event); 

private:
	wxString comma_;
	DECLARE_DYNAMIC_CLASS(FloatEdit) 
}; 

class IntEdit : public wxTextCtrl 
{ 
public: 
	IntEdit();
	virtual void OnChar(wxKeyEvent& event); 

private:
	DECLARE_DYNAMIC_CLASS(IntEdit) 
}; 

class CheckBoxListCtrl : public wxListCtrl 
{ 
public: 
	CheckBoxListCtrl();
	void onMouseRelease(wxMouseEvent& event);

private:
	DECLARE_DYNAMIC_CLASS(CheckBoxListCtrl) 
}; 

class SlotEdit : public wxTextCtrl 
{ 
public: 
	SlotEdit();
	virtual void OnChar(wxKeyEvent& event); 
	void setRange(int min, int max); 
	void changeNumber(int number);
	void saveNumber(int number);

private:
	int min_;
	int max_;
	int number_;

	DECLARE_DYNAMIC_CLASS(SlotEdit) 
}; 

class HexEdit : public wxTextCtrl 
{ 
public: 
	HexEdit();
	virtual void OnChar(wxKeyEvent& event); 
	void setStart(int min); 
	void changeNumber(int number);
	void saveNumber(int number);

private:
	int min_;
	int number_;

	DECLARE_DYNAMIC_CLASS(HexEdit) 
}; 

class HexEditX : public wxTextCtrl 
{ 
public: 
	HexEditX();
	virtual void OnChar(wxKeyEvent& event); 

private:
	DECLARE_DYNAMIC_CLASS(HexEditX) 
}; 

class MemEdit : public wxTextCtrl 
{ 
public: 
	MemEdit();
	virtual void OnChar(wxKeyEvent& event); 
	void changeNumber1X(int number);
	void changeNumber2X(int number);
	void saveNumber(int number);

private:
	int number_;

	DECLARE_DYNAMIC_CLASS(MemEdit) 
}; 
class ScreenInfo
{
public:
	int start;
	int number;
	wxString defaultColour[66];
	int numberVideo;
	int borderX[10];
	int borderY[10];
};

#include "cidelsa.h"
#include "comx35.h"
#include "hbelf.h"
#include "member.h"
#include "elf2.h"
#include "elf2k.h"
#include "cosmicos.h"
#include "super.h"
#include "studio2.h"
#include "visicom.h"
#include "victory.h"
#include "vip.h"
#include "pecom.h"
#include "tmc600.h"
#include "tmc1800.h"
#include "tmc2000.h"
#include "eti660.h"
#include "nano.h"
#include "guicomx.h"
#include "debug.h"
#include "video.h"

#define EMMA_VERSION "1.18"
#define ELF 0
#define ELFII 1
#define SUPERELF 2
#define ELF2K 3
#define COSMICOS 4
#define MEMBER 5
#define ETI 6
#define COMX 7
#define STUDIO 8
#define VIP 9
#define CIDELSA 10
#define TMC600 11
#define TMC1800 12
#define TMC2000 13
#define NANO 14
#define PECOM 15
#define VISICOM 16
#define VICTORY 17
#define DEBUGGER 18
#define TELMACPRINTER 0
#define PECOMPRINTER 3
#define VIPPRINTER 4
#define ELFPRINTER 5
#define COMXPRINTER 1
#define COMXTHPRINTER 2
#define COMXFLOP 3
#define COMX80COLUMN 4
#define COMXRAM 6
#define COMXJOY 16
#define COMXRS232 17
#define COMXEPROMBOARD 0x73
#define COMXEMPTY 255
#define PRINTFILE 0
#define PRINTWINDOW 1
#define NOPRINTER 2
#define COMXPRINTPLOTTER 1
#define COMXPRINTPRINTER 2
#define TIMERINTERVAL 50
#define PRINTER_PLOTTER 6000
#define PLOTTEREXTTEXT 6001
#define PLOTTEREXT 6002
#define PLOTTERROMTEXT 6003
#define PLOTTERROM 6004
#define PRINTERROWS 6005
#define PRINTERFONT 6006
#define PAL 0
#define NTSC 1
#define COMXTAB 0
#define COSMACTAB 1
#define STUDIOTAB 2
#define CIDELSATAB 3
#define TELMACTAB 4
#define PECOMTAB 5
#define ETITAB 6
#define DEBUGGERTAB 7
#define DISKNONE 0
#define DISKFDC 1
#define DISKIDE 2

#define ELF2KTAB 0
#define VIPTAB 1
#define COSMICOSTAB 2
#define ELFTAB 3
#define ELFIITAB 4
#define SUPERELFTAB 5
#define MEMBERTAB 6

#define STUDIOIITAB 0
#define VISICOMTAB 1
#define VICTORYTAB 2

#define TMC600TAB 0
#define TMC1800TAB 1
#define TMC2000TAB 2
#define NANOTAB 3

#define MESSAGETAB 0
#define TRACETAB 1
#define CHIP8TAB 2
#define MEMORYTAB 3
#define ASSTAB 4

#define VIDEONONE 0
#define VIDEO1870 0
#define VIDEO80COL 2

#define VIDEOVT 0
#define VIDEOPIXIE 1
#define VIDEO2KI8275 2

//#define VIDEOVT 0
//#define VIDEOPIXIE 1
#define VIDEO6845 2
#define VIDEO6847 3
#define VIDEOTMS 4
#define VIDEOI8275 5
#define VIDEOS100 6

#define ELFLED 0
#define ELFIILED 1
#define SUPERELFLED 2
#define ELF2KLED1 3
#define ELF2KLED2 4
#define ELF2KLED3 5
#define COSMICOSLED 6
#define MEMBERLED 7

#define KEYBOARDNONE 0
#define KEYBOARDPS2 1
#define KEYBOARDPS2INT 2
#define KEYBOARDASCII 3
#define KEYBOARDHEXEF3 4
#define KEYBOARDHEX 5

#define KEYBOARD2KPS2GPIOJP4 2
#define KEYBOARDELF2KHEX 3

#define KEYBOARDHEXCOSMICOS 1

#define COMXPARALLEL 0
#define COMXSERIAL 1
#define COMXTHERMAL 2

#define UNDEFINED 0
#define RAM 1
#define ROM 2
#define PAGER 3
#define CRAM1870 5
#define PRAM1870 6
#define COMXEXPROM 7
#define COPYCOMXEXPROM 8
#define RAMBANK 9
#define MC6845RAM 11
#define MC6845REGISTERS 12
#define COPYFLOPROM 13
#define EMSMEMORY 15
#define COMXEXPBOX 16
#define MC6847RAM 18
#define CARTRIDGEROM 19
#define MAPPEDRAM 20
#define COLOURRAM 21
#define VP570RAM 22 
#define EPROMBANK 23
#define NOCHANGE 30

#define SHOWNAME true
#define NONAME false

#define HEX_UP 0
#define HEX_LEFT 1
#define HEX_RIGHT 2
#define HEX_DOWN 3
#define HEX_FIRE 4

#define NOPROGRAM 0
#define SUPERBASICV1 1 // bit 0=1: .super/.rca load
#define SUPERBASICV3 3 
#define SUPERBASICV5 5 
#define SUPERBASICV6 7 
#define RCABASIC3 9
#define RCABASIC4 11

#define TINYBASIC 2
#define MINIMON 4
#define GOLDMON 6

#define BASICADDR_KEY 0
#define BASICADDR_READY 1
#define BASICADDR_KEY_VT_RESTART 2
#define BASICADDR_KEY_VT_INPUT 3

#define BASICADDR_KEY_SB1 0x2625
#define BASICADDR_READY_SB1 0x3f
#define BASICADDR_RUN_SB1 0x16a3
#define BASICADDR_CALL_SB1 0x1082
#define BASICADDR_PSAVE_SB1 0x0107
#define BASICADDR_PLOAD_SB1 0x0101
#define BASICADDR_DSAVE_SB1 0x010a
#define BASICADDR_DLOAD_SB1 0x0104
#define BASICADDR_ROUTINE1_SB1 0x2a9f
#define BASICADDR_ROUTINE2_SB1 0x35a3
#define BASICADDR_VT_RESTART_SB1 0x2880
#define BASICADDR_VT_INPUT_SB1 0x71

#define BASICADDR_KEY_SB3 0x2e09
#define BASICADDR_READY_SB3 0x37
#define BASICADDR_RUN_SB3 0x0cdc
#define BASICADDR_CALL_SB3 0x03d8
#define BASICADDR_PSAVE_SB3 0x2b06
#define BASICADDR_PLOAD_SB3 0x2b00
#define BASICADDR_DSAVE_SB3 0x2b09
#define BASICADDR_DLOAD_SB3 0x2b03
#define BASICADDR_ROUTINE1_SB3 0x2f9f
#define BASICADDR_ROUTINE2_SB3 0x2fa3
#define BASICADDR_VT_RESTART_SB3 -1
#define BASICADDR_VT_INPUT_SB3 -1

#define BASICADDR_KEY_SB5 0x3309
#define BASICADDR_READY_SB5 0x143
#define BASICADDR_RUN_SB5 0x1691
#define BASICADDR_CALL_SB5 0x0900
#define BASICADDR_PSAVE_SB5 0x0E06
#define BASICADDR_PLOAD_SB5 0x0E00
#define BASICADDR_DSAVE_SB5 0x0E09
#define BASICADDR_DLOAD_SB5 0x0E03
#define BASICADDR_ROUTINE1_SB5 0x359f
#define BASICADDR_ROUTINE2_SB5 0x35a3
#define BASICADDR_VT_RESTART_SB5 0x34a3
#define BASICADDR_VT_INPUT_SB5 0x3428

#define BASICADDR_KEY_SB6 0x3109
#define BASICADDR_READY_SB6 0x143
#define BASICADDR_RUN_SB6 0x1d00
#define BASICADDR_CALL_SB6 0x1b00
#define BASICADDR_PSAVE_SB6 0x03f1
#define BASICADDR_PLOAD_SB6 0x03eb
#define BASICADDR_DSAVE_SB6 0x03f4
#define BASICADDR_DLOAD_SB6 0x03ee
#define BASICADDR_ROUTINE1_SB6 0x359f
#define BASICADDR_ROUTINE2_SB6 0x35a3
#define BASICADDR_VT_RESTART_SB6 -1
#define BASICADDR_VT_INPUT_SB6 -1

#define BASICADDR_READY_RCA3 0x2257
#define BASICADDR_RUN_RCA3 0x3276
#define BASICADDR_CALL_RCA3 -1
#define BASICADDR_ROUTINE1_RCA3 0x529f
#define BASICADDR_ROUTINE2_RCA3 0x52a3

#define BASICADDR_READY_RCA4 0x2257
#define BASICADDR_RUN_RCA4 0x3276
#define BASICADDR_CALL_RCA4 -1
#define BASICADDR_ROUTINE1_RCA4 0x529f
#define BASICADDR_ROUTINE2_RCA4 0x52a3

#define BASICADDR_VT_RESTART_RCA 0xfc19
#define BASICADDR_VT_INPUT_RCA 0xfc98

#define NOOS 0
#define ELFOS 1

#define HEXMON 2
#define ASCIIMON 4

#define QSOUNDOFF 0
#define QSOUNDSW 1
#define QSOUNDEXT 2

#define VTNONE 0
#define VT52 1
#define VT100 2

	// 1 Scroll - repeat - screen reverse - cursor block line
	// 2 bell - keyklick - ansi/vt52 - xon/xoff
	// 3 US - wrap - newline - interlace
	// 4 parity sense - parity - bits per char - 60/50Hz

#define VTSMOOTHSCROLL 15
#define VTREPEAT 14
#define VTREVERSESCREEN 13
#define VTCURSORBLOCK 12
#define VTBELL 11
#define VTKEYCLICK 10
#define VTANSI 9
#define VTAUTOXON 8
#define VTUSASCII 7
#define VTWRAPAROUND 6
#define VTNEWLINE 5
#define VTINTERLACE 4
#define VTPARITYSENSE 3
#define VTPARITY 2
#define VTBITS 1
#define VTPOWER 0

#define VIP_BEEP 0
#define VIP_1864 1
#define VIP_SUPER2 2
#define VIP_SUPER4 3

#define GUIQUIT "MI_Quit"
#define GUIABOUT "MI_About"
#define GUIHELP "MI_Help"
#define GUISAVEONEXIT "MI_SaveOnExit"
#define GUISAVECONFIG "MI_SaveConfig"
#define GUIDEFAULTWINDOWPOS "MI_DefaultWindowPosition"
#define GUIDEFAULT "MI_DefaultSettings"
#define GUIPROTECTEDMODE "ProtectedMode"

#define GUICOMPUTERNOTEBOOK "Computer"

#define RESETSTATE 0
#define BASICSTATE 1
#define RUNSTATE 2

#define LEFTCHANNEL false
#define RIGHTCHANNEL true

#define GUI_ELF2K_BAUDR 30000
#define GUI_ELF2K_BAUDT 30001
#define GUI_ELF_BAUDR 30002
#define GUI_ELF_BAUDT 30003
//#define GUI_ELFII_BAUDR 30004
//#define GUI_ELFII_BAUDT 30005
//#define GUI_SUPERELF_BAUDR 30006
//#define GUI_SUPERELF_BAUDT 30007
#define GUI_COSMICOS_BAUDR 30008
#define GUI_COSMICOS_BAUDT 30009
#define GUI_MEMBER_BAUDR 30010
#define GUI_MEMBER_BAUDT 30011
#define GUI_CLOCK_TEXTCTRL 30020
#define GUI_START_BUTTON 30040

#define TAPE_STOP 0
#define TAPE_PLAY 1
#define TAPE_RECORD 2

#define PRINT_BUFFER_SIZE 1000

#define CHIP_NONE 0
#define CHIP8 1
#define CHIP8X 2
#define CHIPETI 3
#define CHIPST2 4

#define CPU1802 0
#define CPU1804 4
#define CPU1805 5

class Emu1802: public wxApp
{
	virtual bool OnInit();
	virtual int OnExit();
    virtual int OnRun();
    virtual void OnInitCmdLine(wxCmdLineParser& parser);
    virtual bool OnCmdLineParsed(wxCmdLineParser& parser);
	void getSoftware(wxString computer, wxString type, wxString software);

private:
	wxConfigBase *regPointer;
	wxConfigBase *configPointer;

	Mode mode_;
    int startComputer_;
	wxString dataDir_;
};

WindowInfo getWinSizeInfo();

class Main: public DebugWindow
{
public:

	Main(const wxString& title, const wxPoint& pos, const wxSize& size, Mode mode, wxString dataDir, wxConfigBase *regPointer);
	~Main();

	void pageSetup();
	void onClose(wxCloseEvent&event );

	bool checkUpdateEmma();
	bool updateEmma();
	wxString downloadString(wxString url);

	void onQuit(wxCommandEvent& event);
	void onAbout(wxCommandEvent& event);
	void onDataDir(wxCommandEvent& event);
	void onHome(wxCommandEvent& event);
	void onUpdateCheck(wxCommandEvent& event);
	void onUpdateEmma(wxCommandEvent& event);
	void onHelp(wxCommandEvent& event);
	void onSaveConfig(wxCommandEvent& event);
	void onSaveOnExit(wxCommandEvent& event);
	void onDefaultWindowPosition(wxCommandEvent& event);
	void onFixedWindowPosition(wxCommandEvent& event);
	void nonFixedWindowPosition();
	void fixedWindowPosition();
	void onFunctionKeys(wxCommandEvent& event);
	void onDefaultSettings(wxCommandEvent& event);
	void setDefaultSettings();
	void on1802(wxCommandEvent& event);
	void on1804(wxCommandEvent& event);
	void on1805(wxCommandEvent& event);
	void onFlat(wxCommandEvent& event);
	void onCrisp(wxCommandEvent& event);
	void onDefault(wxCommandEvent& event);
	void onTvSpeaker(wxCommandEvent& event);
	void onHandheld(wxCommandEvent& event);
	void onChar(wxKeyEvent&event);
	void onKeyDown(wxKeyEvent&event);
	bool checkFunctionKey(wxKeyEvent& event);
	void onActivateMainWindow(wxCommandEvent& event);
	void activateMainWindow();
	void onFullScreenMenu(wxCommandEvent& event);
	void fullScreenMenu();
	void popUp();
	void onKeyUp(wxKeyEvent&event);
	void connectKeyEvent(wxWindow* pclComponent);

	void onComputer(wxNotebookEvent& event);
	void onStudioChoiceBook(wxChoicebookEvent& event);
	void onTelmacChoiceBook(wxChoicebookEvent& event);
	void onCosmacChoiceBook(wxChoicebookEvent& event);
	void onDebuggerChoiceBook(wxChoicebookEvent& event);
	void setNoteBook();
	void onStart(wxCommandEvent& event);
	void onStart(int computer);

	void stopComputer();
	void killComputer(wxCommandEvent&WXUNUSED(event));
	void enableElfConfig(bool status);
	void enableGui(bool status);
	void message(wxString buffer);
	void messageInt(int value);
	void messageHex(int value);
	void readConfig();
	void writeConfig();

	wxString getApplicationDir();

	wxChar getPathSep();
	int setFdcStepRate(int rate);
	int getPsaveData(int item);
	void setPsaveData(int item, int data);
	int getFunctionKey(int item);
	void setFunctionKey(int item, int value);
	bool getUseExitKey() {return useExitKey_;};
	void setUseExitKey(bool status) {useExitKey_ = status;};
	void vuTimeout(wxTimerEvent& event);
	void updateMemoryTab();
	void updateSlotInfo();
	void ledTimeout(wxTimerEvent& event);
	void cpuTimeout(wxTimerEvent& event);
	void updateCheckTimeout(wxTimerEvent& event);
	void startTime();
	void showTime();
	void zoomEvent(double zoom);
	void zoomEventVt(double zoom);
	void vuSet(wxString Item, int value);
	void errorMessageEvent(wxErrorMsgEvent& event);
	void errorMessage(wxString msg);

	void setLocationEvent(guiEvent& event);
	void eventSetLocation(bool state, Word saveStart, Word saveEnd, Word saveExec);
	void eventSetLocation(bool state);

	void setSaveStartEvent(guiEvent& event);
	void eventSaveStart(Word saveStart);

	void setSaveEndEvent(guiEvent& event);
	void eventSaveEnd(Word saveEnd);

	void setSwNameEvent(guiEvent& event);
	void eventSetSwName(wxString swName);

	void setTapeStateEvent(guiEvent& event);
	void eventSetTapeState(int status);

	void setTextValueEvent(guiEvent& event);
	void eventSetTextValue(wxString info, wxString value);

	void setCheckBoxEvent(guiEvent& event);
	void eventSetCheckBox(wxString info, bool state);

	void printDefaultEvent(guiEvent& event);
	void eventPrintDefault(Byte value);

	void printParallelEvent(guiEvent& event);
	void eventPrintParallel(Byte value);
	void printParallelFinishedEvent(guiEvent& event);
	void eventPrintParallelFinished();

	void printThermalEvent(guiEvent& event);
	void eventPrintThermal(Byte value, Byte Qflag);
	void printThermalFinishedEvent(guiEvent& event);
	void eventPrintThermalFinished();

	void printSerialEvent(guiEvent& event);
	void eventPrintSerial(Byte value);
	void printSerialFinishedEvent(guiEvent& event);
	void eventPrintSerialFinished();

	void printPecomEvent(guiEvent& event);
	void eventPrintPecom(Byte value);

	void updateDebugMemory(Word address);

	void showChip8Register(int variable, int value);

	void setFandMBasicGuiEvent(guiEvent& event);
	void eventSetFandMBasicGui();

	void enableMemAccesEvent(guiEvent& event);
	void eventEnableMemAccess(bool state);

	void setVideoFullScreenEvent(guiEvent& event);
	void eventVideoSetFullScreen(bool state);

	void setVtFullScreenEvent(guiEvent& event);
	void eventVtSetFullScreen(bool state);

	void loadKeyDefinition(wxString gameName1, wxString gameName2, int *, int *);
	int getDefaultInKey(wxString computerStr);
	void getDefaultGameKeys(wxString computerStr, wxString player, int *, int *);
	void storeDefaultGameKeys(wxString computerStr, wxString player, int *, int *);
	void getDefaultHexKeys(wxString computerStr, wxString player, int *);
	bool emuClosing() {return emuClosing_;};
	bool getThermalEf() {return thermalEf_;};
	void setStatusLedUpdate(bool status) {statusLedUpdate_ =  status;};

private:
	wxHtmlHelpController *help_;
	wxString latestVersion_;

	Sync_Audio *audioPointer;

	bool saveOnExit_;
	bool checkForUpdate_;

	bool thermalEf_;
	bool statusLedUpdate_;

	bool emuClosing_;
	bool emmaClosing_;
	int bass_;
	int treble_;
	wxTimer *cpuPointer;
	wxTimer *updateCheckPointer;
	int oldGauge_;

	wxString message_;
	time_t startTime_;

	int eventNumber_;

	bool updateMemory_;
	bool updateMemoryPage_;
	bool updateSlotinfo_;
	bool rowChanged_[16];
	bool memoryChanged_[16][16];

	int functionKey_[13];

	DECLARE_EVENT_TABLE()
};

#endif // MAIN_H

#undef EXT
#define EXT extern

#ifdef MAIN
#undef EXT
#define EXT
#endif

EXT Main *p_Main;
EXT Video *p_Video;
EXT Video *p_Vt100;
EXT Cdp1802 *p_Computer;

EXT	Printer *p_PrinterParallel;
EXT	Printer *p_PrinterSerial;
EXT	Printer *p_PrinterThermal;
EXT	Printer *p_Printer;
EXT wxPrintData *PrintDataPointer;
EXT wxPageSetupDialogData *p_PageSetupData;
