#ifndef MEMORY_H
#define MEMORY_H

class Memory
{
public:
	Memory();
	~Memory();

	void defineMemoryType(long start, long end, int type); 
	void defineMemoryType(long address, int type); 
	void defineExpansionMemoryType(int slot, long start, long end, int type); 
	void defineExpansionMemoryType(int slot, long address, int type); 
	void defineBankMemoryType(int bank, long address, int type); 
	void defineEpromBankMemoryType(int bank, long address, int type); 
	void defineEmsMemoryType(long address, int type); 
	void definePagerMemoryType(long address, int type); 
	void allocPagerMemory();
	void allocComxExpansionMemory();
	void allocEmsMemory();
	void setEmsPage(Byte value);
	void setPager(int page, Byte value);
	void randomMemory(long start, long end);
	
	virtual Byte readMem(Word addr) = 0;
	virtual void writeMem(Word addr, Byte value, bool writeRom) = 0; 

	Byte getRam(long address) {return mainMemory_[address];};
	void setRam(long address, Byte value) {mainMemory_[address] = value;};
	Byte getEms(long address) {return emsRam_[address];};
	Byte getEmsPage() {return emsPage_;};
	Byte getPager(int port) {return pager_[port];};
	int getMemoryType(int i) {return memoryType_[i];};
	int getExpansionMemoryType(int slot, int i) {return expansionMemoryType_[slot*32 + i];};
	int getBankMemoryType(int bank, int i) {return bankMemoryType_[bank*32 + i];};
	int getEpromBankMemoryType(int bank, int i) {return epromBankMemoryType_[bank*32 + i];};
	int getEmsMemoryType(int i) {return emsMemoryType_[emsPage_*64 + i];};
	int getPagerMemoryType(int i) {return pagerMemoryType_[pager_[(i>>4)&0xf] * 16 + (i&0xf)];};

protected: 
	Word address_;
	Byte* mainMemory_;
	int memoryType_[256];
	int expansionMemoryType_[128];
	int bankMemoryType_[128];
	int epromBankMemoryType_[160];
	int emsMemoryType_[2048];
	int pagerMemoryType_[4096];
	int colorMemory1864_[1024];
	Byte mc6845ram_[2048];
	Byte mc6845CharRom_[2048];

	Byte* expansionRom_;
	Byte* expansionRam_;
	Byte* expansionEprom_;

	Byte* emsRam_;
	Byte emsPage_;
	bool emsMemoryDefined_;
	bool pagerDefined_;

private:
	bool comxExpansionMemoryDefined_;
	int pager_[16];

};

#endif  // MEMORY_H