/*
 *******************************************************************
 *** This software is copyright 2008 by Marcel van Tongeren      ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 *******************************************************************
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "main.h"
#include "ports.h"

#include "wx/xrc/xmlres.h"             
#include "wx/spinctrl.h"
#include "elfconfiguration.h"

BEGIN_EVENT_TABLE(DevicePortsDialog, wxDialog)
    EVT_BUTTON(XRCID("DevicePorts_Save"), DevicePortsDialog::onSaveButton)
	EVT_CHOICE(XRCID("MC6847-B7"), DevicePortsDialog::onMC6847B7)
	EVT_CHOICE(XRCID("MC6847-B6"), DevicePortsDialog::onMC6847B6)
	EVT_CHOICE(XRCID("MC6847-B5"), DevicePortsDialog::onMC6847B5)
	EVT_CHOICE(XRCID("MC6847-B4"), DevicePortsDialog::onMC6847B4)
	EVT_CHOICE(XRCID("MC6847-B3"), DevicePortsDialog::onMC6847B3)
	EVT_CHOICE(XRCID("MC6847-B2"), DevicePortsDialog::onMC6847B2)
	EVT_CHOICE(XRCID("MC6847-B1"), DevicePortsDialog::onMC6847B1)
	EVT_CHOICE(XRCID("MC6847-B0"), DevicePortsDialog::onMC6847B0)
	EVT_CHOICE(XRCID("MC6847-DD7"), DevicePortsDialog::onMC6847DD7)
	EVT_CHOICE(XRCID("MC6847-DD6"), DevicePortsDialog::onMC6847DD6)
END_EVENT_TABLE()

DevicePortsDialog::DevicePortsDialog(wxWindow* parent)
{
	wxXmlResource::Get()->Load(p_Main->getApplicationDir()+p_Main->getPathSep()+"ports.xrc");
	wxXmlResource::Get()->LoadDialog(this, parent, "DevicePorts");
	ElfConfiguration elfConfiguration = p_Main->getElfConfiguration();
	elfTypeStr_ = p_Main->getSelectedComputerStr();
	this->SetTitle("Device Ports Definition "+p_Main->getSelectedComputerText());

	XRCCTRL(*this, "PixieInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PixieInput", 1l));
	XRCCTRL(*this, "PixieOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PixieOutput", 1l));
	XRCCTRL(*this, "PixieEF", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PixieEF", 1l));
	if (!elfConfiguration.usePixie)
	{
		XRCCTRL(*this, "PixieInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PixieOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PixieEF", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PixieInputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "PixieOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "PixieEfText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "PortExtenderSelectOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PortExtenderSelectOutput", 5l));
	XRCCTRL(*this, "PortExtenderWriteOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PortExtenderWriteOutput", 6l));
	XRCCTRL(*this, "PortExtenderInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PortExtenderInput", 6l));
	if (!elfConfiguration.usePortExtender)
	{
		XRCCTRL(*this, "PortExtenderSelectOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PortExtenderWriteOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PortExtenderInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PortExtenderSelectOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "PortExtenderWriteOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "PortExtenderInputText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "IdeSelectOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeSelectOutput", 2l));
	XRCCTRL(*this, "IdeWriteOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeWriteOutput", 3l));
	XRCCTRL(*this, "IdeStatus", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeStatus", 2l));
	XRCCTRL(*this, "IdeInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeInput", 3l));
	XRCCTRL(*this, "IdeTracks", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeTracks", 512l));
	XRCCTRL(*this, "IdeHeads", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeHeads", 4l));
	XRCCTRL(*this, "IdeSectors", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/IdeSectors", 26l));
	if (!elfConfiguration.ideEnabled)
	{
		XRCCTRL(*this, "IdeSelectOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeWriteOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeStatus", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeTracks", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeHeads", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeSectors", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeSelectOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "IdeWriteOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "IdeInputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "IdeStatusText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "text_tr", wxStaticText)->Enable(false);
		XRCCTRL(*this, "text_hd", wxStaticText)->Enable(false);
		XRCCTRL(*this, "text_sc", wxStaticText)->Enable(false);
	}	
	if (elfTypeStr_ != "Elf2K")
	{
		XRCCTRL(*this, "IdeStatus", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "IdeStatusText", wxStaticText)->Enable(false);
	}

	XRCCTRL(*this, "FdcSelectOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/FdcSelectOutput", 2l));
	XRCCTRL(*this, "FdcWriteOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/FdcWriteOutput", 3l));
	XRCCTRL(*this, "FdcInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/FdcInput", 3l));
	XRCCTRL(*this, "FdcEf", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/FdcEf", 2l));
	if (!elfConfiguration.fdcEnabled)
	{
		XRCCTRL(*this, "FdcSelectOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "FdcWriteOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "FdcInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "FdcEf", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "FdcSelectOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "FdcWriteOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "FdcInputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "FdcEfText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "KeyboardInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/KeyboardInput", 7l));
	XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/KeyboardEf", 3l));
	if (!elfConfiguration.useKeyboard && !elfConfiguration.usePs2gpio)
	{
		XRCCTRL(*this, "KeyboardInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "KeyboardInputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "KeyboardEfText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "Ps2KeyboardInput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Ps2KeyboardInput", 7l));
	XRCCTRL(*this, "Ps2KeyboardOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Ps2KeyboardOutput", 7l));
	XRCCTRL(*this, "Ps2KeyboardEf", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Ps2KeyboardEf", 3l));
	if (!elfConfiguration.UsePS2)
	{
		XRCCTRL(*this, "Ps2KeyboardInput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Ps2KeyboardOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Ps2KeyboardEf", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Ps2KeyboardInputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Ps2KeyboardOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Ps2KeyboardEfText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "PrinterOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/PrinterOutput", 7l));
	if (!p_Main->getPrinterStatus(p_Main->getSelectedComputerId()))
	{
		XRCCTRL(*this, "PrinterOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "PrinterOutputText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "Vt100Output", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Vt100Output", 7l));
	if (elfTypeStr_ != "Elf2K")
	{
		XRCCTRL(*this, "Vt100Ef", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Vt100Ef", 2l));
	}
	else
	{
		XRCCTRL(*this, "Vt100Ef", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Vt100Ef", 3l));
		XRCCTRL(*this, "Vt100Output", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Vt100OutputText", wxStaticText)->Enable(false);
	}
	XRCCTRL(*this, "Vt100ReverseEf", wxCheckBox)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Vt100ReverseEf", 1l) != 1);
	XRCCTRL(*this, "Vt100ReverseQ", wxCheckBox)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Vt100ReverseQ", 0l) != 0);
	if ((elfConfiguration.vtType == VTNONE) || elfConfiguration.useUart)
	{
		XRCCTRL(*this, "Vt100Q", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Vt100Output", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Vt100Ef", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Vt100ReverseQText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Vt100OutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Vt100EfText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "Vt100ReverseEf", wxCheckBox)->Enable(false);
		XRCCTRL(*this, "Vt100ReverseQ", wxCheckBox)->Enable(false);
	}	

	XRCCTRL(*this, "UartOut", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/UartOut", 2l));
	XRCCTRL(*this, "UartIn", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/UartIn", 2l));
	XRCCTRL(*this, "UartControl", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/UartControl", 3l));
	XRCCTRL(*this, "UartStatus", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/UartStatus", 3l));
	if (!elfConfiguration.useUart || (elfTypeStr_ == "Elf2K"))
	{
		XRCCTRL(*this, "UartOutText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "UartOut", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "UartInText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "UartIn", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "UartControlText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "UartControl", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "UartStatusText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "UartStatus", wxSpinCtrl)->Enable(false);
	}	

	XRCCTRL(*this, "TmsModeHighOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/TmsModeHighOutput", 5l));
	XRCCTRL(*this, "TmsModeLowOutput", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/TmsModeLowOutput", 6l));
	if (!elfConfiguration.useTMS9918)
	{
		XRCCTRL(*this, "TmsModeHighOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "TmsModeLowOutput", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "TmsModeHighOutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "TmsModeLowOutputText", wxStaticText)->Enable(false);
	}	


	XRCCTRL(*this, "I8275WriteCommand", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/I8275WriteCommand", 5l));
	XRCCTRL(*this, "I8275ReadStatus", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/I8275ReadStatus", 5l));
	XRCCTRL(*this, "I8275WriteParameter", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/I8275WriteParameter", 1l));
	XRCCTRL(*this, "I8275ReadParameter", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/I8275ReadParameter", 1l));
	XRCCTRL(*this, "I8275VerticalRetrace", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/I8275VerticalRetrace", 1l));
	if (!elfConfiguration.use8275)
	{
		XRCCTRL(*this, "I8275WriteCommand", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "I8275ReadStatus", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "I8275WriteParameter", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "I8275ReadParameter", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "I8275VerticalRetrace", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "I8275WriteCommandText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "I8275ReadStatusText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "I8275WriteParameterText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "I8275ReadParameterText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "I8275VerticalRetraceText", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "Led_Module_Output", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/Led_Module_Output", 4l));
	if (!elfConfiguration.useLedModule)
	{
		XRCCTRL(*this, "Led_Module_Output", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "Led_Module_Output_Text", wxStaticText)->Enable(false);
	}	

	XRCCTRL(*this, "MC6847Output", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/MC6847Output", 5l));
	XRCCTRL(*this, "MC6847-B7", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B7", 0l));
	XRCCTRL(*this, "MC6847-B6", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B6", 0l));
	XRCCTRL(*this, "MC6847-B5", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B5", 0l));
	XRCCTRL(*this, "MC6847-B4", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B4", 0l));
	XRCCTRL(*this, "MC6847-B3", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B3", 3l));
	XRCCTRL(*this, "MC6847-B2", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B2", 4l));
	XRCCTRL(*this, "MC6847-B1", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B1", 6l));
	XRCCTRL(*this, "MC6847-B0", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-B0", 5l));
	XRCCTRL(*this, "MC6847-DD7", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-DD7", 1l));
	XRCCTRL(*this, "MC6847-DD6", wxChoice)->SetSelection(p_Main->getConfigItem(elfTypeStr_+"/MC6847-DD6", 0l));

	int start = p_Main->getConfigItem(elfTypeStr_+"/mc6847StartRam", 0xe000l);
	int sel = (start - 0x8000)/0x1000;
	XRCCTRL(*this, "MC6847StartRam", wxChoice)->SetSelection(sel);

	int end = p_Main->getConfigItem(elfTypeStr_+"/mc6847EndRam", 0xe3ffl);
	int size = end - start;
	sel = 0;
	if (size <= 1024)
		sel = 0;
	else if (size <= 1536)
		sel = 1;
	else if (size <= 2048)
		sel = 2;
	else if (size <= 3072)
		sel = 3;
	else if (size <= 6144)
		sel = 4;
	XRCCTRL(*this, "MC6847SizeRam", wxChoice)->SetSelection(sel);

	if (!elfConfiguration.use6847)
	{
		XRCCTRL(*this, "MC6847Output", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "MC6847OutputText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B7Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B6Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B5Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B4Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B3Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B2Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B1Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B0Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-DD7Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-DD6Text", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847-B7", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B6", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B5", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B4", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B3", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B2", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B1", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-B0", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-DD7", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847-DD6", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847StartRam", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847StartRamText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6847SizeRam", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6847SizeRamText", wxStaticText)->Enable(false);
	}	

	start = p_Main->getConfigItem(elfTypeStr_+"/mc6845StartRam", 0xe000l);
	sel = start/0x1000;
	XRCCTRL(*this, "MC6845StartRam", wxChoice)->SetSelection(sel);

	end = p_Main->getConfigItem(elfTypeStr_+"/mc6845EndRam", 0xe7ffl);
	size = end - start;
	sel = 0;
	if (size <= 1024)
		sel = 0;
	else if (size <= 2048)
		sel = 1;
	else if (size <= 4096)
		sel = 2;
	else if (size <= 8192)
		sel = 3;
	else if (size <= 16384)
		sel = 4;
	XRCCTRL(*this, "MC6845SizeRam", wxChoice)->SetSelection(sel);

	int addr = p_Main->getConfigItem(elfTypeStr_+"/mc6845Address", 0xe800l);
	wxString addrStr;
	addrStr.Printf("%04X", addr);
	XRCCTRL(*this, "MC6845Address", wxTextCtrl)->SetValue(addrStr);
	int data = p_Main->getConfigItem(elfTypeStr_+"/mc6845Data", 0xe801l);
	wxString dataStr;
	dataStr.Printf("%04X", data);
	XRCCTRL(*this, "MC6845Data", wxTextCtrl)->SetValue(dataStr);
	XRCCTRL(*this, "MC6845EF", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/mc6845Ef", 2l));

	if (!elfConfiguration.use6845)
	{
		XRCCTRL(*this, "MC6845StartRam", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6845StartRamText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6845SizeRam", wxChoice)->Enable(false);
		XRCCTRL(*this, "MC6845SizeRamText", wxStaticText)->Enable(false);

		XRCCTRL(*this, "MC6845AddressText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6845Address", wxTextCtrl)->Enable(false);
		XRCCTRL(*this, "MC6845DataText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6845Data", wxTextCtrl)->Enable(false);

		XRCCTRL(*this, "MC6845EfText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "MC6845EF", wxSpinCtrl)->Enable(false);
	}	

	if (elfConfiguration.useS100)
	{
		XRCCTRL(*this, "MC6845StartRam", wxChoice)->Enable(true);
		XRCCTRL(*this, "MC6845StartRamText", wxStaticText)->Enable(true);
		XRCCTRL(*this, "MC6845SizeRam", wxChoice)->Enable(true);
		XRCCTRL(*this, "MC6845SizeRamText", wxStaticText)->Enable(true);
	}	

	if (elfConfiguration.useHexKeyboardEf3)
	{
		XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/HexEf", 3l));
		XRCCTRL(*this, "KeyboardInput", wxSpinCtrl)->SetValue(4);
		XRCCTRL(*this, "KeyboardInputText", wxStaticText)->Enable(true);
		XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->Enable(true);
		XRCCTRL(*this, "KeyboardEfText", wxStaticText)->Enable(true);
	}

	XRCCTRL(*this, "TapeEf", wxSpinCtrl)->SetValue(p_Main->getConfigItem(elfTypeStr_+"/TapeEf", 2l));
	if (!elfConfiguration.useTape)
	{
		XRCCTRL(*this, "TapeEf", wxSpinCtrl)->Enable(false);
		XRCCTRL(*this, "TapeEfText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "TapeQ", wxStaticText)->Enable(false);
		XRCCTRL(*this, "TapeQText", wxStaticText)->Enable(false);
	}
}

void DevicePortsDialog::onSaveButton( wxCommandEvent& WXUNUSED(event) )
{
	int addr = p_Main->get16BitValue("MC6845Address");
	if (addr == -1)  return;
	int data = p_Main->get16BitValue("MC6845Data");
	if (data == -1)  return;

	p_Main->setConfigItem(elfTypeStr_+"/mc6845Address", addr);
	p_Main->setConfigItem(elfTypeStr_+"/mc6845Data", data);

	p_Main->setConfigItem(elfTypeStr_+"/PixieInput", XRCCTRL(*this, "PixieInput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/PixieOutput", XRCCTRL(*this, "PixieOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/PixieEF", XRCCTRL(*this, "PixieEF", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/PortExtenderSelectOutput", XRCCTRL(*this, "PortExtenderSelectOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/PortExtenderWriteOutput", XRCCTRL(*this, "PortExtenderWriteOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/PortExtenderInput", XRCCTRL(*this, "PortExtenderInput", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/IdeSelectOutput", XRCCTRL(*this, "IdeSelectOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/IdeWriteOutput", XRCCTRL(*this, "IdeWriteOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/IdeInput", XRCCTRL(*this, "IdeInput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/IdeTracks", XRCCTRL(*this, "IdeTracks", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/IdeHeads", XRCCTRL(*this, "IdeHeads", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/IdeSectors", XRCCTRL(*this, "IdeSectors", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/FdcSelectOutput", XRCCTRL(*this, "FdcSelectOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/FdcWriteOutput", XRCCTRL(*this, "FdcWriteOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/FdcInput", XRCCTRL(*this, "FdcInput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/FdcEf", XRCCTRL(*this, "FdcEf", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/KeyboardInput", XRCCTRL(*this, "KeyboardInput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/KeyboardEf", XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/Ps2KeyboardInput", XRCCTRL(*this, "Ps2KeyboardInput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/Ps2KeyboardOutput", XRCCTRL(*this, "Ps2KeyboardOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/Ps2KeyboardEf", XRCCTRL(*this, "Ps2KeyboardEf", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/I8275WriteCommand", XRCCTRL(*this, "I8275WriteCommand", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/I8275ReadStatus", XRCCTRL(*this, "I8275ReadStatus", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/I8275WriteParameter", XRCCTRL(*this, "I8275WriteParameter", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/I8275ReadParameter", XRCCTRL(*this, "I8275ReadParameter", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/I8275VerticalRetrace", XRCCTRL(*this, "I8275VerticalRetrace", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/PrinterOutput", XRCCTRL(*this, "PrinterOutput", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/Vt100Output", XRCCTRL(*this, "Vt100Output", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/Vt100Ef", XRCCTRL(*this, "Vt100Ef", wxSpinCtrl)->GetValue());

	if (XRCCTRL(*this, "Vt100ReverseEf", wxCheckBox)->IsChecked())
		p_Main->setConfigItem(elfTypeStr_+"/Vt100ReverseEf", 0); 
	else
		p_Main->setConfigItem(elfTypeStr_+"/Vt100ReverseEf", 1); 

	if (XRCCTRL(*this, "Vt100ReverseQ", wxCheckBox)->IsChecked())
		p_Main->setConfigItem(elfTypeStr_+"/Vt100ReverseQ", 1); 
	else
		p_Main->setConfigItem(elfTypeStr_+"/Vt100ReverseQ", 0); 

	p_Main->setConfigItem(elfTypeStr_+"/UartOut", XRCCTRL(*this, "UartOut", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/UartIn", XRCCTRL(*this, "UartIn", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/UartControl", XRCCTRL(*this, "UartControl", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/UartStatus", XRCCTRL(*this, "UartStatus", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/TmsModeHighOutput", XRCCTRL(*this, "TmsModeHighOutput", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/TmsModeLowOutput", XRCCTRL(*this, "TmsModeLowOutput", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/Led_Module_Output", XRCCTRL(*this, "Led_Module_Output", wxSpinCtrl)->GetValue());

	p_Main->setConfigItem(elfTypeStr_+"/MC6847Output", XRCCTRL(*this, "MC6847Output", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B7", XRCCTRL(*this, "MC6847-B7", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B6", XRCCTRL(*this, "MC6847-B6", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B5", XRCCTRL(*this, "MC6847-B5", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B4", XRCCTRL(*this, "MC6847-B4", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B3", XRCCTRL(*this, "MC6847-B3", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B2", XRCCTRL(*this, "MC6847-B2", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B1", XRCCTRL(*this, "MC6847-B1", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-B0", XRCCTRL(*this, "MC6847-B0", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-DD7", XRCCTRL(*this, "MC6847-DD7", wxChoice)->GetCurrentSelection());
	p_Main->setConfigItem(elfTypeStr_+"/MC6847-DD6", XRCCTRL(*this, "MC6847-DD6", wxChoice)->GetCurrentSelection());

	int sel = XRCCTRL(*this, "MC6847StartRam", wxChoice)->GetCurrentSelection();
	int start = (sel * 0x1000) + 0x8000;
	p_Main->setConfigItem(elfTypeStr_+"/MC6847StartRam", start);

	int size = 0x3ff;
	sel = XRCCTRL(*this, "MC6847SizeRam", wxChoice)->GetCurrentSelection();
	switch (sel)
	{
		case 0:
			size = 0x3ff;
		break;
		case 1:
			size = 0x5ff;
		break;
		case 2:
			size = 0x7ff;
		break;
		case 3:
			size = 0xbff;
		break;
		case 4:
			size = 0x17ff;
		break;
	}
	int end = start + size;
	if (end > 0xffff)  end = 0xffff;
	p_Main->setConfigItem(elfTypeStr_+"/mc6847EndRam", end);

	sel = XRCCTRL(*this, "MC6845StartRam", wxChoice)->GetCurrentSelection();
	start = sel * 0x1000;
	p_Main->setConfigItem(elfTypeStr_+"/MC6845StartRam", start);

	size = 0x3ff;
	sel = XRCCTRL(*this, "MC6845SizeRam", wxChoice)->GetCurrentSelection();
	switch (sel)
	{
		case 0:
			size = 0x3ff;
		break;
		case 1:
			size = 0x7ff;
		break;
		case 2:
			size = 0xfff;
		break;
		case 3:
			size = 0x1fff;
		break;
		case 4:
			size = 0x3fff;
		break;
	}
	end = start + size;
	if (end > 0xffff)  end = 0xffff;
	p_Main->setConfigItem(elfTypeStr_+"/mc6845EndRam", end);
	p_Main->setConfigItem(elfTypeStr_+"/mc6845Ef", XRCCTRL(*this, "MC6845EF", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/HexEf", XRCCTRL(*this, "KeyboardEf", wxSpinCtrl)->GetValue());
	p_Main->setConfigItem(elfTypeStr_+"/TapeEf", XRCCTRL(*this, "TapeEf", wxSpinCtrl)->GetValue());

	EndModal(wxID_OK);
}

void DevicePortsDialog::reset6847Item(int dontCheck, int num)
{
	if ((XRCCTRL(*this, "MC6847-B7", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 10))
		XRCCTRL(*this, "MC6847-B7", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B6", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 9))
		XRCCTRL(*this, "MC6847-B6", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B5", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 8))
		XRCCTRL(*this, "MC6847-B5", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B4", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 7))
		XRCCTRL(*this, "MC6847-B4", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B3", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 6))
		XRCCTRL(*this, "MC6847-B3", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B2", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 5))
		XRCCTRL(*this, "MC6847-B2", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B1", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 4))
		XRCCTRL(*this, "MC6847-B1", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-B0", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 3))
		XRCCTRL(*this, "MC6847-B0", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-DD7", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 2))
		XRCCTRL(*this, "MC6847-DD7", wxChoice)->SetSelection(0);
	if ((XRCCTRL(*this, "MC6847-DD6", wxChoice)->GetCurrentSelection() == num) && (dontCheck != 1))
		XRCCTRL(*this, "MC6847-DD6", wxChoice)->SetSelection(0);
}

void DevicePortsDialog::onMC6847DD6(wxCommandEvent&event)
{
	reset6847Item(1, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847DD7(wxCommandEvent&event)
{
	reset6847Item(2, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B0(wxCommandEvent&event)
{
	reset6847Item(3, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B1(wxCommandEvent&event)
{
	reset6847Item(4, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B2(wxCommandEvent&event)
{
	reset6847Item(5, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B3(wxCommandEvent&event)
{
	reset6847Item(6, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B4(wxCommandEvent&event)
{
	reset6847Item(7, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B5(wxCommandEvent&event)
{
	reset6847Item(8, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B6(wxCommandEvent&event)
{
	reset6847Item(9, event.GetSelection()); 
}

void DevicePortsDialog::onMC6847B7(wxCommandEvent&event)
{
	reset6847Item(10, event.GetSelection()); 
}
