/*
 *******************************************************************
 *** This software is copyright 2008 by Marcel van Tongeren      ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 ***                                                             ***
 *** 1802 Code based on elf emulator by Michael H Riley with     ***
 *** copyright as below                                          ***
 *******************************************************************
*/

/*
 *******************************************************************
 *** This software is copyright 2006 by Michael H Riley          ***
 *** You have permission to use, modify, copy, and distribute    ***
 *** this software so long as this copyright notice is retained. ***
 *** This software may not be used in commercial applications    ***
 *** without express written permission from the author.         ***
 *******************************************************************
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#ifdef __WXMSW__
#include "wx/msw/private.h"
#endif

#if !wxUSE_COMBOCTRL
    #error "Please set wxUSE_COMBOCTRL to 1 and rebuild the library."
#endif

#include "wx/xrc/xmlres.h"
#include "wx/textfile.h"
#include "wx/tglbtn.h"
#include "wx/listbox.h"

#include "main.h"
#include "debug.h"
#include "elfconfiguration.h"
#include "breakpoints.h"

#define CHIP8_I 10
#define CHIP8_PC 5

enum
{
	CHIP8_VX,
	CHIP8_VX_MEM,
	ASS_HEX_VALUE,
	ASS_STRING,
	ASS_REG,
	ERROR_START,
	ERROR_4BIT,
	ERROR_8BIT,
	ERROR_16BIT,
	ERROR_12BIT,
	ERROR_NS_8,
	ERROR_NS_8X,
	ERROR_NS_ETI,
	ERROR_NO_SLASH,
	ERROR_2ND_REG,
	ERROR_NO_REG0,
	ERROR_ONLY_REG0,
	ERROR_ONLY_REGB,
	ERROR_ONLY_REGC,
	ERROR_NO_VALUE_0,
	ERROR_NO_VALUE_F,
	ERROR_ONLY_VALUE_0,
	ERROR_4REG,
	ERROR_8REG,
	ERROR_SYNTAX,
	ERROR_REG,
	ERROR_DEC,
	ERROR_REG_EXP,
	ERROR_PAR,
	ERROR_IO,
	ERROR_COMMA,
	ERROR_SLASH,
	ERROR_CPU_1802,
	ERROR_CPU_1804,
	ERROR_CHIP,
	ERROR_1802,
	ERROR_COMMAND_SEP,
	ERROR_SPRITE,
	ERROR_VIDEO,
	ERROR_LD,
	ERROR_CLR,
};

wxString error_code[] =
{
	"4 bit value expected",
	"8 bit value expected",
	"16 bit value expected",
	"12 bit value expected",
	"Not supported on Chip-8",
	"Not supported on Chip-8X",
	"Not supported on ETI Chip-8",
	"Seperator not allowed",
	"Second register value incorrect",
	"Register 0 not allowed",
	"Only register 0 allowed",
	"Only register B allowed",
	"Only register C allowed",
	"Value 0 not allowed",
	"Value F not allowed",
	"Only fill with 0 allowed",
	"4 bit value or register expected",
	"8 bit value or register expected",
	"Syntax error",
	"Register value not recognized",
	"Decimal value not recognized",
	"Register value expected",
	"Too many parameters",
	"I/O value not recognized",
	"Comma expected after parameter",
	"Slash expected after parameter",
	"Not supported on CDP1802",
	"Not supported on CDP1804,",
	"Chip-8 Instruction not recognized",
	"1802 Instruction not recognized",
	"Space expected after command",
	"Incorrect sprite (SP) command",
	"Incorrect video command",
	"Incorrect load (LD) command",
	"Incorrect colour (CLR) command",
};

int opCode[] =
{
	0x74, 0x7c, 0xf4, 0xfc, 0xf2, 0xfa, // A
	0x34, 0x35, 0x36, 0x37, 0x68, 0x33, 0x3c, 0x3d, 0x3e, 0x3f, 0x3b, 0x39, 0x3a, 0x31, 0x30, 0x68, 0x32, // B
	0x68, 0x68, // C
	0x68, 0x68, 0x68, 0x68, 0x68, 0x20, 0x71, 0x68, 0x68, 0x68, 0x68, 0x68, 0x68, // D
	0x68, // E
	0x68, 0x90, 0x80, // G
	0x00, 0x10, 0x69, 0x60, // I
	0xc3, 0xcb, 0xc9, 0xca, 0xc1, 0xc0, 0xc2, 0x40, 0x68, 0xf8, 0x00, 0xf0, 0x72, 0xcf, 0xcc, 0xc8, 0xc7, 0xc5, 0xc6, 0xcd, 0xce, // L
	0x79, // M
	0x38, 0xc8, 0xc4, // N
	0xf1, 0xf9, 0x61, // O
	0xb0, 0xa0, // P
	0x7a, 0x70, 0x68, 0x68, 0x68, 0x68, // R
	0x78, 0x68, 0x68, 0x68, 0xf5, 0x75, 0x7d, 0xfd, 0xd0, 0x7b, 0xe0, 0xfe, 0x7e, 0xf6, 0x76, 0x38, 0xf7, 0x77, 0x7f, 0xff, 0x68, 0x68, 0x68, 0x68, 0x68, 0x50, 0x73, // S
	0x68, 0x68, 0xf3, 0xfb // X
};

int opCode2[] =
{
	0x30, 0x30, 0x30, 0x30, 0x30, 0x30, // A
	0x30, 0x30, 0x30, 0x30, 0x3e, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x3f, 0x30, // B
	0x0d, 0x0c, // C
	0x7c, 0x74, 0xf4, 0xfc, 0x20, 0x30, 0x30, 0x76, 0x7f, 0xf7, 0x77, 0xff, 0x01, // D
	0x09,
	0x08, 0x30, 0x30, // G
	0x30, 0x30, 0x30, 0x30, // I
	0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x06, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, // L
	0x30, // M
	0x30, 0x30, 0x30, // N
	0x30, 0x30, 0x30, // O
	0x30, 0x30, // P
	0x30, 0x30, 0xc0, 0x60, 0xb0, 0xa0, // R
	0x30, 0x80, 0x05, 0x03, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x04, 0x02, 0x90, 0x07, 0x00, 0x30, 0x30, // S
	0x0b, 0x0a, 0x30, 0x30 // X
};

int minCpuType[] =
{
	2, 2, 2, 2, 2, 2, // A
	2, 2, 2, 2, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 2, // B
	4, 4, // C
	5, 5, 5, 5, 5, 2, 2, 5, 5, 5, 5, 5, 4, // D
	4,
	4, 2, 2, // G
	2, 2, 2, 2, // I
	2, 2, 2, 2, 2, 2, 2, 2, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, // L
	2, // M
	2, 2, 2, // N
	2, 2, 2, // O
	2, 2, // P
	2, 2, 4, 4, 4, 4, // R
	2, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 4, 4, 4, 2, 2, // S
	4, 4, 2, 2 // X
};

int numberOfBytes[] =
{
	1, 2, 1, 2, 1, 2,
	2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2,
	2, 2, // C
	3, 2, 2, 3, 4, 1, 1, 2, 3, 2, 2, 3, 2, // D
	2,
	2, 1, 1,
	1, 1, 2, 1, // I
	3, 3, 3, 3, 3, 3, 3 , 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 , 1,
	1,
	1, 1, 1,
	1, 2, 2, // O
	1, 1,
	1, 1, 4, 2, 2, 2,// R
	1, 4, 2, 2, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, // S
	2, 2, 1, 2
};

bool useRegister[] =
{
	false, false, false, false, false, false,
	false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false,
	false, false, // C
	false, false, false, false, true, true, false, false, false, false, false, false, false, // D
	false,
	false, true, true,
	false, true, false, false,
	false, false, false, false, false, false, false, true, false, false, true, false, false, false, false, false, false, false, false, false, false,
	false,
	false, false, false,
	false, false, false,
	true, true,
	false, false, true, true, true,
	false, true, true, false, false, false, false, false, false, true, false, true, false, false, false, false, false, false, false, false, false, false, false, true, false, false, true, false,
	false, false, false, false,
};

Word chip8Sprite[] =
{
	0x130,
	0x139,
	0x122,
	0x12a,
	0x13e,
	0x120,
	0x124,
	0x134,
	0x126,
	0x128,
	0x12e,
	0x118,
	0x114,
	0x11c,
	0x110,
	0x112
};

BEGIN_EVENT_TABLE(DebugWindow, GuiComx)

	EVT_CHECKBOX(XRCID("DebugMode"), Main::onDebugMode)
	EVT_TEXT_ENTER(XRCID("InputWindow"), DebugWindow::onEnter)

	EVT_BUTTON(XRCID("TraceLog"), DebugWindow::onLog)
	EVT_TOGGLEBUTTON(XRCID("TraceButton"), DebugWindow::onTrace)
	EVT_TOGGLEBUTTON(XRCID("DmaButton"), DebugWindow::onTraceDma)
	EVT_TOGGLEBUTTON(XRCID("IntButton"), DebugWindow::onTraceInt)
	EVT_TOGGLEBUTTON(XRCID("Chip8IntButton"), DebugWindow::onChip8TraceInt)
	EVT_TOGGLEBUTTON(XRCID("TraceTrapButton"), DebugWindow::onTraceTrap)

	EVT_BUTTON(XRCID("DebugInterrupt"), DebugWindow::onInt)
	EVT_BUTTON(XRCID("ClearButton"), DebugWindow::onClear)

	EVT_BUTTON(XRCID("DebugPauseButton"), DebugWindow::onPauseButton)
	EVT_BUTTON(XRCID("DebugStepButton"), DebugWindow::onStepButton)
	EVT_BUTTON(XRCID("DebugRunButton"), DebugWindow::onRunButton)
	EVT_TEXT(XRCID("DebugRunAddress"), DebugWindow::onRunAddress)
	EVT_TEXT_ENTER(XRCID("BreakPointAddress"), DebugWindow::onBreakPointSet)
	EVT_TEXT(XRCID("TregValue"), DebugWindow::onTregValue)
	EVT_TEXT(XRCID("NumberOfSteps"), DebugWindow::onNumberOfSteps)

	EVT_BUTTON(XRCID("BreakPointSet"), DebugWindow::onBreakPointSet)
	EVT_BUTTON(XRCID("TregSet"), DebugWindow::onTregSet)
	EVT_TEXT_ENTER(XRCID("TregValue"), DebugWindow::onTregSet)
	EVT_CHOICE(XRCID("TrapCommand"), DebugWindow::onTrapCommand)
	EVT_BUTTON(XRCID("TrapSet"), DebugWindow::onTrapSet)
	EVT_TEXT_ENTER(XRCID("TrapValue"), DebugWindow::onTrapSet)

	EVT_LIST_DELETE_ITEM(XRCID("BreakPointWindow"), DebugWindow::deleteBreakPoint)
	EVT_LIST_DELETE_ITEM(XRCID("TregWindow"), DebugWindow::deleteTreg)
	EVT_LIST_DELETE_ITEM(XRCID("TrapWindow"), DebugWindow::deleteTrap)
	EVT_LIST_END_LABEL_EDIT(XRCID("BreakPointWindow"), DebugWindow::editBreakPoint)
	EVT_LIST_END_LABEL_EDIT(XRCID("TregWindow"), DebugWindow::editTreg)
	EVT_LIST_END_LABEL_EDIT(XRCID("TrapWindow"), DebugWindow::editTrap)

	EVT_LIST_ITEM_SELECTED(XRCID("BreakPointWindow"), DebugWindow::selectBreakPoint)
	EVT_LIST_ITEM_DESELECTED(XRCID("BreakPointWindow"), DebugWindow::deselectBreakPoint)
	EVT_LIST_KEY_DOWN(XRCID("BreakPointWindow"), DebugWindow::keyBreakPoint)

	EVT_LIST_ITEM_SELECTED(XRCID("Chip8BreakPointWindow"), DebugWindow::selectChip8BreakPoint)
	EVT_LIST_ITEM_DESELECTED(XRCID("Chip8BreakPointWindow"), DebugWindow::deselectChip8BreakPoint)
	EVT_LIST_KEY_DOWN(XRCID("Chip8BreakPointWindow"), DebugWindow::keyChip8BreakPoint)

	EVT_LIST_ITEM_SELECTED(XRCID("TregWindow"), DebugWindow::selectTreg)
	EVT_LIST_ITEM_DESELECTED(XRCID("TregWindow"), DebugWindow::deselectTreg)
	EVT_LIST_KEY_DOWN(XRCID("TregWindow"), DebugWindow::keyTreg)

	EVT_LIST_ITEM_SELECTED(XRCID("TrapWindow"), DebugWindow::selectTrap)
	EVT_LIST_ITEM_DESELECTED(XRCID("TrapWindow"), DebugWindow::deselectTrap)
	EVT_LIST_KEY_DOWN(XRCID("TrapWindow"), DebugWindow::keyTrap)

	EVT_COMMAND_SCROLL_THUMBTRACK(XRCID("PercentageClock"), DebugWindow::onPercentageClock)
	EVT_COMMAND_SCROLL_CHANGED(XRCID("PercentageClock"), DebugWindow::onPercentageClock)
	EVT_COMMAND_SCROLL_THUMBTRACK(XRCID("Chip8PercentageClock"), DebugWindow::onChip8PercentageClock)
	EVT_COMMAND_SCROLL_CHANGED(XRCID("Chip8PercentageClock"), DebugWindow::onChip8PercentageClock)

	EVT_TEXT_ENTER(XRCID("O1"), DebugWindow::O1)
	EVT_TEXT_ENTER(XRCID("O2"), DebugWindow::O2)
	EVT_TEXT_ENTER(XRCID("O3"), DebugWindow::O3)
	EVT_TEXT_ENTER(XRCID("O4"), DebugWindow::O4)
	EVT_TEXT_ENTER(XRCID("O5"), DebugWindow::O5)
	EVT_TEXT_ENTER(XRCID("O6"), DebugWindow::O6)
	EVT_TEXT_ENTER(XRCID("O7"), DebugWindow::O7)

	EVT_TEXT_ENTER(XRCID("D"), DebugWindow::D)
	EVT_TEXT_ENTER(XRCID("P"), DebugWindow::P)
	EVT_TEXT_ENTER(XRCID("X"), DebugWindow::X)
	EVT_TEXT_ENTER(XRCID("T"), DebugWindow::T)
	EVT_TEXT_ENTER(XRCID("DF"), DebugWindow::DF)
	EVT_TEXT_ENTER(XRCID("Q"), DebugWindow::Q)
	EVT_TEXT_ENTER(XRCID("IE"), DebugWindow::IE)

	EVT_TEXT_ENTER(XRCID("R0"), DebugWindow::R0)
	EVT_TEXT_ENTER(XRCID("R1"), DebugWindow::R1)
	EVT_TEXT_ENTER(XRCID("R2"), DebugWindow::R2)
	EVT_TEXT_ENTER(XRCID("R3"), DebugWindow::R3)
	EVT_TEXT_ENTER(XRCID("R4"), DebugWindow::R4)
	EVT_TEXT_ENTER(XRCID("R5"), DebugWindow::R5)
	EVT_TEXT_ENTER(XRCID("R6"), DebugWindow::R6)
	EVT_TEXT_ENTER(XRCID("R7"), DebugWindow::R7)
	EVT_TEXT_ENTER(XRCID("R8"), DebugWindow::R8)
	EVT_TEXT_ENTER(XRCID("R9"), DebugWindow::R9)
	EVT_TEXT_ENTER(XRCID("R10"), DebugWindow::RA)
	EVT_TEXT_ENTER(XRCID("R11"), DebugWindow::RB)
	EVT_TEXT_ENTER(XRCID("R12"), DebugWindow::RC)
	EVT_TEXT_ENTER(XRCID("R13"), DebugWindow::RD)
	EVT_TEXT_ENTER(XRCID("R14"), DebugWindow::RE)
	EVT_TEXT_ENTER(XRCID("R15"), DebugWindow::RF)

	EVT_TEXT_ENTER(XRCID("V0"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V1"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V2"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V3"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V4"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V5"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V6"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V7"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V8"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("V9"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VA"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VB"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VC"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VD"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VE"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("VF"), DebugWindow::Vx)
	EVT_TEXT_ENTER(XRCID("Chip8I"), DebugWindow::chip8I)

	EVT_TEXT(XRCID("DebugDisplayPage"), DebugWindow::onDebugDisplayPage)
//	EVT_TEXT_ENTER(XRCID("DebugDisplayPage"), DebugWindow::onDebugDisplayPage)
	EVT_SPIN_UP(XRCID("DebugDisplayPageSpinButton"), DebugWindow::onDebugDisplayPageSpinUp)
	EVT_SPIN_DOWN(XRCID("DebugDisplayPageSpinButton"), DebugWindow::onDebugDisplayPageSpinDown)
	EVT_CHOICE(XRCID("DebugMemType"), DebugWindow::onDebugMemType)
	EVT_TEXT(XRCID("DebugExpansionSlot"), DebugWindow::onDebugExpansionSlot)
	EVT_TEXT(XRCID("DebugExpansionRam"), DebugWindow::onDebugExpansionRam)
	EVT_TEXT(XRCID("DebugExpansionEprom"), DebugWindow::onDebugExpansionEprom)
	EVT_TEXT(XRCID("DebugEmsPage"), DebugWindow::onDebugEmsPage)
	EVT_TEXT(XRCID("DebugPager"), DebugWindow::onDebugPager)
	EVT_TEXT(XRCID("DebugPortExtender"), DebugWindow::onDebugPortExtender)
	EVT_BUTTON(XRCID("DebugSave"), DebugWindow::onDebugSaveDump)
	EVT_BUTTON(XRCID("DebugCopy"), DebugWindow::onDebugCopy)
	EVT_BUTTON(XRCID("DebugDis"), DebugWindow::onDebugDis)
	EVT_BUTTON(XRCID("DebugDisLog"), DebugWindow::onDebugDisLog)
	EVT_TOGGLEBUTTON(XRCID("DebugDisChip8"), DebugWindow::onDebugDisChip8)
	EVT_TEXT(XRCID("DebugDisStart"), DebugWindow::onDebugDisStart)
	EVT_TEXT(XRCID("DebugDisEnd"), DebugWindow::onDebugDisEnd)
	EVT_TEXT_ENTER(XRCID("DebugDisEnd"), DebugWindow::onDebugDis)
	EVT_TEXT(XRCID("DebugAssemblerAddress"), DebugWindow::onDebugAssemblerAddress)
	EVT_TEXT(XRCID("DebugCopyStart"), DebugWindow::onDebugCopyStart)
	EVT_TEXT(XRCID("DebugCopyEnd"), DebugWindow::onDebugCopyEnd)
	EVT_TEXT(XRCID("DebugCopyTo"), DebugWindow::onDebugCopyTo)

	EVT_TEXT(XRCID("MEM00"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM01"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM02"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM03"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM04"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM05"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM06"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM07"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM08"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM09"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM0F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM10"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM11"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM12"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM13"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM14"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM15"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM16"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM17"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM18"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM19"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM1F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM20"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM21"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM22"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM23"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM24"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM25"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM26"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM27"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM28"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM29"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM2F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM30"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM31"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM32"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM33"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM34"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM35"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM36"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM37"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM38"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM39"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM3F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM40"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM41"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM42"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM43"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM44"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM45"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM46"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM47"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM48"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM49"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM4F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM50"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM51"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM52"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM53"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM54"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM55"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM56"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM57"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM58"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM59"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM5F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM60"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM61"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM62"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM63"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM64"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM65"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM66"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM67"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM68"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM69"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM6F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM70"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM71"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM72"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM73"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM74"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM75"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM76"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM77"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM78"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM79"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM7F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM80"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM81"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM82"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM83"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM84"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM85"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM86"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM87"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM88"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM89"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM8F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEM90"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM91"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM92"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM93"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM94"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM95"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM96"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM97"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM98"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM99"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9A"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9B"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9C"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9D"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9E"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEM9F"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEMA0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMA9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAD"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMAF"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEMB0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMB9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBD"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMBF"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEMC0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMC9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCD"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMCF"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEMD0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMD9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDD"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMDF"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEME0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEME9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMEA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMEB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMEC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMED"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMEE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMEF"), DebugWindow::onEditMemory)

	EVT_TEXT(XRCID("MEMF0"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF1"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF2"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF3"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF4"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF5"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF6"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF7"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF8"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMF9"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFA"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFB"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFC"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFD"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFE"), DebugWindow::onEditMemory)
	EVT_TEXT(XRCID("MEMFF"), DebugWindow::onEditMemory)

	EVT_CHECKBOX(XRCID("Chip8DebugMode"), Main::onChip8DebugMode)
	EVT_BUTTON(XRCID("Chip8PauseButton"), DebugWindow::onChip8PauseButton)
	EVT_BUTTON(XRCID("Chip8ClearButton"), DebugWindow::onChip8Clear)
	EVT_BUTTON(XRCID("Chip8StepButton"), DebugWindow::onChip8StepButton)
	EVT_TOGGLEBUTTON(XRCID("Chip8TraceButton"), DebugWindow::onChip8Trace)
	EVT_CHECKBOX(XRCID("Chip8ProtectedMode"), DebugWindow::onChip8ProtectedMode)
	EVT_BUTTON(XRCID("Chip8TraceLog"), DebugWindow::onChip8Log)

	EVT_TEXT_ENTER(XRCID("Chip8BreakPointAddress"), DebugWindow::onChip8BreakPointSet) 
	EVT_BUTTON(XRCID("Chip8BreakPointSet"), DebugWindow::onChip8BreakPointSet)
	EVT_LIST_DELETE_ITEM(XRCID("Chip8BreakPointWindow"), DebugWindow::deleteChip8BreakPoint)
	EVT_LIST_END_LABEL_EDIT(XRCID("Chip8BreakPointWindow"), DebugWindow::editChip8BreakPoint)

	EVT_CHECKBOX(XRCID(GUIPROTECTEDMODE), DebugWindow::onProtectedMode)

END_EVENT_TABLE()

DebugWindow::DebugWindow(const wxString& title, const wxPoint& pos, const wxSize& size, Mode mode, wxString dataDir)
: GuiComx(title, pos, size, mode, dataDir)
{
	numberOfBreakPoints_ = 0;
	numberOfChip8BreakPoints_ = 0;
	numberOfTraps_ = 0;
	numberOfTregs_ = 0;
	debugMode_ = false;
	chip8DebugMode_ = false;
	protectedMode_ = true;
	chip8ProtectedMode_ = true;
	traceDma_ = false;
	traceChip8Int_ = false; 
	traceInt_ = false;
	traceTrap_ = false;
	trace_ = false;
	chip8Trace_ = false;
	additionalChip8Details_ = false;
	performStep_ = false;
	steps_ = 1;
	memoryDisplay_ = CPU_MEMORY;
	debugAddress_ = 0;
	portExtender_ = 1;
	swName_ = "";
	spinning_ = false;
	showInstructionTrap_ = false;
	selectedBreakPoint_ = -1;
	selectedChip8BreakPoint_ = -1;
	selectedTreg_ = -1;
	selectedTrap_ = -1;

#if defined(__WXMSW__)
	pauseOnBitmap = wxBitmap(applicationDirectory_ + "images/pause_on.png", wxBITMAP_TYPE_PNG);
	pauseOffBitmap = wxBitmap(applicationDirectory_ + "images/pause_off.png", wxBITMAP_TYPE_PNG);

	uncheckBitmap_ = wxBitmap(applicationDirectory_ + "images/unchecked.png", wxBITMAP_TYPE_PNG);
	checkedBitmap_ = wxBitmap(applicationDirectory_ + "images/checked.png", wxBITMAP_TYPE_PNG);
	imageList_ = new wxImageList(13, 13, true, 2);
	checkedButton_ = imageList_->Add(checkedBitmap_, wxColour (255, 255, 255));
	uncheckButton_ = imageList_->Add(uncheckBitmap_, wxColour (255, 255, 255));
#endif

	for (int i=0; i<16; i++)
		lineBmp[i] = new wxBitmap(128, 16, 24);
	dcLine.SelectObject(*lineBmp[0]);
#if defined(__WXGTK__)
	dcLine.SetPen(wxPen(wxColour(0xfb, 0xf8, 0xf1)));
	dcLine.SetBrush(wxBrush(wxColour(0xfb, 0xf8, 0xf1)));
#else
	dcLine.SetPen(*wxWHITE_PEN);
	dcLine.SetBrush(*wxWHITE_BRUSH);
#endif
}

DebugWindow::~DebugWindow()
{
	for (int i=0; i<16; i++)
		delete lineBmp[i];
	delete imageList_;
}

void DebugWindow::readDebugConfig()
{
	debugDir_ = configPointer->Read("Dir/Main/Debug", dataDir_);

	if (!mode_.gui)
		return;
#if defined(__WXMSW__)
	breakPointWindowPointer->SetImageList(imageList_, wxIMAGE_LIST_SMALL);
	chip8BreakPointWindowPointer->SetImageList(imageList_, wxIMAGE_LIST_SMALL);
	tregWindowPointer->SetImageList(imageList_, wxIMAGE_LIST_SMALL);
	trapWindowPointer->SetImageList(imageList_, wxIMAGE_LIST_SMALL);
#endif
	XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->setRange(1, 4);
	XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->setRange(0, 3);
	XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->setRange(0, 4);
	XRCCTRL(*this, "DebugPortExtender", HexEdit)->setStart(1);
}

void DebugWindow::writeDebugConfig()
{
	configPointer->Write("Dir/Main/Debug", debugDir_);
}

void DebugWindow::enableDebugGuiMemory ()
{
	if (!mode_.gui)
		return;
	wxString value;
	switch (runningComputer_)
	{
		case COMX:
			XRCCTRL(*this, "DebugExpansionSlotText", wxStaticText)->Enable(p_Comx->checkExpansionRomLoaded());
			XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->Enable(p_Comx->checkExpansionRomLoaded());
			XRCCTRL(*this, "DebugExpansionRamText", wxStaticText)->Enable(p_Comx->isRamCardActive());
			XRCCTRL(*this, "DebugExpansionEpromText", wxStaticText)->Enable(p_Comx->isEpromBoardLoaded());
			XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->Enable(p_Comx->isRamCardActive());
			XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->Enable(p_Comx->isEpromBoardLoaded());
			XRCCTRL(*this, "DebugEmsPageText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugEmsPage", HexEdit)->Enable(false);
			XRCCTRL(*this, "DebugPagerText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugPager", HexEdit)->Enable(false);
			XRCCTRL(*this, "DebugPortExtender", HexEdit)->Enable(false);
		break;

		case ELF:
		case ELFII:
		case SUPERELF:
			XRCCTRL(*this, "DebugExpansionSlotText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugExpansionRamText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionEpromText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugEmsPageText", wxStaticText)->Enable(elfConfiguration[runningComputer_].useEms);
			XRCCTRL(*this, "DebugEmsPage", HexEdit)->Enable(elfConfiguration[runningComputer_].useEms);
			XRCCTRL(*this, "DebugPagerText", wxStaticText)->Enable(elfConfiguration[runningComputer_].usePager);
			XRCCTRL(*this, "DebugPager", HexEdit)->Enable(elfConfiguration[runningComputer_].usePager);
			XRCCTRL(*this, "DebugPortExtender", HexEdit)->Enable(elfConfiguration[runningComputer_].usePager);
			p_Main->updateSlotInfo();
		break;

		default:
			XRCCTRL(*this, "DebugExpansionSlotText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugExpansionRamText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionEpromText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->Enable(false);
			XRCCTRL(*this, "DebugEmsPageText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugEmsPage", HexEdit)->Enable(false);
			XRCCTRL(*this, "DebugPagerText", wxStaticText)->Enable(false);
			XRCCTRL(*this, "DebugPager", HexEdit)->Enable(false);
			XRCCTRL(*this, "DebugPortExtender", HexEdit)->Enable(false);
		break;
	}
}

void DebugWindow::enableChip8DebugGui(bool status)
{
	wxString text;

	XRCCTRL(*this,"Chip8PercentageClock", wxSlider)->Enable(status);
	XRCCTRL(*this,"Chip8PercentageClockText", wxStaticText)->Enable(status);
	XRCCTRL(*this,"Chip8TraceLog", wxButton)->Enable(status);
	XRCCTRL(*this,"Chip8PauseButton", wxBitmapButton)->Enable(status);
	XRCCTRL(*this,"Chip8StepButton", wxBitmapButton)->Enable(false);
	for (int i=0; i<16; i++)
	{
		chip8varTextPointer[i]->Enable(status&&!chip8ProtectedMode_);
		text.Printf("V%01XText",i);
		XRCCTRL(*this,text, wxStaticText)->Enable(status);
	}
	XRCCTRL(*this,"Chip8PCText", wxStaticText)->Enable(status);
	iTextPointer->Enable(status&&!chip8ProtectedMode_);
	XRCCTRL(*this,"Chip8IText", wxStaticText)->Enable(status);
	XRCCTRL(*this,"Chip8ProtectedMode", wxCheckBox)->Enable(status);
	XRCCTRL(*this,"DebugDisChip8", wxToggleButton)->Enable(status);

	if (computerRunning_)
		setChip8PauseState();
	else
		XRCCTRL(*this, "Chip8PauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);
}

void DebugWindow::enableDebugGui(bool status)
{
	wxString text;

	for (int i=0; i<16; i++)
	{
		registerTextPointer[i]->Enable(status&&!protectedMode_);
		text.Printf("R%dText",i);
		XRCCTRL(*this,text, wxStaticText)->Enable(status);
	}
	for (int i=1; i<8; i++)
	{
		text.Printf("I%dText",i);
		XRCCTRL(*this,text, wxStaticText)->Enable(status);
		text.Printf("O%d",i);
		outTextPointer[i]->Enable(status&&!protectedMode_);
		text.Printf("O%dText",i);
		XRCCTRL(*this,text, wxStaticText)->Enable(status);
	}
	for (int i=1; i<5; i++)
	{
		text.Printf("EF%dText",i);
		XRCCTRL(*this,text, wxStaticText)->Enable(status);
	}
	dfTextPointer->Enable(status&&!protectedMode_);
	XRCCTRL(*this,"DFText", wxStaticText)->Enable(status);
	qTextPointer->Enable(status&&!protectedMode_);
	XRCCTRL(*this,"QText", wxStaticText)->Enable(status);
	ieTextPointer->Enable(status&&!protectedMode_);
	XRCCTRL(*this,"IEText", wxStaticText)->Enable(status);
	dTextPointer->Enable(status&&!protectedMode_);
	pTextPointer->Enable(status&&!protectedMode_);
	xTextPointer->Enable(status&&!protectedMode_);
	tTextPointer->Enable(status&&!protectedMode_);
	XRCCTRL(*this,"InputWindow", wxTextCtrl)->Enable(status);
	XRCCTRL(*this,"DebugAssemblerAddress", wxTextCtrl)->Enable(status);
	XRCCTRL(*this,"ProtectedMode", wxCheckBox)->Enable(status);
	XRCCTRL(*this,"DebugRunButton", wxButton)->Enable(status);
	XRCCTRL(*this,"DebugInterrupt", wxButton)->Enable(status);
	XRCCTRL(*this,"DebugDis", wxButton)->Enable(status);
	XRCCTRL(*this,"DebugDisLog", wxButton)->Enable(status);
	XRCCTRL(*this,"DebugCopy", wxButton)->Enable(status);
	XRCCTRL(*this, "DebugSave", wxButton)->Enable(status);
	XRCCTRL(*this,"DebugPauseButton", wxBitmapButton)->Enable(status);
	XRCCTRL(*this,"DebugStepButton", wxBitmapButton)->Enable(false);

	if (computerRunning_)
		setPauseState();
	else
		XRCCTRL(*this, "DebugPauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);

	XRCCTRL(*this, "DebugDisplayPage", wxTextCtrl)->Enable(memoryDisplay_ != CPU_TYPE);

	if (!status)
	{
		XRCCTRL(*this, "DebugPortExtender", HexEdit)->Enable(false);
		XRCCTRL(*this, "DebugExpansionSlotText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->Enable(false);
		XRCCTRL(*this, "DebugExpansionRamText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->Enable(false);
		XRCCTRL(*this, "DebugExpansionEpromText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->Enable(false);
		XRCCTRL(*this, "DebugEmsPageText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "DebugEmsPage", HexEdit)->Enable(false);
		XRCCTRL(*this, "DebugPagerText", wxStaticText)->Enable(false);
		XRCCTRL(*this, "DebugPager", HexEdit)->Enable(false);
		XRCCTRL(*this, "DebugSave", wxButton)->Enable(false);
	}
}

void DebugWindow::cycleChip8Debug()
{
	Byte programCounter = p_Computer->getProgramCounter();
	Word programCounterAddress = p_Computer->getScratchpadRegister(programCounter);

	if (selectedComputer_ == DEBUGGER && debuggerChoice_ == CHIP8TAB)
	{
		if (programCounterAddress == p_Computer->getChip8MainLoop())
			updateChip8Window();
	}

	Word chip8PC = p_Computer->getScratchpadRegister(CHIP8_PC);

	if (chip8DebugMode_)
	{
		if (chip8Steps_ >= 0)
		{
			if (programCounterAddress == p_Computer->getChip8MainLoop())
			{
				if (additionalChip8Details_)
					chip8ShowInstruction(chip8Disassemble(chip8PC-2, true, false));
				if (chip8Steps_ == 1)
				{
					p_Computer->setSteps(0);
					chip8Steps_--;
					setChip8PauseState();
				}
				if (chip8Steps_ != 0)
				{
					if (chip8BreakPointCheck())  return;
					if (chip8Trace_)
						chip8Trace(chip8PC);
					chip8Steps_--;
				}
			}
		}
		else
		{
			if (programCounterAddress == p_Computer->getChip8MainLoop())
			{
				if (additionalChip8Details_)
					chip8ShowInstruction(chip8Disassemble(chip8PC-2, true, false));
				if (chip8BreakPointCheck())  return;
				if (chip8Trace_)
					chip8Trace(chip8PC);
			}
		}
	}
}

void DebugWindow::cycleSt2Debug()
{
	Byte programCounter = p_Computer->getProgramCounter();
	Word programCounterAddress = p_Computer->getScratchpadRegister(programCounter);

	if (selectedComputer_ == DEBUGGER && debuggerChoice_ == CHIP8TAB)
	{
		if (programCounterAddress == p_Computer->getChip8MainLoop())
			updateChip8Window();
	}

	Word chip8PC = p_Computer->getScratchpadRegister(CHIP8_PC);

	if (chip8DebugMode_)
	{
		if (chip8Steps_ >= 0)
		{
			if (programCounterAddress == p_Computer->getChip8MainLoop())
			{
				if (additionalChip8Details_)
					chip8ShowInstruction(chip8Disassemble(chip8PC-2, true, false));
				if (chip8Steps_ == 1)
				{
					p_Computer->setSteps(0);
					chip8Steps_--;
					setChip8PauseState();
				}
				if (chip8Steps_ != 0)
				{
					if (chip8BreakPointCheck())  return;
					if (chip8Trace_)
						st2Trace(chip8PC);
					chip8Steps_--;
				}
			}
		}
		else
		{
			if (programCounterAddress == p_Computer->getChip8MainLoop())
			{
				if (additionalChip8Details_)
					chip8ShowInstruction(chip8Disassemble(chip8PC-2, true, false));
				if (chip8BreakPointCheck())  return;
				if (chip8Trace_)
					st2Trace(chip8PC);
			}
		}
	}
}

bool DebugWindow::chip8BreakPointCheck()
{
	Word chip8PC = p_Computer->getScratchpadRegister(CHIP8_PC);
	if (chip8Steps_ != 0 && numberOfChip8BreakPoints_ > 0 && !performChip8Step_)
	{
		for (int i=0; i<numberOfChip8BreakPoints_; i++)
		{
			if (chip8BreakPoints_[i] == chip8PC && chip8BreakPointsSelected_[i])
			{
				chip8ShowInstruction("Hit Breakpoint");
				p_Computer->setSteps(0);
				chip8Steps_ = 0;
				setChip8PauseState();
				return true;
			}
		}
	}
	performChip8Step_ = false;
	return false;
}

void DebugWindow::showInstructionTrace()
{
	if (showInstructionTrap_)
	{
		if (traceTrap_ && !trace_ && steps_ != 0)
		{
			showInstructionTrap_ = false;
			debugTrace(cdp1802disassemble(&showInstructionTrapAddress_, true, false));
		}
	}
}

void DebugWindow::cycleDebug()
{
	wxString printBuffer, printBuffer2;
	bool trapFound = false;

	Byte programCounter = p_Computer->getProgramCounter();
	Word programCounterAddress = p_Computer->getScratchpadRegister(programCounter);
	Byte mask;
	Byte instruction;

	if (percentageClock_ != 1)
		if (selectedComputer_ == DEBUGGER && debuggerChoice_ == TRACETAB)
			p_Main->updateWindow();

	if (p_Computer->getSteps() != 0 && numberOfBreakPoints_ > 0)
	{
		for (int i=0; i<numberOfBreakPoints_; i++)
		{
			if (breakPoints_[i] == programCounterAddress && breakPointsSelected_[i])
			{
				p_Computer->setSteps(0);
				debugTrace("Hit Breakpoint");
				setPauseState();
			}
		}
	}
	if (p_Computer->getSteps() != 0 && numberOfTraps_ > 0)
	{
		for (int i=0; i<numberOfTraps_; i++)
		{
			mask = traps_[i][5];
			instruction = p_Computer->readMem(programCounterAddress);
			if (instruction == 0x60)
				mask = 0xff;
			if (instruction == 0x68)
			{
				if ((traps_[i][0] == 1 && traps_[i][1] == instruction && trapsSelected_[i]) ||
					(traps_[i][0] == 2 && traps_[i][1] == instruction &&
										 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) && trapsSelected_[i]) ||
					(traps_[i][0] == 3 && traps_[i][1] == instruction &&
										 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) &&
										 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) && trapsSelected_[i]) ||
					(traps_[i][0] == 4 && traps_[i][1] == instruction &&
										 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) &&
										 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) &&
										 traps_[i][4] == p_Computer->readMem(programCounterAddress+3) && trapsSelected_[i]))
				{
					trapFound = true;
				}
			}
			else
			{
				if ((traps_[i][0] == 1 && (traps_[i][1]&mask) == (instruction&mask) && trapsSelected_[i]) ||
					(traps_[i][0] == 2 && traps_[i][1] == instruction &&
										 traps_[i][2] == p_Computer->readMem(programCounterAddress+1) && trapsSelected_[i]) ||
					(traps_[i][0] == 3 && traps_[i][1] == instruction &&
										 traps_[i][2] == p_Computer->readMem(programCounterAddress+1) &&
										 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) && trapsSelected_[i]))
				{
					trapFound = true;
				}
			}
			if (trapFound)
			{
				if (traceTrap_)
				{
					if (!trace_)
					{
						showInstructionTrap_ = true;
						showInstructionTrapAddress_ = programCounterAddress;
					}
				}
				else
				{
					p_Computer->setSteps(0);
					debugTrace("Instruction Trap");
					setPauseState();
				}
			}
		}
	}
	if (p_Computer->getSteps() != 0 && numberOfTregs_ > 0)
	{
		int j = -1;
		printBuffer2 = "Register Trap: ";
		for (int i=0; i<numberOfTregs_; i++)
		{
			if (tregsSelected_[i])
			{
				if (tregs_[i][0] == TREG_D && tregs_[i][1] == p_Computer->getAccumulator())
				{
					printBuffer.Printf("D=%02X", p_Computer->getAccumulator());
					j = i;
				}
				if (tregs_[i][0] == TREG_DF && tregs_[i][1] == p_Computer->getDataFlag())
				{
					printBuffer.Printf("DF=%X", p_Computer->getDataFlag());
					j = i;
				}
				if (tregs_[i][0] == TREG_P && tregs_[i][1] == programCounter)
				{
					printBuffer.Printf("P=%X", programCounter);
					j = i;
				}
				if (tregs_[i][0] == TREG_X && tregs_[i][1] == p_Computer->getDataPointer())
				{
					printBuffer.Printf("X=%X", p_Computer->getDataPointer());
					j = i;
				}
				if (tregs_[i][0] == TREG_T && tregs_[i][1] == p_Computer->getRegisterT())
				{
					printBuffer.Printf("X=%02X",p_Computer->getRegisterT());
					j = i;
				}
				if (tregs_[i][0] == TREG_Q && tregs_[i][1] == p_Computer->getFlipFlopQ())
				{
					printBuffer.Printf("Q=%X", p_Computer->getFlipFlopQ());
					j = i;
				}
				if (tregs_[i][0] >= TREG_R0 && tregs_[i][0] <= TREG_RF &&
					tregs_[i][1] == p_Computer->getScratchpadRegister(tregs_[i][0]-TREG_R0))
				{
					printBuffer.Printf("R%X=%04X",tregs_[i][0]-TREG_R0,tregs_[i][1]);
					j = i;
				}
			}
		}
		if (j >= 0)
		{
			p_Computer->setSteps(0);
			printBuffer2.operator += (printBuffer);
			debugTrace(printBuffer2);
			setPauseState();
		}
	}

	if (p_Computer->getSteps() > 0)
	{
		p_Computer->setSteps(p_Computer->getSteps()-1);

		if (p_Computer->getSteps() == 0)
			setPauseState();
	}

	if (performStep_)
	{
		if (traceTrap_)
		{
			if (!trace_)
			{
				for (int i=0; i<numberOfTraps_; i++)
				{
					mask = traps_[i][5];
					instruction = p_Computer->readMem(programCounterAddress);
					if (instruction == 0x60)
						mask = 0xff;
					if (instruction == 0x68)
					{
						if ((traps_[i][0] == 1 && traps_[i][1] == instruction && trapsSelected_[i]) ||
							(traps_[i][0] == 2 && traps_[i][1] == instruction &&
												 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) && trapsSelected_[i]) ||
							(traps_[i][0] == 3 && traps_[i][1] == instruction &&
												 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) &&
												 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) && trapsSelected_[i]) ||
							(traps_[i][0] == 4 && traps_[i][1] == instruction &&
												 (traps_[i][2]&mask) == (p_Computer->readMem(programCounterAddress+1)&mask) &&
												 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) &&
												 traps_[i][4] == p_Computer->readMem(programCounterAddress+3) && trapsSelected_[i]))
						{
							trapFound = true;
						}
					}
					else
					{
						if ((traps_[i][0] == 1 && (traps_[i][1]&mask) == (instruction&mask) && trapsSelected_[i]) ||
							(traps_[i][0] == 2 && traps_[i][1] == instruction &&
												 traps_[i][2] == p_Computer->readMem(programCounterAddress+1) && trapsSelected_[i]) ||
							(traps_[i][0] == 3 && traps_[i][1] == instruction &&
												 traps_[i][2] == p_Computer->readMem(programCounterAddress+1) &&
												 traps_[i][3] == p_Computer->readMem(programCounterAddress+2) && trapsSelected_[i]))
						{
							trapFound = true;
						}
					}
					if (trapFound)
					{
						showInstructionTrap_ = true;
						showInstructionTrapAddress_ = programCounterAddress;
					}
				}
			}
		}
		p_Computer->setSteps(steps_);
		performStep_ = false;
		setPauseState();
	}
}

void DebugWindow::resetDisplay()
{
	for (int i=0; i<16; i++) lastR_[i] = p_Computer->getScratchpadRegister(i)+1;
	for (int i=1; i<8; i++) lastOut_[i] = p_Computer->getOutValue(i) + 1;
	for (int i=1; i<8; i++) lastIn_[i] = p_Computer->getInValue(i) + 1;
	lastD_ = p_Computer->getAccumulator() + 1;
	lastP_ = p_Computer->getProgramCounter() + 1;
	lastX = p_Computer->getDataPointer() + 1;
	lastT_ = p_Computer->getRegisterT() + 1;
	lastDf_ = p_Computer->getDataFlag() ^ 1;
	lastQ_ = p_Computer->getFlipFlopQ() ^ 1;
	lastIe_ = p_Computer->getInterruptEnable() ^ 1;
	Byte cpuFlag = p_Computer->getEfFlags();
	lastEf1_ = (cpuFlag & 1) ^ 0x1;
	lastEf2_ = (cpuFlag & 2) ^ 0x2;
	lastEf3_ = (cpuFlag & 4) ^ 0x4;
	lastEf4_ = (cpuFlag & 8) ^ 0x8;
	updateWindow();
}

void DebugWindow::updateChip8Window()
{
	wxString buffer;
	Word scratchpadRegister;
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiEnter();
#endif
	scratchpadRegister = p_Computer->getScratchpadRegister(CHIP8_PC);
	if (scratchpadRegister != lastPC_)
	{
		buffer.Printf("%03X", scratchpadRegister&0xfff);
		pcTextPointer->ChangeValue(buffer);
		lastPC_ = scratchpadRegister;
	}
	scratchpadRegister = p_Computer->getScratchpadRegister(CHIP8_I);
	if (scratchpadRegister != lastI_)
	{
		buffer.Printf("%03X", scratchpadRegister&0xfff);
		iTextPointer->ChangeValue(buffer);
		lastI_ = scratchpadRegister;
	}
	p_Computer->showChip8Registers();
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiLeave();
#endif
}

void DebugWindow::updateWindow()
{
	wxString buffer;
	Word scratchpadRegister;
	Byte cpucpuRegister;
	Byte cpuFlag;

#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiEnter();
#endif
	for (int i=0; i<16; i++)
	{
		scratchpadRegister = p_Computer->getScratchpadRegister(i);
		if (scratchpadRegister != lastR_[i])
		{
			buffer.Printf("%04X", scratchpadRegister);
			registerTextPointer[i]->ChangeValue(buffer);
			lastR_[i] = scratchpadRegister;
		}
	}

	for (int i=1; i<8; i++)
	{
		if (p_Computer->getOutValue(i) != lastOut_[i])
		{
			switch (runningComputer_)
			{
				case COMX:
				case CIDELSA:
				case PECOM:
					if (i>3)
						buffer.Printf("%04X",p_Computer->getOutValue(i));
					else
						buffer.Printf("%02X",p_Computer->getOutValue(i));
				break;

				case TMC600:
					if (i==5 && (p_Computer->getOutValue(7) != 0x20) && (p_Computer->getOutValue(7) != 0x30))
						buffer.Printf("%04X",p_Computer->getOutValue(i));
					else
						buffer.Printf("%02X",p_Computer->getOutValue(i));
				break;

				default:
					buffer.Printf("%02X",p_Computer->getOutValue(i));
				break;
			}
			outTextPointer[i]->ChangeValue(buffer);
			lastOut_[i] = p_Computer->getOutValue(i);
		}
		if (p_Computer->getInValue(i) != lastIn_[i])
		{
			buffer.Printf("%02X",p_Computer->getInValue(i));
			inTextPointer[i]->ChangeValue(buffer);
			lastIn_[i] = p_Computer->getInValue(i);
		}
	}

	cpucpuRegister = p_Computer->getAccumulator();
 	if (cpucpuRegister != lastD_)
	{
		buffer.Printf("%02X", cpucpuRegister);
		dTextPointer->ChangeValue(buffer);
		lastD_ = cpucpuRegister;
	}
	cpucpuRegister = p_Computer->getProgramCounter();
	if (cpucpuRegister != lastP_)
	{
		buffer.Printf("%01X", cpucpuRegister);
		pTextPointer->ChangeValue(buffer);
		lastP_ = cpucpuRegister;
	}
	cpucpuRegister = p_Computer->getDataPointer();
	if (cpucpuRegister != lastX)
	{
		buffer.Printf("%01X", cpucpuRegister);
		xTextPointer->ChangeValue(buffer);
		lastX = cpucpuRegister;
	}
	cpucpuRegister = p_Computer->getRegisterT();
	if (cpucpuRegister != lastT_)
	{
		buffer.Printf("%02X", cpucpuRegister);
		tTextPointer->ChangeValue(buffer);
		lastT_ = cpucpuRegister;
	}

	cpuFlag = p_Computer->getDataFlag();
	if (cpuFlag != lastDf_)
	{
		buffer.Printf("%01X", cpuFlag);
		dfTextPointer->ChangeValue(buffer);
		lastDf_ = cpuFlag;
	}
	cpuFlag = p_Computer->getFlipFlopQ();
	if (cpuFlag != lastQ_)
	{
		buffer.Printf("%01X", cpuFlag);
		qTextPointer->ChangeValue(buffer);
		lastQ_ = cpuFlag;
	}
	cpuFlag = p_Computer->getInterruptEnable();
	if (cpuFlag != lastIe_)
	{
		buffer.Printf("%01X", cpuFlag);
		ieTextPointer->ChangeValue(buffer);
		lastIe_ = cpuFlag;
	}
	cpuFlag = p_Computer->getEfFlags();
	if ((cpuFlag & 1) != lastEf1_)
	{
		buffer.Printf("%01X", cpuFlag & 1);
		efTextPointer[1]->ChangeValue(buffer);
		lastEf1_ = cpuFlag & 1;
	}
	if ((cpuFlag & 2) != lastEf2_)
	{
		buffer.Printf("%01X", (cpuFlag >> 1) & 1);
		efTextPointer[2]->ChangeValue(buffer);
		lastEf2_ = cpuFlag & 2;
	}
	if ((cpuFlag & 4) != lastEf3_)
	{
		buffer.Printf("%01X", (cpuFlag >> 2) & 1);
		efTextPointer[3]->ChangeValue(buffer);
		lastEf3_ = cpuFlag & 4;
	}
	if ((cpuFlag & 8) != lastEf4_)
	{
		buffer.Printf("%01X", (cpuFlag >> 3) & 1);
		efTextPointer[4]->ChangeValue(buffer);
		lastEf4_ = cpuFlag & 8;
	}

#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiLeave();
#endif
}

void DebugWindow::debugTrace(wxString buffer)
{
	if (!debugMode_)  return;
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiEnter();
#endif
	traceWindowPointer->AppendText(buffer);
	traceWindowPointer->AppendText("\n");
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiLeave();
#endif
}

void DebugWindow::chip8DebugTrace(wxString buffer)
{
	if (!chip8DebugMode_)  return;
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiEnter();
#endif
	chip8TraceWindowPointer->AppendText(buffer);
	chip8TraceWindowPointer->AppendText("\n");
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiLeave();
#endif
}

void DebugWindow::assemblerDisplay(wxString buffer)
{
	assemblerWindowPointer->AppendText(buffer);
	assemblerWindowPointer->AppendText("\n");
}

void DebugWindow::disassemblerDisplay(wxString buffer)
{
	disassemblerWindowPointer->AppendText(buffer);
	disassemblerWindowPointer->AppendText("\n");
}

void DebugWindow::deleteBreakPoint(wxListEvent&event)
{
	long selectedItem = event.GetIndex();

	if (selectedItem > -1)
	{
		for (int i=selectedItem; i<numberOfBreakPoints_-1; i++)
			breakPoints_[i] = breakPoints_[i+1];
		numberOfBreakPoints_--;
		SetDebugMode();
	}
}

void DebugWindow::deleteTrap(wxListEvent&event)
{
	long selectedItem = event.GetIndex();

	if (selectedItem > -1)
	{
		for (int j=selectedItem; j<numberOfTraps_-1; j++)
		{
			traps_[j][0] = traps_[j+1][0];
			traps_[j][1] = traps_[j+1][1];
			traps_[j][2] = traps_[j+1][2];
			traps_[j][3] = traps_[j+1][3];
			traps_[j][4] = traps_[j+1][4];
			traps_[j][5] = traps_[j+1][5];
		}
		numberOfTraps_--;
		SetDebugMode();
	}
}

void DebugWindow::deleteTreg(wxListEvent&event)
{
	long selectedItem = event.GetIndex();

	if (selectedItem > -1)
	{
		for (int j=selectedItem; j<numberOfTregs_-1; j++)
		{
			tregs_[j][0] = tregs_[j+1][0];
			tregs_[j][1] = tregs_[j+1][1];
		}
		numberOfTregs_--;
		SetDebugMode();
	}
}

void DebugWindow::editBreakPoint(wxListEvent&event)
{
	wxString bpStr = event.GetText();
	long selectedItem = event.GetIndex();

	if (bpStr == "")
	{
		breakPointWindowPointer->DeleteItem(selectedItem);
		event.Veto();
		return;
	}

	wxString strValue = extractWord (&bpStr);
	long value;

	if (!strValue.ToLong(&value, 16))
	{
		(void)wxMessageBox( "Please specify value in hexadecimal\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}

	if (value > 0xffff)
	{
		(void)wxMessageBox( "Please specify value of 16 bit max\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}

	breakPoints_[selectedItem] = value;
}

void DebugWindow::editTreg(wxListEvent&event)
{
	wxString tregStr = event.GetText();
	long selectedItem = event.GetIndex();

	if (tregStr == "")
	{
		tregWindowPointer->DeleteItem(selectedItem);
		event.Veto();
		return;
	}

	wxString registerStr = extractWord (&tregStr);
	int registerValue = getRegister (registerStr);

	if (registerValue != TREG_FAULT)
	{
		wxString strValue = extractWord (&tregStr);
		long value;

		if (!strValue.ToLong(&value, 16))
		{
			(void)wxMessageBox( "Please specify value in hexadecimal\n",
										"Emma 02", wxICON_ERROR | wxOK );
			event.Veto();
			return;
		}

		if ((registerValue >= TREG_R0) && (registerValue <= TREG_RF))
		{
			if (value > 0xffff)
			{
				(void)wxMessageBox( "Please specify value of 16 bit max\n",
											"Emma 02", wxICON_ERROR | wxOK );
				event.Veto();
				return;
			}
		}
		else
		{
			if (registerValue <= TREG_T)
			{
				if (value > 0xff)
				{
					(void)wxMessageBox( "Please specify value of 8 bit max\n",
												"Emma 02", wxICON_ERROR | wxOK );
					event.Veto();
					return;
				}
			}
			else
			{
				if (value > 1)
				{
					(void)wxMessageBox( "Please specify value of 0 or 1\n",
												"Emma 02", wxICON_ERROR | wxOK );
					event.Veto();
					return;
				}
			}
		}

		tregs_[selectedItem][0] = registerValue;
		tregs_[selectedItem][1] = value;
	}
	else
	{
		(void)wxMessageBox( "Please specify R0 to RF, D, P, X, T, DF or Q\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}
}

void DebugWindow::editTrap(wxListEvent&event)
{
	wxString trapStr = event.GetText();
	trapStr = trapStr.MakeUpper();
	wxString error;
	long selectedItem = event.GetIndex();

	if (trapStr == "")
	{
		trapWindowPointer->DeleteItem(selectedItem);
		event.Veto();
		return;
	}

	Word count;
	Byte b1, b2, b3, b4, b5;

	count = assemble(&trapStr, &b1, &b2, &b3, &b4, &b5, true);
	if (count > 0 && count < 5)
	{
		traps_[selectedItem][0] = count;
		traps_[selectedItem][1] = b1;
		traps_[selectedItem][2] = b2;
		traps_[selectedItem][3] = b3;
		traps_[selectedItem][4] = b4;
		traps_[selectedItem][5] = b5; 
	}
	else
	{
		(void)wxMessageBox( error_code[count-ERROR_START-1]+"\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}
}

void DebugWindow::switchBreakPoint(int item)
{
	breakPointsSelected_[item] = !breakPointsSelected_[item] ;
	if (breakPointsSelected_[item])
		breakPointWindowPointer->SetItemImage(item, checkedButton_, -1);
	else
		breakPointWindowPointer->SetItemImage(item, uncheckButton_, -1);
}

void DebugWindow::selectBreakPoint(wxListEvent&event)
{
	selectedBreakPoint_ = event.GetIndex();
}

void DebugWindow::deselectBreakPoint(wxListEvent&WXUNUSED(event))
{
	selectedBreakPoint_ = -1;
}

void DebugWindow::keyBreakPoint(wxListEvent&event)
{
	const int keyCode = event.GetKeyCode();
	if (keyCode == WXK_DELETE && selectedBreakPoint_ != -1)
  		XRCCTRL(*this,"BreakPointWindow", wxListCtrl)->DeleteItem(selectedBreakPoint_);
}

void DebugWindow::switchChip8BreakPoint(int item)
{
	chip8BreakPointsSelected_[item] = !chip8BreakPointsSelected_[item] ;
	if (chip8BreakPointsSelected_[item])
		chip8BreakPointWindowPointer->SetItemImage(item, checkedButton_, -1);
	else
		chip8BreakPointWindowPointer->SetItemImage(item, uncheckButton_, -1);
}

void DebugWindow::selectChip8BreakPoint(wxListEvent&event)
{
	selectedChip8BreakPoint_ = event.GetIndex();
}

void DebugWindow::deselectChip8BreakPoint(wxListEvent&WXUNUSED(event))
{
	selectedChip8BreakPoint_ = -1;
}

void DebugWindow::keyChip8BreakPoint(wxListEvent&event)
{
	const int keyCode = event.GetKeyCode();
	if (keyCode == WXK_DELETE && selectedChip8BreakPoint_ != -1)
  		XRCCTRL(*this,"Chip8BreakPointWindow", wxListCtrl)->DeleteItem(selectedChip8BreakPoint_);
}

void DebugWindow::switchTreg(int item)
{
	tregsSelected_[item] = !tregsSelected_[item] ;
	if (tregsSelected_[item])
		tregWindowPointer->SetItemImage(item, checkedButton_, -1);
	else
		tregWindowPointer->SetItemImage(item, uncheckButton_, -1);
}

void DebugWindow::selectTreg(wxListEvent&event)
{
	selectedTreg_ = event.GetIndex();
}

void DebugWindow::deselectTreg(wxListEvent&WXUNUSED(event))
{
	selectedTreg_ = -1;
}

void DebugWindow::keyTreg(wxListEvent&event)
{
	const int keyCode = event.GetKeyCode();
	if (keyCode == WXK_DELETE && selectedTreg_ != -1)
  		XRCCTRL(*this,"TregWindow", wxListCtrl)->DeleteItem(selectedTreg_);
}

void DebugWindow::switchTrap(int item)
{
	trapsSelected_[item] = !trapsSelected_[item] ;
	if (trapsSelected_[item])
		trapWindowPointer->SetItemImage(item, checkedButton_, -1);
	else
		trapWindowPointer->SetItemImage(item, uncheckButton_, -1);
}

void DebugWindow::selectTrap(wxListEvent&event)
{
	selectedTrap_ = event.GetIndex();
}

void DebugWindow::deselectTrap(wxListEvent&WXUNUSED(event))
{
	selectedTrap_ = -1;
}

void DebugWindow::keyTrap(wxListEvent&event)
{
	const int keyCode = event.GetKeyCode();
	if (keyCode == WXK_DELETE && selectedTrap_ != -1)
  		XRCCTRL(*this,"TrapWindow", wxListCtrl)->DeleteItem(selectedTrap_);
}

void DebugWindow::addBreakPoint()
{
	wxString printBuffer;

	printBuffer.Printf("%04X", breakPoints_[numberOfBreakPoints_]);
	breakPointWindowPointer->InsertItem(numberOfBreakPoints_, printBuffer);
	numberOfBreakPoints_++;
}

void DebugWindow::addTrap()
{
	wxString printBuffer, printBuffer2;

	int i = numberOfTraps_;
	printBuffer = "";
	printBuffer2 = "";

	int n = traps_[i][1] & 0xf, n1805;
	int inst = traps_[i][1] >> 4, i1805;
	switch(inst)
	{
		case 0x0:
			switch(n)
			{
				case 0x0:
					printBuffer.operator += ("IDL");
				break;

				default:
					if (traps_[i][5] == 0xf0)
						printBuffer.Printf("LDN  Rx");
					else
						printBuffer.Printf("LDN  R%X",n);
				break;
			}
		break;

		case 0x1:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("INC  Rx");
			else
				printBuffer.Printf("INC  R%X",n);
		break;

		case 0x2:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("DEC  Rx");
			else
				printBuffer.Printf("DEC  R%X",n);
		break;

		case 0x3:
			printBuffer2.Printf("%02X", traps_[i][2]);
			switch(n)
			{
				case 0x0:
					printBuffer.Printf("BR   ");
				break;
				case 0x1:
					printBuffer.Printf("BQ   ");
				break;
				case 0x2:
					printBuffer.Printf("BZ   ");
				break;
				case 0x3:
					printBuffer.Printf("BDF  ");
				break;
				case 0x4:
					printBuffer.Printf("B1   ");
				break;
				case 0x5:
					printBuffer.Printf("B2   ");
				break;
				case 0x6:
					printBuffer.Printf("B3   ");
				break;
				case 0x7:
					printBuffer.Printf("B4   ");
				break;
				case 0x8:
					printBuffer.operator += ("NBR"); break;
				case 0x9:
					printBuffer.Printf("BNQ  ");
				break;
				case 0xa:
					printBuffer.Printf("BNZ  ");
				break;
				case 0xb:
					printBuffer.Printf("BNF  ");
				break;
				case 0xc:
					printBuffer.Printf("BN1  ");
				break;
				case 0xd:
					printBuffer.Printf("BN2  ");
				break;
				case 0xe:
					printBuffer.Printf("BN3  ");
				break;
				case 0xf:
					printBuffer.Printf("BN4  ");
				break;
			}
			if (traps_[i][0] == 2)
				printBuffer.operator += (printBuffer2);
			else
			{
				if (printBuffer.Len() == 5)
					printBuffer.operator += ("x");
			}
		break;

		case 0x4:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("LDA  Rx");
			else
				printBuffer.Printf("LDA  R%X",n);
		break;

		case 0x5:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("STR  Rx");
			else
				printBuffer.Printf("STR  R%X",n);
		break;

		case 0x6:
			switch(n)
			{
				case 0x0:
					printBuffer.operator += ("IRX");
				break;
				case 0x1:
				case 0x2:
				case 0x3:
				case 0x4:
				case 0x5:
				case 0x6:
				case 0x7:
					if (traps_[i][5] == 0xf8)
						printBuffer.Printf("OUT  x");
					else
						printBuffer.Printf("OUT  %X",n);
				break;
				case 0x8:
					n1805 = traps_[i][2] & 0xf;
					i1805 = traps_[i][2] >> 4;
					switch(i1805)
					{
						case 0:
							switch(n1805)
							{
								case 0:
									printBuffer.Printf("STPC");
								break;
								case 1: 
									printBuffer.Printf("DTC");
								break;
								case 2: 
									printBuffer.Printf("SPM2");
								break;
								case 3: 
									printBuffer.Printf("SCM2");
								break;
								case 4: 
									printBuffer.Printf("SPM1");
								break;
								case 5: 
									printBuffer.Printf("SCM1");
								break;
								case 6: 
									printBuffer.Printf("LDC");
								break;
								case 7:
									printBuffer.Printf("STM");
								break;
								case 8: 
									printBuffer.Printf("GEC");
								break;
								case 9: 
									printBuffer.Printf("ETQ");
								break;
								case 0xa: 
									printBuffer.Printf("XIE");
								break;
								case 0xb: 
									printBuffer.Printf("XID");
								break;
								case 0xc: 
									printBuffer.Printf("CIE");
								break;
								case 0xd: 
									printBuffer.Printf("CID");
								break;
							}
						break;
						case 2:
							if (traps_[i][0] == 2)
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("DBNZ Rx,x");
								else
									printBuffer.Printf("DBNZ R%X,x", n1805);
							}
							else
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("DBNZ Rx,%04X", traps_[i][3]<<8|traps_[i][4]);
								else
									printBuffer.Printf("DBNZ R%X,%04X", n1805, traps_[i][3]<<8|traps_[i][4]);
							}
						break;
						case 3:
							printBuffer2.Printf("%02X", traps_[i][3]);
							switch(n1805)
							{
								case 0xe:
									printBuffer.Printf("BCI  ");
								break;
								case 0xf:
									printBuffer.Printf("BXI  ");
								break;
							}
							if (traps_[i][0] == 3)
								printBuffer.operator += (printBuffer2);
							else
							{
								printBuffer.operator += ("x");
							}
						break;
						case 6:
							if (traps_[i][5] == 0xf0)
								printBuffer.Printf("RLXA Rx");
							else
								printBuffer.Printf("RLXA R%X", n1805);
						break;
						case 7:
							printBuffer2.Printf("%02X", traps_[i][3]);
							switch(n1805)
							{
								case 0x4:
									printBuffer.Printf("DADC");
								break;
								case 0x6:
									printBuffer.Printf("DSAV");
								break;
								case 0x7:
									printBuffer.Printf("DSMB");
								break;
								case 0xc:
									printBuffer.Printf("DACI ");
								break;
								case 0xf:
									printBuffer.Printf("DSBI ");
								break;
							}
							if (traps_[i][0] == 3)
								printBuffer.operator += (printBuffer2);
							else
							{
								if (printBuffer.Len() == 5)
									printBuffer.operator += ("x");
							}
						break;
						case 8:
							if (traps_[i][0] == 2)
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("SCAL Rx,x");
								else
									printBuffer.Printf("SCAL R%X,x", n1805);
							}
							else
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("SCAL Rx,%04X", traps_[i][3]<<8|traps_[i][4]);
								else
									printBuffer.Printf("SCAL R%X,%04X", n1805, traps_[i][3]<<8|traps_[i][4]);
							}
						break;
						case 9:
							if (traps_[i][5] == 0xf0)
								printBuffer.Printf("SRET Rx");
							else
								printBuffer.Printf("SRET R%X", n1805);
						break;
						case 0xa: 
							if (traps_[i][5] == 0xf0)
								printBuffer.Printf("RSXD Rx");
							else
								printBuffer.Printf("RSXD R%X", n1805);
						break;
						case 0xb:
							if (traps_[i][5] == 0xf0)
								printBuffer.Printf("RNX  Rx");
							else
								printBuffer.Printf("RNX  R%X", n1805);
						break;
						case 0xc:
							if (traps_[i][0] == 2)
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("RLDI Rx,x");
								else
									printBuffer.Printf("RLDI R%X,x", n1805);
							}
							else
							{
								if (traps_[i][5] == 0xf0)
									printBuffer.Printf("RLDI Rx,%04X", traps_[i][3]<<8|traps_[i][4]);
								else
									printBuffer.Printf("RLDI R%X,%04X", n1805, traps_[i][3]<<8|traps_[i][4]);
							}
						break;
						case 0xf:
							printBuffer2.Printf("%02X", traps_[i][3]);
							switch(n1805)
							{
								case 0x4:
									printBuffer.Printf("DADD");
								break;
								case 0x7:
									printBuffer.Printf("DSM");
								break;
								case 0xc:
									printBuffer.Printf("DADI ");
								break;
								case 0xf:
									printBuffer.Printf("DSMI ");
								break;
							}
							if (traps_[i][0] == 3)
								printBuffer.operator += (printBuffer2);
							else
							{
								if (printBuffer.Len() == 5)
									printBuffer.operator += ("x");
							}
						break;

					}
				break;
				case 0x9:
				case 0xa:
				case 0xb:
				case 0xc:
				case 0xd:
				case 0xe:
				case 0xf:
					if (traps_[i][5] == 0xf8)
						printBuffer.Printf("INP  x");
					else
						printBuffer.Printf("INP  %X",n-8);
				break;
			}
		break;
		case 0x7:
			printBuffer2.Printf("%02X", traps_[i][2]);
			switch(n)
			{
				case 0x0:
					printBuffer.operator += ("RET");
				break;
				case 0x1:
					printBuffer.operator += ("DIS");
				break;
				case 0x2:
					printBuffer.operator += ("LDXA");
				break;
				case 0x3:
					printBuffer.operator += ("STXD");
				break;
				case 0x4:
					printBuffer.operator += ("ADC");
				break;
				case 0x5:
					printBuffer.operator += ("SDB");
				break;
				case 0x6:
					printBuffer.operator += ("SHRC");
				break;
				case 0x7:
					printBuffer.operator += ("SMB");
				break;
				case 0x8:
					printBuffer.operator += ("SAV");
				break;
				case 0x9:
					printBuffer.operator += ("MARK");
				break;
				case 0xa:
					printBuffer.operator += ("REQ");
				break;
				case 0xb:
					printBuffer.operator += ("SEQ");
				break;
				case 0xe:
					printBuffer.operator += ("SHLC");
				break;
				case 0xc:
					printBuffer.Printf("ADCI ");
				break;
				case 0xd:
					printBuffer.Printf("SDBI ");
				break;
				case 0xf:
					printBuffer.Printf("SMBI ");
				break;
			}
			if (traps_[i][0] == 2)
				printBuffer.operator += (printBuffer2);
			else
			{
				if (printBuffer.Len() == 5)
					printBuffer.operator += ("x");
			}
		break;
		case 0x8:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("GLO  Rx");
			else
				printBuffer.Printf("GLO  R%X",n);
		break;
		case 0x9:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("GHI  Rx");
			else
				printBuffer.Printf("GHI  R%X",n);
		break;
		case 0xa:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("PLO  Rx");
			else
				printBuffer.Printf("PLO  R%X",n);
		break;
		case 0xb:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("PHI  Rx");
			else
				printBuffer.Printf("PHI  R%X",n);
		break;
		case 0xc:
			printBuffer2.Printf("%04X", traps_[i][2]<<8|traps_[i][3]);
			switch(n)
			{
				case 0x0:
					printBuffer.Printf("LBR  "); 
				break;
				case 0x1:
					printBuffer.Printf("LBQ  "); 
				break;
				case 0x2:
					printBuffer.Printf("LBZ  "); 
				break;
				case 0x3:
					printBuffer.Printf("LBDF ");
				break;
				case 0x4:
					printBuffer.operator += ("NOP");
				break;
				case 0x5:
					printBuffer.operator += ("LSNQ");
				break;
				case 0x6:
					printBuffer.operator += ("LSNZ");
				break;
				case 0x7:
					printBuffer.operator += ("LSNF");
				break;
				case 0x8:
					printBuffer.operator += ("LSKP");
				break;
				case 0x9:
					printBuffer.Printf("LBNQ ");
				break;
				case 0xa:
					printBuffer.Printf("LBNZ "); 
				break;
				case 0xb:
					printBuffer.Printf("LBNF "); 
				break;
				case 0xc:
					printBuffer.operator += ("LSIE");
				break;
				case 0xd:
					printBuffer.operator += ("LSQ");
				break;
				case 0xe:
					printBuffer.operator += ("LSZ");
				break;
				case 0xf:
					printBuffer.operator += ("LSDF");
				break;
			}
			if (traps_[i][0] == 3)
				printBuffer.operator += (printBuffer2);
			else
			{
				if (printBuffer.Len() == 5)
					printBuffer.operator += ("x");
			}
		break;
		case 0xd:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("SEP  Rx");
			else
				printBuffer.Printf("SEP  R%X",n);
		break;
		case 0xe:
			if (traps_[i][5] == 0xf0)
				printBuffer.Printf("SEX  Rx");
			else
				printBuffer.Printf("SEX  R%X",n);
		break;
		case 0xf:
			printBuffer2.Printf("%02X", traps_[i][2]);
			switch(n)
			{
				case 0x0:
					printBuffer.operator += ("LDX");
				break;
				case 0x1:
					printBuffer.operator += ("OR");
				break;
				case 0x2:
					printBuffer.operator += ("AND");
				break;
				case 0x3:
					printBuffer.operator += ("XOR");
				break;
				case 0x4:
					printBuffer.operator += ("ADD");
				break;
				case 0x5:
					printBuffer.operator += ("SD");
				break;
				case 0x6:
					printBuffer.operator += ("SHR");
				break;
				case 0x7:
					printBuffer.operator += ("SM");
				break;
				case 0x8:
					printBuffer.Printf("LDI  ");
				break;
				case 0x9:
					printBuffer.Printf("ORI  ");
				break;
				case 0xa:
					printBuffer.Printf("ANI  ");
				break;
				case 0xb:
					printBuffer.Printf("XRI  ");
				break;
				case 0xc:
					printBuffer.Printf("ADI  ");
				break;
				case 0xd:
					printBuffer.Printf("SDI  ");
				break;
				case 0xe:
					printBuffer.operator += ("SHL");
				break;
				case 0xf:
					printBuffer.Printf("SMI  ");
				break;
			}
			if (traps_[i][0] == 2)
				printBuffer.operator += (printBuffer2);
			else
			{
				if (printBuffer.Len() == 5)
					printBuffer.operator += ("x");
			}
		break;
	}
	trapWindowPointer->InsertItem(i, printBuffer);
	numberOfTraps_++;
}

void DebugWindow::addTreg()
{
	wxString printBuffer;

	switch(tregs_[numberOfTregs_][0])
	{
		case TREG_R0: case TREG_R1: case TREG_R2: case TREG_R3: case TREG_R4: case TREG_R5: case TREG_R6: case TREG_R7: case TREG_R8: case TREG_R9:
		case TREG_RA: case TREG_RB: case TREG_RC: case TREG_RD: case TREG_RE: case TREG_RF:
			printBuffer.Printf("R%X %04X", tregs_[numberOfTregs_][0] - TREG_R0, tregs_[numberOfTregs_][1]);
		break;

		case TREG_D: printBuffer.Printf("D  %02X", tregs_[numberOfTregs_][1]);
		break;

		case TREG_DF: printBuffer.Printf("DF %X", (tregs_[numberOfTregs_][1])?1:0);
		break;

		case TREG_Q: printBuffer.Printf("Q  %X", (tregs_[numberOfTregs_][1])?1:0);
		break;

		case TREG_P: printBuffer.Printf("P  %X", tregs_[numberOfTregs_][1]);
		break;

		case TREG_X: printBuffer.Printf("X  %X", tregs_[numberOfTregs_][1]);
		break;

		case TREG_T: printBuffer.Printf("T  %02X", tregs_[numberOfTregs_][1]);
		break;
	}
	tregWindowPointer->InsertItem(numberOfTregs_, printBuffer);
	numberOfTregs_++;
}

wxString DebugWindow::extractWord(wxString *buffer)
{
	int end;
	wxString ret;

	buffer->Trim(false);

	end = buffer->Find(' ');
	if (end == -1)
	{
		ret = *buffer;
		*buffer = "";
		return ret;
	}

	ret = buffer->Mid(0, end);
	*buffer = buffer->Mid(end, buffer->Len()-end);
	return ret;
}

wxString DebugWindow::extractNextWord(wxString *buffer, wxString *seperator)
{
	size_t end;
	wxString ret;

	buffer->Trim(false);
	buffer->Trim(true);

	end = 0;
	while (buffer->Mid(end, 1) != " " && buffer->Mid(end, 1) != "/" && buffer->Mid(end, 1) != "," && end != buffer->Len())
		end++;

	if (end == buffer->Len())
	{
		ret = *buffer;
		*buffer = "";
		*seperator = " ";
		return ret;
	}

	ret = buffer->Mid(0, end);
	*seperator = buffer->Mid(end, 1);
	*buffer = buffer->Mid(end + 1, buffer->Len()- end);
	return ret;
}

void DebugWindow::disassemble(Word start, Word end)
{
	wxString printBuffer;

	while(start <= end)
	{
		printBuffer = cdp1802disassemble(&start, false, true);
		disassemblerDisplay(printBuffer);
	}
}

wxString DebugWindow::cdp1802disassemble(Word* address, bool showDetails, bool showOpcode)
{
	wxString printBufferOpcode, printBufferAssembler, printBufferTemp, printBufferAddress, printBufferDetails;
	int i, n, i1805, n1805;

	i = p_Computer->readMem(*address);

	printBufferAddress.Printf("%04X: ", *address);
	printBufferOpcode.Printf("%02X ", i);
	printBufferAssembler = "";
	printBufferDetails = "";
	n = i & 0xf;
	i >>= 4;
	*address = *address + 1;

	Byte accumulator = p_Computer->getAccumulator();

	switch(i)
	{
		case 0x0:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.operator += ("IDL");
				break;

				default:
					printBufferAssembler.Printf("LDN  R%X",n);
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
			}
		break;

		case 0x1:
			printBufferAssembler.Printf("INC  R%X",n);
			printBufferDetails.Printf("R%X=%04X", n, p_Computer->getScratchpadRegister(n));
		break;

		case 0x2:
			printBufferAssembler.Printf("DEC  R%X",n);
			printBufferDetails.Printf("R%X=%04X", n, p_Computer->getScratchpadRegister(n));
		break;

		case 0x3:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.Printf("BR   %02X", p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ", p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x1:
					printBufferAssembler.Printf("BQ   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x2:
					printBufferAssembler.Printf("BZ   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x3:
					printBufferAssembler.Printf("BDF  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x4:
					printBufferAssembler.Printf("B1   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x5:
					printBufferAssembler.Printf("B2   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x6:
					printBufferAssembler.Printf("B3   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x7:
					printBufferAssembler.Printf("B4   %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x8:
					printBufferAssembler.operator += ("NBR"); break;
				case 0x9:
					printBufferAssembler.Printf("BNQ  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xa:
					printBufferAssembler.Printf("BNZ  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xb:
					printBufferAssembler.Printf("BNF  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xc:
					printBufferAssembler.Printf("BN1  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xd:
					printBufferAssembler.Printf("BN2  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xe:
					printBufferAssembler.Printf("BN3  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0xf:
					printBufferAssembler.Printf("BN4  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
			}
		break;

		case 0x4:
			printBufferAssembler.Printf("LDA  R%X",n);
			printBufferDetails.Printf("D=M(%04X)=%02X", p_Computer->getScratchpadRegister(n)-1, accumulator);
		break;

		case 0x5:
			printBufferAssembler.Printf("STR  R%X",n);
			printBufferDetails.Printf("M(%04X)=%02X", p_Computer->getScratchpadRegister(n), accumulator);
		break;

		case 0x6:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.operator += ("IRX");
					printBufferDetails.Printf("R%X=%04X", p_Computer->getDataPointer(), p_Computer->getScratchpadRegister(p_Computer->getDataPointer()));
				break;
				case 0x1:
				case 0x2:
				case 0x3:
				case 0x4:
				case 0x5:
				case 0x6:
				case 0x7:
					printBufferAssembler.Printf("OUT  %X",n);
					switch (runningComputer_)
					{
						case COMX:
						case CIDELSA:
						case PECOM:
							if (n>3)
								printBufferDetails.Printf("[%04X]", p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1);
							else
								printBufferDetails.Printf("[%02X]", p_Computer->readMem(p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1));
						break;

						case TMC600:
							if (n==5 && (p_Computer->getOutValue(7) != 0x20) && (p_Computer->getOutValue(7) != 0x30))
								printBufferDetails.Printf("[%04X]", p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1);
							else
								printBufferDetails.Printf("[%02X]", p_Computer->readMem(p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1));
						break;

						default:
							printBufferDetails.Printf("[%02X]", p_Computer->readMem(p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1));
						break;
					}
				break;
				case 0x8:
					if (cpuType_ == CPU1804 || cpuType_ == CPU1805)
					{
						i1805 = p_Computer->readMem(*address);

						printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
						printBufferOpcode.operator += (printBufferTemp);
						*address = *address + 1;

						n1805 = i1805 & 15;
						i1805 = i1805>>4;
						switch(i1805)
						{
							case 0:
								switch(n1805)
								{
									case 0: // 1804
										printBufferAssembler.Printf("STPC");
									break;
									case 1: // 1804
										printBufferAssembler.Printf("DTC");
										printBufferDetails.Printf("CNTR=%02X", p_Computer->get1805Counter());
									break;
									case 2: // 1804
										printBufferAssembler.Printf("SPM2");
									break;
									case 3: // 1804
										printBufferAssembler.Printf("SCM2");
									break;
									case 4: // 1804
										printBufferAssembler.Printf("SPM1");
									break;
									case 5: // 1804
										printBufferAssembler.Printf("SCM1");
									break;
									case 6: // 1804
										printBufferAssembler.Printf("LDC");
										if (!p_Computer->is1805CtrRunning())
											printBufferDetails.Printf("CH/CNTR=%02X", p_Computer->get1805Ch());
										else
											printBufferDetails.Printf("CNTR=%02X", p_Computer->get1805Ch());
									break;
									case 7: // 1804
										printBufferAssembler.Printf("STM");
									break;
									case 8: // 1804
										printBufferAssembler.Printf("GEC");
										printBufferDetails.Printf("D=%02X", accumulator);
									break;
									case 9: // 1804
										printBufferAssembler.Printf("ETQ");
									break;
									case 0xa: // 1804
										printBufferAssembler.Printf("XIE");
									break;
									case 0xb: // 1804
										printBufferAssembler.Printf("XID");
									break;
									case 0xc: // 1804
										printBufferAssembler.Printf("CIE");
									break;
									case 0xd: // 1804
										printBufferAssembler.Printf("CID");
									break;
									default:
										printBufferAssembler.Printf("Illegal instruction");
									break;
								}
							break;

							case 2:
								if (cpuType_ == CPU1805)
								{
									printBufferAssembler.Printf("DBNZ R%X,%04X", n1805, p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
									printBufferTemp.Printf("%02X %02X ", p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
									printBufferOpcode.operator += (printBufferTemp);
									printBufferDetails.Printf(" R%X=%04X", n1805, p_Computer->getScratchpadRegister(n1805));
									*address = *address + 2;
								}
							break;

							case 3:
								switch(n1805)
								{
									case 0xe: // 1804
										printBufferAssembler.Printf("BCI  %02X", p_Computer->readMem(*address));
										printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
										printBufferOpcode.operator += (printBufferTemp);
										*address = *address + 1;
									break;
									case 0xf: // 1804
										printBufferAssembler.Printf("BXI  %02X", p_Computer->readMem(*address));
										printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
										printBufferOpcode.operator += (printBufferTemp);
										*address = *address + 1;
									break;
									default:
										printBufferAssembler.Printf("Illegal instruction");
									break;
								}
 							break;

							case 6: // 1804
								printBufferAssembler.Printf("RLXA R%X", n1805);
 								printBufferDetails.Printf("R%X=%04X", n1805, p_Computer->getScratchpadRegister(n1805));
 							break;

							case 7:
								if (cpuType_ == CPU1805)
								{
									switch(n1805)
									{
										case 4:
											printBufferAssembler.Printf("DADC");
											printBufferDetails.Printf("D=%02X", accumulator);
										break;
										case 6:
											printBufferAssembler.Printf("DSAV");
											printBufferDetails.Printf("D=%02X", accumulator);
										break;
										case 7:
											printBufferAssembler.Printf("DSMB");
											printBufferDetails.Printf("D=%02X", accumulator);
										break;
										case 0xc:
											printBufferAssembler.Printf("DACI %02X", p_Computer->readMem(*address));
											printBufferDetails.Printf("D=%02X", accumulator);
											printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
											printBufferOpcode.operator += (printBufferTemp);
											*address = *address + 1;
										break;
										case 0xf:
											printBufferAssembler.Printf("DSBI %02X", p_Computer->readMem(*address));
											printBufferDetails.Printf("D=%02X", accumulator);
											printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
											printBufferOpcode.operator += (printBufferTemp);
											*address = *address + 1;
										break;
										default:
											printBufferAssembler.Printf("Illegal instruction");
										break;
									}
								}
 							break;

							case 8: // 1804
								printBufferAssembler.Printf("SCAL R%X,%04X", n1805, p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
								printBufferTemp.Printf("%02X %02X ", p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
								printBufferOpcode.operator += (printBufferTemp);
								*address = *address + 2;
  							break;

							case 9: // 1804
								printBufferAssembler.Printf("SRET R%X", n1805);
								printBufferDetails.Printf("R%X(P)=%04X", p_Computer->getProgramCounter(), p_Computer->getScratchpadRegister(p_Computer->getProgramCounter()));
  							break;

							case 0xa: // 1804
								printBufferAssembler.Printf("RSXD R%X", n1805);
  							break;

							case 0xb: // 1804
								printBufferAssembler.Printf("RNX  R%X", n1805);
 								printBufferDetails.Printf("R%X=%04X", n1805, p_Computer->getScratchpadRegister(n1805));
 							break;

							case 0xc:  // 1804
								printBufferAssembler.Printf("RLDI R%X,%04X",n1805, p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
								printBufferTemp.Printf("%02X %02X ", p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
								printBufferOpcode.operator += (printBufferTemp);
								*address = *address + 2;
  							break;

							case 0xF:
								if (cpuType_ == CPU1805)
								{
									switch(n1805)
									{
										case 4:
											printBufferAssembler.Printf("DADD");
											printBufferDetails.Printf("D=%02X", accumulator);
										break;
										case 7:
											printBufferAssembler.Printf("DSM");
											printBufferDetails.Printf("D=%02X", accumulator);
										break;
										case 0xc:
											printBufferAssembler.Printf("DADI %02X", p_Computer->readMem(*address));
											printBufferDetails.Printf("D=%02X", accumulator);
											printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
											printBufferOpcode.operator += (printBufferTemp);
											*address = *address + 1;
										break;
										case 0xf:
											printBufferAssembler.Printf("DSMI %02X", p_Computer->readMem(*address));
											printBufferDetails.Printf("D=%02X", accumulator);
											printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
											printBufferOpcode.operator += (printBufferTemp);
											*address = *address + 1;
										break;
										default:
											printBufferAssembler.Printf("Illegal instruction");
										break;
									}
								}
							break;
							default:
								printBufferAssembler.Printf("Illegal instruction");
							break;
						}
					}
				break;
				case 0x9:
				case 0xa:
				case 0xb:
				case 0xc:
				case 0xd:
				case 0xe:
				case 0xf:
					printBufferAssembler.Printf("INP  %X",n-8);
					printBufferDetails.Printf("D=M(%04X)=%02X", p_Computer->getScratchpadRegister(p_Computer->getDataPointer()), accumulator);
				break;
			}
		break;
		case 0x7:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.operator += ("RET");
					printBufferDetails.Printf("P=R%X, X=R%X", p_Computer->getProgramCounter(), p_Computer->getDataPointer());
				break;
				case 0x1:
					printBufferAssembler.operator += ("DIS");
					printBufferDetails.Printf("P=R%X, X=R%X", p_Computer->getProgramCounter(), p_Computer->getDataPointer());
				break;
				case 0x2:
					printBufferAssembler.operator += ("LDXA");
					printBufferDetails.Printf("D=M(%04X)=%02X", p_Computer->getScratchpadRegister(p_Computer->getDataPointer())-1, accumulator);
				break;
				case 0x3:
					printBufferAssembler.operator += ("STXD");
					printBufferDetails.Printf("M(%04X)=%02X", p_Computer->getScratchpadRegister(p_Computer->getDataPointer())+1, accumulator);
				break;
				case 0x4:
					printBufferAssembler.operator += ("ADC");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x5:
					printBufferAssembler.operator += ("SDB");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x6:
					printBufferAssembler.operator += ("SHRC");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x7:
					printBufferAssembler.operator += ("SMB");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x8:
					printBufferAssembler.operator += ("SAV");
					printBufferDetails.Printf("M(%04X)=%02X", p_Computer->getScratchpadRegister(p_Computer->getDataPointer()), p_Computer->getRegisterT());
				break;
				case 0x9:
					printBufferAssembler.operator += ("MARK");
				break;
				case 0xa:
					printBufferAssembler.operator += ("REQ");
				break;
				case 0xb:
					printBufferAssembler.operator += ("SEQ");
				break;
				case 0xc:
					printBufferAssembler.Printf("ADCI %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xd:
					printBufferAssembler.Printf("SDBI %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xe:
					printBufferAssembler.operator += ("SHLC");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0xf:
					printBufferAssembler.Printf("SMBI %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
			}
		break;
		case 0x8:
			printBufferAssembler.Printf("GLO  R%X",n);
			printBufferDetails.Printf("D=%02X", accumulator);
		break;
		case 0x9:
			printBufferAssembler.Printf("GHI  R%X",n);
			printBufferDetails.Printf("D=%02X", accumulator);
		break;
		case 0xa:
			printBufferAssembler.Printf("PLO  R%X",n);
			printBufferDetails.Printf("R%X=%04X", n, p_Computer->getScratchpadRegister(n));
		break;
		case 0xb:
			printBufferAssembler.Printf("PHI  R%X",n);
			printBufferDetails.Printf("R%X=%04X", n, p_Computer->getScratchpadRegister(n));
		break;
		case 0xc:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.Printf("LBR  %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0x1:
					printBufferAssembler.Printf("LBQ  %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0x2:
					printBufferAssembler.Printf("LBZ  %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0x3:
					printBufferAssembler.Printf("LBDF %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0x4:
					printBufferAssembler.operator += ("NOP");
				break;
				case 0x5:
					printBufferAssembler.operator += ("LSNQ");
				break;
				case 0x6:
					printBufferAssembler.operator += ("LSNZ");
				break;
				case 0x7:
					printBufferAssembler.operator += ("LSNF");
				break;
				case 0x8:
					printBufferAssembler.operator += ("NLBR");
				break;
				case 0x9:
					printBufferAssembler.Printf("LBNQ %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0xa:
					printBufferAssembler.Printf("LBNZ %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0xb:printBufferAssembler.Printf("LBNF %04X",p_Computer->readMem(*address)<<8|p_Computer->readMem((*address)+1));
					printBufferTemp.Printf("%02X %02X ",p_Computer->readMem(*address),p_Computer->readMem((*address)+1));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 2;
				break;
				case 0xc:
					printBufferAssembler.operator += ("LSIE");
				break;
				case 0xd:
					printBufferAssembler.operator += ("LSQ");
				break;
				case 0xe:
					printBufferAssembler.operator += ("LSZ");
				break;
				case 0xf:
					printBufferAssembler.operator += ("LSDF");
				break;
			}
		break;
		case 0xd:
			printBufferAssembler.Printf("SEP  R%X",n);
		break;
		case 0xe:
			printBufferAssembler.Printf("SEX  R%X",n);
		break;
		case 0xf:
			switch(n)
			{
				case 0x0:
					printBufferAssembler.operator += ("LDX");
					printBufferDetails.Printf("D=M(%04X)=%02X", p_Computer->getScratchpadRegister(p_Computer->getDataPointer()), accumulator);
				break;
				case 0x1:
					printBufferAssembler.operator += ("OR");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x2:
					printBufferAssembler.operator += ("AND");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x3:
					printBufferAssembler.operator += ("XOR");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x4:
					printBufferAssembler.operator += ("ADD");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x5:
					printBufferAssembler.operator += ("SD");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x6:
					printBufferAssembler.operator += ("SHR");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x7:
					printBufferAssembler.operator += ("SM");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0x8:
					printBufferAssembler.Printf("LDI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					*address = *address + 1;
				break;
				case 0x9:
					printBufferAssembler.Printf("ORI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xa:printBufferAssembler.Printf("ANI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xb:printBufferAssembler.Printf("XRI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xc:printBufferAssembler.Printf("ADI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xd:printBufferAssembler.Printf("SDI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
				case 0xe:
					printBufferAssembler.operator += ("SHL");
					printBufferDetails.Printf("D=%02X", accumulator);
				break;
				case 0xf:printBufferAssembler.Printf("SMI  %02X",p_Computer->readMem(*address));
					printBufferTemp.Printf("%02X ",p_Computer->readMem(*address));
					printBufferOpcode.operator += (printBufferTemp);
					printBufferDetails.Printf("D=%02X", accumulator);
					*address = *address + 1;
				break;
			}
		break;
	}
	while(printBufferOpcode.Len() < 12) printBufferOpcode.operator += (" ");
	while(printBufferAssembler.Len() < 10) printBufferAssembler.operator += (" ");

	if (showOpcode)
		printBufferTemp = printBufferAddress + printBufferOpcode + printBufferAssembler;
	else
		printBufferTemp =  printBufferAddress + printBufferAssembler;

	if (showDetails)
		return printBufferTemp + printBufferDetails;
	else
		return printBufferTemp;
}

void DebugWindow::disassembleChip8(Word start, Word end)
{
	wxString printBuffer;

	while(start <= end)
		disassemblerDisplay(disassembleChip8(&start));
}

wxString DebugWindow::disassembleChip8(Word* address)
{
	wxString printBuffer;

	if (p_Computer->getChip8Type() == CHIPST2)
	{
		printBuffer = st2Disassemble(*address, false, true);
		switch (p_Computer->readMem(*address))
		{
			case 0xC0:
			case 0xE0:
			case 0xE1:
			case 0xE2:
			case 0xE4:
				*address = *address + 1;
			break;

			default:		
				*address = *address + 2;
			break;
		}
	}
	else
	{
		printBuffer = chip8Disassemble(*address, false, true);
		*address = *address + 2;
	}
	return printBuffer;
}

AssInput DebugWindow::getAssInput(wxString buffer)
{
	AssInput result;
	result.numberOfParameters = 0;
	result.errorCode = 0;

	result.command = extractNextWord(&buffer, &result.commandSeperator);

	while (buffer != "" && result.numberOfParameters <= 8)
	{
		result.parameterString[result.numberOfParameters] = extractNextWord(&buffer, &result.seperator[result.numberOfParameters]);

		result.errorCode = translateChipParameter(result.parameterString[result.numberOfParameters], &result.parameterValue[result.numberOfParameters], &result.parameterType[result.numberOfParameters]);
		if (result.errorCode != 0)  return result;

		result.numberOfParameters++;
	}
	
	return result;
}

int DebugWindow::assembleSt2(wxString *buffer, Byte* b1, Byte* b2)
{
	AssInput assInput = getAssInput(*buffer);
	if (assInput.errorCode != 0)  return assInput.errorCode;
	if (assInput.commandSeperator != " ") return ERROR_COMMAND_SEP;

	if (assInput.command == "ADD") 
	{
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE && assInput.parameterType[1] != CHIP8_VX)
			return ERROR_8REG;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // ADD vx, kk
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x70 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{ // ADD Vx, Vy
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x04;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "AFOR") 
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			if (assInput.parameterValue[0] == 0) return ERROR_NO_REG0;
			*b1 = 0xF0 | assInput.parameterValue[0];
			*b2 = 0xb6;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "AND") 
	{ // AND Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x02;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "CALL") 
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) | 0x20;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "DJNZ") 
	{
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] != 0)
				return ERROR_ONLY_REG0;
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x70;
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "JKP") 
	{
		if (assInput.parameterType[0] != CHIP8_VX && assInput.parameterType[0] != ASS_HEX_VALUE)
			return ERROR_4REG;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] == 0xf) 
				return ERROR_NO_VALUE_F;
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xf)
				return ERROR_4BIT;
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0xd0 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] != 0xb) 
				return ERROR_ONLY_REGB;
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0xdf;
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "JP") 
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) | 0x10;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "JNZ") 
	{
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x30 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "JZ") 
	{
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x40 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_8BIT;
	}
	if (assInput.command == "LD") 
	{
		if (assInput.parameterType[0] == ASS_HEX_VALUE && assInput.parameterValue[0] == 0xb)
		{ // LD B, [Vy], Vx or LD B, Vy, Vx
			if (assInput.parameterType[1] != CHIP8_VX && assInput.parameterType[1] != CHIP8_VX_MEM)
				return ERROR_REG_EXP;
			if (assInput.parameterType[2] != CHIP8_VX)
				return ERROR_REG_EXP;
			if (assInput.seperator[0] != "," || assInput.seperator[1] != "," )
				return ERROR_COMMA;
			if (assInput.seperator[2] != " " || assInput.numberOfParameters > 3)
				return ERROR_PAR;

			if ((assInput.parameterType[1] == CHIP8_VX_MEM || assInput.parameterType[1] == CHIP8_VX)  && assInput.parameterType[2] == CHIP8_VX)
			{  
				*b1 = 0x90 | assInput.parameterValue[2];
				*b2 = assInput.parameterValue[1] << 4 | 0x8;
				return 2;
			}
		}
		if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "[I]" && assInput.numberOfParameters > 2)
		{ // LD [I], kk, n or LD [I], kk, +n
			if (assInput.parameterType[1] != ASS_HEX_VALUE)
				return ERROR_8BIT;
			if (assInput.parameterType[2] != ASS_HEX_VALUE)
				return ERROR_4BIT;
			if (assInput.seperator[0] != "," || assInput.seperator[1] != "," )
				return ERROR_COMMA;
			if (assInput.seperator[2] != " " || assInput.numberOfParameters > 3)
				return ERROR_PAR;

			if (assInput.parameterType[1] == ASS_HEX_VALUE && assInput.parameterType[1] == ASS_HEX_VALUE)
			{ 
				if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
					return ERROR_8BIT;
				if (assInput.parameterValue[2] < 0 || assInput.parameterValue[2] > 0xf)
					return ERROR_4BIT;
				*b1 = 0xb0 | assInput.parameterValue[2];
				*b2 = assInput.parameterValue[1];
				return 2;
			}
		}
		if (assInput.parameterType[0] != CHIP8_VX && assInput.parameterType[0] != ASS_STRING && assInput.parameterType[0] != CHIP8_VX_MEM)
			return ERROR_SYNTAX;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "I")
		{ // LD I, aaa
			if (assInput.parameterType[1] != ASS_HEX_VALUE)
				return ERROR_12BIT;
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xfff)
				return ERROR_12BIT;
			*b1 = 0xa0 | (assInput.parameterValue[1]>>8);
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX)
		{ 
			if (assInput.parameterType[1] == ASS_HEX_VALUE)
			{ // LD Vx, kk
				if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
					return ERROR_8BIT;
				*b1 = 0x60 | assInput.parameterValue[0];
				*b2 = assInput.parameterValue[1];
				return 2;
			}
			if (assInput.parameterType[1] == CHIP8_VX)
			{ // LD Vx, Vy
				*b1 = 0x80 | assInput.parameterValue[0];
				*b2 = (assInput.parameterValue[1] << 4) &0xf0;
				return 2;
			}
			if (assInput.parameterType[1] == CHIP8_VX_MEM)
			{ // LD [Vx], Vy
				*b1 = 0x90 | assInput.parameterValue[0];
				*b2 = (assInput.parameterValue[1] << 4) | 0x2;
				return 2;
			}
			return ERROR_8REG;
		}
		if (assInput.parameterType[0] == CHIP8_VX_MEM && assInput.parameterType[1] == CHIP8_VX)
		{ // LD [Vy], Vx
			*b1 = 0x90 | assInput.parameterValue[1];
			*b2 = (assInput.parameterValue[0] << 4) | 0x4;
			return 2;
		}
		if (assInput.parameterType[0] == ASS_STRING && (assInput.parameterString[0] == "[->I]" || assInput.parameterString[0] == "[I]") && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // LD [->I], 0 or LD [I], 0
			if (assInput.parameterValue[1] != 0)
				return ERROR_ONLY_VALUE_0;
			*b1 = 0x02;
			*b2 = 0xf2;
			return 2;
		}
		return ERROR_8REG;
	} 
	if (assInput.command == "LDM") 
	{ // LDM Vx, [I] or LDM [I], Vx
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "[I]")
		{
			if (assInput.parameterValue[0] == 0) 
				return ERROR_NO_REG0;
			*b1 = 0xf0 | assInput.parameterValue[0];
			*b2 = 0xa6;
			return 2;
		}
		if (assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "[I]")
		{
			if (assInput.parameterValue[1] == 0) 
				return ERROR_NO_REG0;
			*b1 = 0xf0 | assInput.parameterValue[1];
			*b2 = 0xa9;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "LDMA") 
	{ // LDMA Vx, [I] or LDMA [I], Vx
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "[I]")
		{
			if (assInput.parameterValue[0] == 0) 
				return ERROR_NO_REG0;
			*b1 = 0xf0 | assInput.parameterValue[0];
			*b2 = 0xac;
			return 2;
		}
		if (assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "[I]")
		{
			if (assInput.parameterValue[1] == 0) 
				return ERROR_NO_REG0;
			*b1 = 0xf0 | assInput.parameterValue[1];
			*b2 = 0xaf;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "OR") 
	{ // OR Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x01;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "RET") 
	{ // RET
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;

		*b1 = 0xc0;
		return 1;
	} 
	if (assInput.command == "RND") 
	{ // RND Vx, kk
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0xc0 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SLSB") 
	{ // SLSB Vx
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			if (assInput.parameterValue[0] == 0) return ERROR_NO_REG0;
			*b1 = 0xF0 | assInput.parameterValue[0];
			*b2 = 0xb3;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "SHL") 
	{ // SHL Vx or SHL Vx, Vy
		if (assInput.parameterType[0] == CHIP8_VX && assInput.numberOfParameters == 1)
		{
			if (assInput.seperator[0] != " ")
				return ERROR_REG_EXP;
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[0] << 4) | 0xe;
			return 2;
		}
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0xe;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SHR") 
	{ // SHR Vx or SHR Vx, Vy
		if (assInput.parameterType[0] == CHIP8_VX && assInput.numberOfParameters == 1)
		{
			if (assInput.seperator[0] != " ")
				return ERROR_REG_EXP;
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[0] << 4) | 0x6;
			return 2;
		}
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x6;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SNE") 
	{ // SNE Vx, kk or SNE Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE && assInput.parameterType[1] != CHIP8_VX)
			return ERROR_8REG;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x40 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x90 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) & 0xf0;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SP") 
	{ 
		if (assInput.parameterType[0] != ASS_STRING && assInput.parameterType[0] != CHIP8_VX)
			return ERROR_SPRITE;
		if (assInput.numberOfParameters == 1)
		{ 
			if (assInput.seperator[0] != " ")
				return ERROR_SYNTAX;
			if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "CLEAR")
			{ // SP CLEAR
				*b1 = 0xe0;
				return 1;
			}
			if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "MOVE")
			{ // SP MOVE
				*b1 = 0xe1;
				return 1;
			}
			return ERROR_SPRITE;
		}
		if (assInput.parameterType[1] != ASS_STRING && assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_SPRITE;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.numberOfParameters == 2)
		{ 
			if (assInput.seperator[1] != " ")
				return ERROR_SYNTAX;
			if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "MOVE")
			{ // SP VC, Move
				if (assInput.parameterValue[0] != 0xc) 
					return ERROR_ONLY_REGC;
				*b1 = 0xe2;
				return 1;
			}
			if (assInput.parameterType[0] == ASS_STRING &&  assInput.parameterString[0] == "XOR" && assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "[I]")
			{ // SP XOR, [I]
				*b1 = 0xe4;
				return 1;
			}
			if (assInput.parameterType[0] == ASS_STRING &&  assInput.parameterString[0] == "DRW" && assInput.parameterType[1] == ASS_HEX_VALUE)
			{ // SP DRW, kk
				if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
					return ERROR_8BIT;
				*b1 = 0xe8;
				*b2 = assInput.parameterValue[1];
				return 2;
			}
		}
		if (assInput.parameterType[2] != ASS_HEX_VALUE || assInput.parameterValue[2] < 0 || assInput.parameterValue[2] > 0xff)
			return ERROR_8BIT;
		if (assInput.seperator[1] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[2] != " " || assInput.numberOfParameters > 3)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_STRING &&  assInput.parameterString[0] == "DRW" && assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "JC")
		{ // SP DRW, JC, kk
			*b1 = 0xe8;
			*b2 = assInput.parameterValue[2];
			return 2;
		}
		return ERROR_VIDEO;
	} 
	if (assInput.command == "SUB") 
	{ // SUB Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x05;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SUBN") 
	{ // SUBN Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x07;
			return 2;
		}
		return ERROR_8REG;
	}
	if (assInput.command == "SYS") 
	{ // SYS aaa
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) & 0xf;
			if (*b1 == 2)
				*b1 = 0xf0;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "VIDEO") 
	{ 
		if (assInput.parameterType[0] != ASS_STRING)
			return ERROR_VIDEO;
		if (assInput.numberOfParameters == 1)
		{ 
			if (assInput.seperator[0] != " ")
				return ERROR_SYNTAX;
			if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "ON")
			{ // VIDEO ON
				*b1 = 0;
				*b2 = 0x66;
				return 2;
			}
		}
		return ERROR_VIDEO;
	}
	if (assInput.command == "XOR") 
	{ // XOR Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x03;
			return 2;
		}
		return ERROR_8REG;
	}
	return ERROR_CHIP;
}

int DebugWindow::assembleChip(wxString *buffer, Byte* b1, Byte* b2)
{
	AssInput assInput = getAssInput(*buffer);
	if (assInput.errorCode != 0)  return assInput.errorCode;
	if (assInput.commandSeperator != " ") return ERROR_COMMAND_SEP;

	if (assInput.command == "ADD") 
	{
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "I" && assInput.parameterType[1] == CHIP8_VX)
		{ // ADD I, Vx
			*b1 = 0xf0 | assInput.parameterValue[1];
			*b2 = 0x1e;
			return 2;
		}

		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE && assInput.parameterType[1] != CHIP8_VX)
			return ERROR_8REG;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // ADD Vx, kk
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x70 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{ // ADD Vx, Vy
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x04;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "AND") 
	{ // ADD Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x02;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "CALL") 
	{ // CALL aaa
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) | 0x20;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "CLS") 
	{ // CLS
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;

		*b1 = 0x00;
		*b2 = 0xe0;
		return 2;
	}
	if (assInput.command == "DRW") 
	{ // DRW Vx, Vy, n
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[2] != ASS_HEX_VALUE)
			return ERROR_4BIT;
		if (assInput.seperator[0] != "," || assInput.seperator[1] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[2] != " " || assInput.numberOfParameters > 3)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[2] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xf)
				return ERROR_4BIT;
			*b1 = 0xd0 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | assInput.parameterValue[2];
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "JP") 
	{
		if (assInput.parameterType[0] == ASS_HEX_VALUE && assInput.numberOfParameters == 1 && assInput.seperator[0] == " ")
		{ // JP aaa
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) | 0x10;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}

		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_12BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // JP V0, aaa
			if (p_Computer->getChip8Type() == CHIP8X)
				return ERROR_NS_8X;
			if (assInput.parameterValue[0] != 0)
				return ERROR_ONLY_REG0;
			*b1 = 0xb0 | (assInput.parameterValue[1]>>8);
			*b2 = assInput.parameterValue[1] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "LD") 
	{
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] == 0xb)
			{ // LD B, Vx
				if (assInput.parameterType[1] != CHIP8_VX)
					return ERROR_REG_EXP;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x33;
				return 2;
			}
			if (assInput.parameterValue[0] == 0xf)
			{ // LD F, Vx
				if (assInput.parameterType[1] != CHIP8_VX)
					return ERROR_REG_EXP;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x29;
				return 2;
			}
			return ERROR_LD;
		}
		if (assInput.parameterType[0] == ASS_STRING)
		{
			if (assInput.parameterString[0] == "DT")
			{ // LD DT, Vx
				if (assInput.parameterType[1] != CHIP8_VX)
					return ERROR_REG_EXP;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x15;
				return 2;
			}
			if (assInput.parameterString[0] == "I")
			{ // LD I, aaa
				if (assInput.parameterType[1] != ASS_HEX_VALUE || assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xfff)
					return ERROR_12BIT;

				*b1 = 0xa0 | (assInput.parameterValue[1]>>8);
				*b2 = assInput.parameterValue[1];
				return 2;
			}
			if (assInput.parameterString[0] == "ST")
			{ // LD ST, Vx
				if (assInput.parameterType[1] != CHIP8_VX)
					return ERROR_REG_EXP;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x18;
				return 2;
			}
			if (assInput.parameterString[0] == "[I]")
			{ // LD [I], Vx
				if (assInput.parameterType[1] != CHIP8_VX)
					return ERROR_REG_EXP;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x55;
				return 2;
			}
			if (assInput.parameterString[0] == "P")
			{ // LD P, Vx
				if (p_Computer->getChip8Type() != CHIPETI)
					return ERROR_NS_ETI;
				*b1 = 0xf0 | assInput.parameterValue[1];
				*b2 = 0x00;
				return 2;
			}
		}
		if (assInput.parameterType[0] == CHIP8_VX)
		{
			if (assInput.parameterType[1] == ASS_HEX_VALUE)
			{ // LD Vx, kk
				if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xff)
					return ERROR_8BIT;
				*b1 = 0x60 | assInput.parameterValue[0];
				*b2 = assInput.parameterValue[1];
				return 2;
			}
			if (assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "DT")
			{ // LD Vx, DT
				*b1 = 0xf0 | assInput.parameterValue[0];
				*b2 = 0x07;
				return 2;
			}
			if (assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "K")
			{ // LD Vx, K
				*b1 = 0xf0 | assInput.parameterValue[0];
				*b2 = 0x0a;
				return 2;
			}
			if (assInput.parameterType[1] == CHIP8_VX)
			{ // LD Vx Vy
				*b1 = 0x80 | assInput.parameterValue[0];
				*b2 = (assInput.parameterValue[0] << 4) &0xf0;
				return 2;
			}
			if (assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "[I]")
			{ // LD Vx, [I]
				*b1 = 0xf0 | assInput.parameterValue[0];
				*b2 = 0x65;
				return 2;
			}
		}
		return ERROR_LD;
	}
	if (assInput.command == "OR") 
	{ // OR Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x01;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "RET") 
	{ // RET
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;

		*b1 = 0x00;
		*b2 = 0xee;
		return 2;
	}
	if (assInput.command == "RND") 
	{ // RND Vx,, kk
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE)
			return ERROR_8BIT;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;

			*b1 = 0xc0 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		return ERROR_8REG;
	}
	if (assInput.command == "SE") 
	{ 
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE && assInput.parameterType[1] != CHIP8_VX)
			return ERROR_8REG;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // SE Vx, kk
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;

			*b1 = 0x30 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{ // SE vx, Vy
			*b1 = 0x50 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) & 0xf0;
			return 2;
		}
		return ERROR_8REG;
	}
	if (assInput.command == "SHL") 
	{ // SHL Vx or SHL Vx, Vy
		if (assInput.parameterType[0] == CHIP8_VX && assInput.numberOfParameters == 1)
		{
			if (assInput.seperator[0] != " ")
				return ERROR_REG_EXP;
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[0] << 4) | 0xe;
			return 2;
		}
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0xe;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SHR") 
	{ // SHR Vx or SHR Vx, Vy
		if (assInput.parameterType[0] == CHIP8_VX && assInput.numberOfParameters == 1)
		{
			if (assInput.seperator[0] != " ")
				return ERROR_REG_EXP;
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[0] << 4) | 0x6;
			return 2;
		}
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x6;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SKP") 
	{ // SKP Vx
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xe0 | assInput.parameterValue[0];
			*b2 = 0x9e;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "SKNP") 
	{ // SKNP
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xe0 | assInput.parameterValue[0];
			*b2 = 0xa1;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "SNE") 
	{
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != ASS_HEX_VALUE && assInput.parameterType[1] != CHIP8_VX)
			return ERROR_8REG;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == ASS_HEX_VALUE)
		{ // SNE Vx, kk
			if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xff)
				return ERROR_8BIT;
			*b1 = 0x40 | assInput.parameterValue[0];
			*b2 = assInput.parameterValue[1];
			return 2;
		}
		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{ // SNE Vx, Vy
			*b1 = 0x90 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) & 0xf0;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SUB") 
	{ // SUB Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x05;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SUBN") 
	{ // SUBN Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x07;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "SYS") 
	{ // SYS aaa
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xfff)
				return ERROR_12BIT;
			*b1 = (assInput.parameterValue[0] >> 8) & 0xf;
			*b2 = assInput.parameterValue[0] & 0xff;
			return 2;
		}
		return ERROR_12BIT;
	}
	if (assInput.command == "XOR") 
	{ // XOR Vx, Vy
		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x80 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x03;
			return 2;
		}
		return ERROR_SYNTAX;
	}
	if (assInput.command == "CLR" || assInput.command == "CLR-L" || assInput.command == "CLR-H") 
	{
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.numberOfParameters > 4)
			return ERROR_PAR;

		if (assInput.numberOfParameters == 1)
		{
			if (assInput.command == "CLR-H" || assInput.command == "CLR-L")
				return ERROR_CLR;
			if (assInput.seperator[0] != " ")
				return ERROR_PAR;
			if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "BACK") 
			{ // CLR BACK or CLR B
				*b1 = 0x02;
				*b2 = 0xa0;
				return 2;
			}
			if (assInput.parameterType[0] == ASS_HEX_VALUE && assInput.parameterValue[0] == 0xb) 
			{ // CLR BACK or CLR B
				*b1 = 0x02;
				*b2 = 0xa0;
				return 2;
			}
			return ERROR_CLR;
		}

		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		
		if (assInput.numberOfParameters == 2)
		{ // CLR Vx, Vy or CLR-L Vx, Vy
			if (assInput.command == "CLR-H")
				return ERROR_CLR;
			if (assInput.seperator[0] != ",")
				return ERROR_COMMA;
			if (assInput.seperator[1] != " ")
				return ERROR_PAR;
			if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
			{
				*b1 = 0xB0 | assInput.parameterValue[0];
				*b2 = (assInput.parameterValue[1] << 4) & 0xf0;
				return 2;
			}
			return ERROR_REG_EXP;
		}

		if (assInput.numberOfParameters == 3)
		{
			if (assInput.seperator[1] != ",")
				return ERROR_COMMA;
			if (assInput.seperator[2] != " ")
				return ERROR_PAR;
			
			if (assInput.seperator[0] == ",")
			{ // CLR Vx, Vy, n or CLR-H Vx, Vy, n
				if (assInput.command == "CLR-L")
					return ERROR_CLR;
				if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[2] == ASS_HEX_VALUE)
				{
					if (assInput.parameterValue[2] < 0 || assInput.parameterValue[2] > 0xf)
						return ERROR_4BIT;
					if (assInput.parameterValue[2] == 0)
						return ERROR_NO_VALUE_0;
					*b1 = 0xB0 | assInput.parameterValue[0];
					*b2 = (assInput.parameterValue[1] << 4) | assInput.parameterValue[2];
					return 2;
				}
				return ERROR_CLR;
			}

			if (assInput.seperator[0] == "/")
			{ // CLR Vx/V(x+1), Vy or CLR-L Vx/V(x+1), Vy
				if (assInput.command == "CLR-H")
					return ERROR_CLR;
				if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[2] == CHIP8_VX)
				{
					if (assInput.parameterValue[0] != assInput.parameterValue[1] - 1)
						return ERROR_2ND_REG;
					*b1 = 0xB0 | assInput.parameterValue[0];
					*b2 = (assInput.parameterValue[2] << 4) & 0xf0;
					return 2;
				}
				return ERROR_REG_EXP;
			}
		}

		if (assInput.numberOfParameters == 4)
		{ // CLR Vx/V(x+1), Vy, n or CLR-H Vx/V(x+1), Vy, n 
			if (assInput.command == "CLR-L")
				return ERROR_CLR;
			if (assInput.seperator[0] != "/")
				return ERROR_SLASH;
			if (assInput.seperator[1] != "," || assInput.seperator[2] != ",")
				return ERROR_COMMA;
			
			if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX && assInput.parameterType[2] == CHIP8_VX && assInput.parameterType[3] == ASS_HEX_VALUE)
			{
				if (assInput.parameterValue[3] < 0 || assInput.parameterValue[3] > 0xf)
					return ERROR_4BIT;
				if (assInput.parameterValue[3] == 0)
					return ERROR_NO_VALUE_0;
				if (assInput.parameterValue[0] != assInput.parameterValue[1] - 1)
					return ERROR_2ND_REG;
				*b1 = 0xB0 | assInput.parameterValue[0];
				*b2 = (assInput.parameterValue[2] << 4) | assInput.parameterValue[3];
				return 2;
			}
			return ERROR_CLR;
		}
		return ERROR_CLR;
	}
	if (assInput.command == "ADD-8") 
	{ // ADD-8 Vx, Vy
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.parameterType[0] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.parameterType[1] != CHIP8_VX)
			return ERROR_REG_EXP;
		if (assInput.seperator[0] != ",")
			return ERROR_COMMA;
		if (assInput.seperator[1] != " " || assInput.numberOfParameters > 2)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX && assInput.parameterType[1] == CHIP8_VX)
		{
			*b1 = 0x50 | assInput.parameterValue[0];
			*b2 = (assInput.parameterValue[1] << 4) | 0x01;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "SKP2") 
	{ // SKP2 Vx
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xe0 | assInput.parameterValue[0];
			*b2 = 0xf2;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "SKNP2") 
	{ // SKNP2 Vx
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xe0 | assInput.parameterValue[0];
			*b2 = 0xf5;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "OUT-3" || assInput.command == "OUT") 
	{ // OUT-3 Vx or OUT Vx
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xf0 | assInput.parameterValue[0];
			*b2 = 0xf8;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	if (assInput.command == "INP-1" || assInput.command == "INP") 
	{ // INP-1 Vx or INP Vx
		if (p_Computer->getChip8Type() != CHIP8X)
		{
			if (p_Computer->getChip8Type() == CHIPETI)
				return ERROR_NS_ETI;
			else
				return ERROR_NS_8;
		}

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == CHIP8_VX)
		{
			*b1 = 0xf0 | assInput.parameterValue[0];
			*b2 = 0xfb;
			return 2;
		}
		return ERROR_REG_EXP;
	}
	return ERROR_CHIP;
}

int DebugWindow::assemble(wxString *buffer, Byte* b1, Byte* b2, Byte* b3, Byte* b4, Byte* b5, bool allowX)
{
	wxString command;
	long registerNumber;
	int ret;

	AssInput assInput = getAssInput(*buffer);
	if (assInput.errorCode != 0)  return assInput.errorCode;
	if (assInput.commandSeperator != " ") return ERROR_COMMAND_SEP;

	*b5 = 0xff;

/*	if (assInput.command == "D") 
	{ 
		*b1 = assInput.parameterValue[0]; 
		if (assInput.parameterType[0] == ASS_HEX_VALUE && assInput.numberOfParameters <= 1)
			return 1;
		*b2 = assInput.parameterValue[1]; 
		if (assInput.parameterType[1] == ASS_HEX_VALUE && assInput.numberOfParameters <= 2)
			return 2;
		*b3 = assInput.parameterValue[2]; 
		if (assInput.parameterType[2] == ASS_HEX_VALUE && assInput.numberOfParameters <= 3)
			return 3;
		*b4 = assInput.parameterValue[3]; 
		if (assInput.parameterType[3] == ASS_HEX_VALUE && assInput.numberOfParameters <= 4)
			return 4;

		return ERROR_SYNTAX; 
	}*/
	if (assInput.command == "ADC") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x74; 
		return 1; 
	}
	if (assInput.command == "ADCI")
	{
		*b1 = 0x7c;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "ADD") 
	{
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf4; 
		return 1;
	}
	if (assInput.command == "ADI")
	{
		*b1 = 0xfc;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "AND") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf2; 
		return 1; 
	}
	if (assInput.command == "ANI")
	{
		*b1 = 0xfa;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "B1")
	{
		*b1 = 0x34;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "B2")
	{
		*b1 = 0x35;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "B3")
	{
		*b1 = 0x36;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "B4")
	{
		*b1 = 0x37;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BDF")
	{
		*b1 = 0x33;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BN1")
	{
		*b1 = 0x3c;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BN2")
	{
		*b1 = 0x3d;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BN3")
	{
		*b1 = 0x3e;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BN4")
	{
		*b1 = 0x3f;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BNF")
	{
		*b1 = 0x3b;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BNQ")
	{
		*b1 = 0x39;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BNZ")
	{
		*b1 = 0x3a;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BQ")
	{
		*b1 = 0x31;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BR")
	{
		*b1 = 0x30;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "BZ")
	{
		*b1 = 0x32;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "DEC")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x20 | registerNumber;
		return ret;
	}
	if (assInput.command == "DIS") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x71; 
		return 1; 
	}
	if (assInput.command == "GHI")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x90 | registerNumber;
		return ret;
	}
	if (assInput.command == "GLO")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x80 | registerNumber;
		return ret;
	}
	if (assInput.command == "IDL") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x00; 
		return 1; 
	}
	if (assInput.command == "INC")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x10 | registerNumber;
		return ret;
	}
	if (assInput.command == "INP")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 1 || assInput.parameterValue[0] > 7)
				return ERROR_IO;

			*b1 = 0x60 | ((assInput.parameterValue[0] & 0x7)+8); 
			return 1;
		}
		if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "X" && allowX)
		{
			*b1 = 0x69;
			*b5 = 0xf0;
			return 1;
		}
		return ERROR_IO;
	}
	if (assInput.command == "IRX") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x60; 
		return 1; 
	}
	if (assInput.command == "LBDF")
	{
		*b1 = 0xc3;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBNF")
	{
		*b1 = 0xcb;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBNQ")
	{
		*b1 = 0xc9;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBNZ")
	{
		*b1 = 0xca;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBQ")
	{
		*b1 = 0xc1;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBR")
	{
		*b1 = 0xc0;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LBZ")
	{
		*b1 = 0xc2;
		return getWord(assInput, b2, b3, allowX);
	}
	if (assInput.command == "LDA")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x40 | registerNumber;
		return ret;
	}
	if (assInput.command == "LDI")
	{
		*b1 = 0xf8;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "LDN")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x00 | registerNumber;
		return ret;
	}
	if (assInput.command == "LDX") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf0; 
		return 1; 
	}
	if (assInput.command == "LDXA") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x72; 
		return 1; 
	}
	if (assInput.command == "LSDF") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xcf; 
		return 1; 
	}
	if (assInput.command == "LSIE") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xcc; 
		return 1; 
	}
	if (assInput.command == "LSKP") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc8; 
		return 1; 
	}
	if (assInput.command == "LSNF") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc7; 
		return 1; 
	}
	if (assInput.command == "LSNQ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc5; 
		return 1; 
	}
	if (assInput.command == "LSNZ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc6; 
		return 1; 
	}
	if (assInput.command == "LSQ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xcd; 
		return 1; 
	}
	if (assInput.command == "LSZ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xce; 
		return 1; 
	}
	if (assInput.command == "MARK") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x79; 
		return 1; 
	}
	if (assInput.command == "NBR") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x38; 
		return 1; 
	}
	if (assInput.command == "NLBR") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc8; 
		return 1; 
	}
	if (assInput.command == "NOP") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xc4; 
		return 1; 
	}
	if (assInput.command == "OR") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf1; 
		return 1; 
	}
	if (assInput.command == "ORI")
	{
		*b1 = 0xf9;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "OUT")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		if (assInput.parameterType[0] == ASS_HEX_VALUE)
		{
			if (assInput.parameterValue[0] < 1 || assInput.parameterValue[0] > 7)
				return ERROR_IO;

			*b1 = 0x60 | (assInput.parameterValue[0] & 0x7); 
			return 1;
		}
		if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "X" && allowX)
		{
			*b1 = 0x61;
			*b5 = 0xf0;
			return 1;
		}
		return ERROR_IO;
	}
	if (assInput.command == "PHI")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0xb0 | registerNumber;
		return ret;
	}
	if (assInput.command == "PLO")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0xa0 | registerNumber;
		return ret;
	}
	if (assInput.command == "REQ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x7a; 
		return 1; 
	}
	if (assInput.command == "RET") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x70; 
		return 1; 
	}
	if (assInput.command == "SAV") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x78; 
		return 1; 
	}
	if (assInput.command == "SD") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf5; 
		return 1; 
	}
	if (assInput.command == "SDB") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x75; 
		return 1; 
	}
	if (assInput.command == "SDBI")
	{
		*b1 = 0x7d;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "SDI")
	{
		*b1 = 0xfd;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "SEP")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0xd0 | registerNumber;
		return ret;
	}
	if (assInput.command == "SEQ") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x7b; 
		return 1; 
	}
	if (assInput.command == "SEX")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0xe0 | registerNumber;
		return ret;
	}
	if (assInput.command == "SHL") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xfe; 
		return 1; 
	}
	if (assInput.command == "SHLC") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x7e; 
		return 1; 
	}
	if (assInput.command == "SHR") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf6; 
		return 1; 
	}
	if (assInput.command == "SHRC") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x76; 
		return 1; 
	}
	if (assInput.command == "SKP") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x38; 
		return 1; 
	}
	if (assInput.command == "SM") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf7; 
		return 1; 
	}
	if (assInput.command == "SMB") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x77; 
		return 1; 
	}
	if (assInput.command == "SMBI")
	{
		*b1 = 0x7f;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "SMI")
	{
		*b1 = 0xff;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "STR")
	{
		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		*b1 = 0x50 | registerNumber;
		return ret;
	}
	if (assInput.command == "STXD") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0x73; 
		return 1; 
	}
	if (assInput.command == "XOR") 
	{ 
		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;
		*b1 = 0xf3; 
		return 1; 
	}
	if (assInput.command == "XRI")
	{
		*b1 = 0xfb;
		return getByte(assInput, b2, allowX);
	}
	if (assInput.command == "STPC")
	{
		*b1 = 0x68;
		*b2 = 0;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "DTC")
	{
		*b1 = 0x68;
		*b2 = 0x1;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "SPM2")
	{
		*b1 = 0x68;
		*b2 = 0x2;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "SCM2")
	{
		*b1 = 0x68;
		*b2 = 0x3;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "SPM1")
	{
		*b1 = 0x68;
		*b2 = 0x4;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "SCM1")
	{
		*b1 = 0x68;
		*b2 = 0x5;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "LDC")
	{
		*b1 = 0x68;
		*b2 = 0x6;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "STM")
	{
		*b1 = 0x68;
		*b2 = 0x7;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "GEC")
	{
		*b1 = 0x68;
		*b2 = 0x8;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "ETQ")
	{
		*b1 = 0x68;
		*b2 = 0x9;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "XIE")
	{
		*b1 = 0x68;
		*b2 = 0xa;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "XID")
	{
		*b1 = 0x68;
		*b2 = 0xb;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "CIE")
	{
		*b1 = 0x68;
		*b2 = 0xc;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "CID")
	{
		*b1 = 0x68;
		*b2 = 0xd;
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		return 2;
	}
	if (assInput.command == "DBNZ")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		if (assInput.numberOfParameters > 2)
			return ERROR_PAR;

		ret = getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x20 | registerNumber;
		return getWordPar2(assInput, b3, b4, allowX);
	}
	if (assInput.command == "BCI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x3e;
		return 3;
	}
	if (assInput.command == "BXI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x3f;
		return 3;
	}
	if (assInput.command == "RLXA")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x60 | registerNumber;
		return 2;
	}
	if (assInput.command == "DADC")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;
		*b1 = 0x68;
		*b2 = 0x74;
		return 2;
	}
	if (assInput.command == "DSAV")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;
		*b1 = 0x68;
		*b2 = 0x76;
		return 2;
	}
	if (assInput.command == "DSMB")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;
		*b1 = 0x68;
		*b2 = 0x77;
		return 2;
	}
	if (assInput.command == "DACI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x7c;
		return 3;
	}
	if (assInput.command == "DSBI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x7f;
		return 3;
	}
	if (assInput.command == "SCAL")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 2)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x80 | registerNumber;
		return getWordPar2(assInput, b3, b4, allowX);
	}
	if (assInput.command == "SRET")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0x90 | registerNumber;
		return 2;
	}
	if (assInput.command == "RSXD")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0xa0 | registerNumber;
		return 2;
	}
	if (assInput.command == "RNX")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 1)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0xb0 | registerNumber;
		return 2;
	}
	if (assInput.command == "RLDI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;

		if (assInput.numberOfParameters > 2)
			return ERROR_PAR;

		ret =  getRegisterNumber(assInput, &registerNumber, b5, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0xc0 | registerNumber;
		return getWordPar2(assInput, b3, b4, allowX);
	}
	if (assInput.command == "DADD")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;

		*b1 = 0x68;
		*b2 = 0xf4;
		return 2;
	}
	if (assInput.command == "DSM")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		if (assInput.numberOfParameters > 0)
			return ERROR_PAR;

		*b1 = 0x68;
		*b2 = 0xf7;
		return 2;
	}
	if (assInput.command == "DADI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0xfc;
		return 3;
	}
	if (assInput.command == "DSMI")
	{
		if (cpuType_ == CPU1802)  return ERROR_CPU_1802;
		if (cpuType_ == CPU1804)  return ERROR_CPU_1804;

		ret = getByte(assInput, b3, allowX);
		if (ret > ERROR_START)  return ret;

		*b1 = 0x68;
		*b2 = 0xff;
		return 3;
	}
	return ERROR_1802;
}

int DebugWindow::getByte(AssInput assInput, Byte* b2, bool allowX)
{
	if (assInput.numberOfParameters > 1)
		return ERROR_PAR;

	if (assInput.parameterType[0] == ASS_HEX_VALUE)
	{
		if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xff)
			return ERROR_8BIT;

		*b2 = assInput.parameterValue[0];
		return 2;
	}
	if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "X" && allowX)
	{
		return 1;
	}
	return ERROR_8BIT;
}

int DebugWindow::getWord(AssInput assInput, Byte* b2, Byte* b3, bool allowX)
{
	if (assInput.numberOfParameters > 1)
		return ERROR_PAR;

	if (assInput.parameterType[0] == ASS_HEX_VALUE)
	{
		if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xffff)
			return ERROR_16BIT;

		*b2 = assInput.parameterValue[0] >> 8;
		*b3 = assInput.parameterValue[0] & 0xff;
		return 3;
	}
	if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "X" && allowX)
	{
		return 1;
	}
	return ERROR_8BIT;
}

int DebugWindow::getWordPar2(AssInput assInput, Byte* b3, Byte* b4, bool allowX)
{
	if (assInput.numberOfParameters > 2)
		return ERROR_PAR;

	if (assInput.parameterType[1] == ASS_HEX_VALUE)
	{
		if (assInput.parameterValue[1] < 0 || assInput.parameterValue[1] > 0xffff)
			return ERROR_16BIT;

		*b3 = assInput.parameterValue[1] >> 8;
		*b4 = assInput.parameterValue[1] & 0xff;
		return 4;
	}
	if (assInput.parameterType[1] == ASS_STRING && assInput.parameterString[1] == "X" && allowX)
	{
		return 2;
	}
	return ERROR_8BIT;
}

int DebugWindow::getRegisterNumber(AssInput assInput, long* registerNumber, Byte* b5, bool allowX)
{
	if (assInput.parameterType[0] == ASS_HEX_VALUE)
	{
		if (assInput.parameterValue[0] < 0 || assInput.parameterValue[0] > 0xf)
			return ERROR_REG;
		*registerNumber = assInput.parameterValue[0];
		return 1;
	}
	if (assInput.parameterType[0] == ASS_REG)
	{
		*registerNumber = assInput.parameterValue[0];
		return 1;
	}
	if (assInput.parameterType[0] == ASS_STRING && assInput.parameterString[0] == "X" && allowX) 
	{
		*registerNumber = 0;
		*b5 = 0xf0;
		return 1;
	}
	return ERROR_REG;
}

int DebugWindow::translateChipParameter(wxString buffer, long* value, int* type)
{
	if (buffer.Left(1) == "V")
	{
		buffer = buffer.Right(buffer.Len()-1);
		if (!buffer.ToLong(value, 16))
			return ERROR_REG;
		if (*value < 0 || *value > 15)
			return ERROR_REG;
		*type = CHIP8_VX;
		return 0;
	}
	if (buffer.Left(1) == "R")
	{
		buffer = buffer.Right(buffer.Len()-1);
		if (!buffer.ToLong(value, 16))
			return ERROR_REG;
		if (*value < 0 || *value > 15)
			return ERROR_REG;
		*type = ASS_REG;
		return 0;
	}
	if (buffer.Left(2) == "[V" && buffer.Right(1)== "]")
	{
		buffer = buffer.Mid(2, 1);
		if (!buffer.ToLong(value, 16))
			return ERROR_REG;
		if (*value < 0 || *value > 15)
			return ERROR_REG;
		*type = CHIP8_VX_MEM;
		return 0;
	}
	if (buffer.Left(1) == "D")
	{
		buffer = buffer.Right(buffer.Len()-1);
		if (!buffer.ToLong(value, 10))
			return ERROR_DEC;
		*type = ASS_HEX_VALUE;
		return 0;
	}
	if (!buffer.ToLong(value, 16))
		*type = ASS_STRING;
	else
		*type = ASS_HEX_VALUE;
	return 0;
}

int DebugWindow::getRegister(wxString buffer)
{
	if (buffer == "D")  return TREG_D;
	if (buffer == "DF")  return TREG_DF;
	if (buffer == "P")  return TREG_P;
	if (buffer == "X")  return TREG_X;
	if (buffer == "T")  return TREG_T;
	if (buffer == "Q")  return TREG_Q;
	if (buffer == "R0")  return TREG_R0;
	if (buffer == "R1")  return TREG_R1;
	if (buffer == "R2")  return TREG_R2;
	if (buffer == "R3")  return TREG_R3;
	if (buffer == "R4")  return TREG_R4;
	if (buffer == "R5")  return TREG_R5;
	if (buffer == "R6")  return TREG_R6;
	if (buffer == "R7")  return TREG_R7;
	if (buffer == "R8")  return TREG_R8;
	if (buffer == "R9")  return TREG_R9;
	if (buffer == "RA")  return TREG_RA;
	if (buffer == "RB")  return TREG_RB;
	if (buffer == "RC")  return TREG_RC;
	if (buffer == "RD")  return TREG_RD;
	if (buffer == "RE")  return TREG_RE;
	if (buffer == "RF")  return TREG_RF;
	return TREG_FAULT;
}

void DebugWindow::onEnter(wxCommandEvent&WXUNUSED(event))
{
	wxString debugIn, address, error;
	Byte b1,b2,b3=0,b4=0,b5;
	int count;
	Word addressValue = debugAddress_;

	debugIn = inputWindowPointer->GetValue();
	debugIn = debugIn.MakeUpper();
	debugIn = debugIn.Trim(false);

	if (debugIn == "")  return;

	if (XRCCTRL(*this,"DebugDisChip8", wxToggleButton)->GetValue())
	{
		if (p_Computer->getChip8Type() == CHIPST2)
			count = assembleSt2(&debugIn, &b1, &b2);
		else
			count = assembleChip(&debugIn, &b1, &b2);
	}
	else
		count = assemble(&debugIn, &b1, &b2, &b3, &b4, &b5, false);

	if (count > 0 && count < 5)
	{
		p_Computer->writeMem(debugAddress_++, b1, true);
		if (count > 1) p_Computer->writeMem(debugAddress_++, b2, true);
		if (count > 2) p_Computer->writeMem(debugAddress_++, b3, true);
		if (count > 3) p_Computer->writeMem(debugAddress_++, b4, true);


		if (XRCCTRL(*this,"DebugDisChip8", wxToggleButton)->GetValue())
		{
			assemblerDisplay(disassembleChip8(&addressValue));
		}
		else
			assemblerDisplay(cdp1802disassemble(&addressValue, false, true));

		address.Printf("%04X", (unsigned int)debugAddress_);
		XRCCTRL(*this,"DebugAssemblerAddress",wxTextCtrl)->SetValue(address);

		inputWindowPointer->Clear();
	}
	else
	{
		assemblerDisplay(error_code[count-ERROR_START-1]);
	}
}

void DebugWindow::onDebugDis(wxCommandEvent&WXUNUSED(event))
{
	long start = get16BitValue("DebugDisStart");
	if (start == -1)  return;

	long end = get16BitValue("DebugDisEnd");
	if (end == -1)  return;

	if (XRCCTRL(*this,"DebugDisChip8", wxToggleButton)->GetValue())
		disassembleChip8(start, end);
	else
		disassemble(start, end);
}

void DebugWindow::onDebugSaveDump(wxCommandEvent&WXUNUSED(event))
{
	int num = 0;
	long addr;
	Byte value;
	wxFile outputFile;
	wxTextFile outputTextFile;
	wxString fileName, memoryStr, number, strValue, line;

	switch (memoryDisplay_)
	{
		case CPU_MEMORY:
			memoryStr = "CPU Memory";
			fileName = "memorydump";
		break;

		case CDP_1870_C:
			memoryStr = "CDP 1870 Character Ram";
			fileName = "cdp1870charramdump";
		break;

		case CDP_1870_P:
			memoryStr = "CDP 1870 Page Ram";
			fileName = "cdp1870pageramdump";
		break;

		case TMS_MEMORY:
			memoryStr = "TMS video Ram";
			fileName = "tmsramdump";
		break;

		case VT_RAM:
			memoryStr = "VT video Ram";
			fileName = "vtramdump";
		break;

		case CDP_1864:
			memoryStr = "CDP 1864 Color Ram";
			fileName = "cdp1864colorramdump";
		break;

		case V_6845:
			memoryStr = "MC6845 Char Rom";
			fileName = "mc6845charromdump";
		break;

		case V_6847:
			memoryStr = "MC6847 Char Rom";
			fileName = "mc6847charromdump";
		break;

		case V_6847_RAM:
			memoryStr = "MC6847 video Ram";
			fileName = "mc6847ramdump";
		break;

		case I_8275:
			memoryStr = "Intel 8275 Char Rom";
			fileName = "i8275charromdump";
		break;
	}

	fileName = wxFileSelector( "Select the " + memoryStr + " dump file to save",
                               debugDir_, fileName,
                               "bin|txt",
                               wxString::Format
                              (
                                   "Binary File (*.bin)|*.bin|Text File (*.txt)|*.txt"
                               ),
                               wxFD_SAVE|wxFD_CHANGE_DIR,
                               this
                              );

	if (!fileName || fileName.empty())
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	wxString name = FullPath.GetName();
	wxString path = FullPath.GetPath();
	wxString ext = FullPath.GetExt();

	while(wxFile::Exists(fileName))
	{
		num++;
		number.Printf("%d", num);
		fileName = path + pathSeparator_ + name + "." + number + "." + ext;
	}

	if (ext == "bin")
	{
		outputFile.Create(fileName);
		for (long address = 0; address <= getAddressMask(); address++)
		{
			value = debugReadMem(address);
			outputFile.Write(&value, 1);
		}
		outputFile.Close();
	}
	else
	{
		outputTextFile.Create(fileName);
		addr = 0;
		while(addr <= getAddressMask())
		{
			line.Printf("%04X:", (unsigned int)addr);
			for (int i=0; i<16; i++)
			{
				strValue.Printf(" %02X", debugReadMem(addr+i));
				line = line + strValue;
			}

			outputTextFile.AddLine(line);
			addr += 16;
		}
		outputTextFile.Write();
		outputTextFile.Close();
	}
}

void DebugWindow::onPauseButton(wxCommandEvent&WXUNUSED(event))
{
	if (p_Computer->getSteps() < 0)
		p_Computer->setSteps(0);
	else
		p_Computer->setSteps(-1);
	setPauseState();
}

void DebugWindow::setPauseState()
{
	if (p_Computer->getSteps() == 0)
	{
		XRCCTRL(*this, "DebugPauseButton", wxBitmapButton)->SetBitmapLabel(pauseOnBitmap);
		XRCCTRL(*this,"DebugStepButton", wxBitmapButton)->Enable(true);
	}
	else
	{
		if (p_Computer->getSteps() > 0)
		{
			XRCCTRL(*this, "DebugPauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);
			XRCCTRL(*this,"DebugStepButton", wxBitmapButton)->Enable(true);
		}
		else
		{
			XRCCTRL(*this, "DebugPauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);
			XRCCTRL(*this,"DebugStepButton", wxBitmapButton)->Enable(false);
		}
	}
	updateTitle();
}

void DebugWindow::onStepButton(wxCommandEvent&WXUNUSED(event))
{
	performStep_ = true;
}

void DebugWindow::onRunButton(wxCommandEvent&WXUNUSED(event))
{
	p_Computer->setSteps(-1);
	setPauseState();

	long runAddress = get16BitValue("DebugRunAddress");
	if (runAddress == -1)  return;

	p_Computer->setScratchpadRegister(p_Computer->getProgramCounter(), runAddress);
}

void DebugWindow::onRunAddress(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugRunAddress");
}

void DebugWindow::onBreakPointAddress(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("BreakPointAddress");
}

void DebugWindow::onTregValue(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("TregValue");
}

void DebugWindow::onNumberOfSteps(wxCommandEvent&WXUNUSED(event))
{
	wxString stepsStr = XRCCTRL(*this,"NumberOfSteps",wxTextCtrl)->GetValue();
	if (!stepsStr.ToLong(&steps_))
		steps_ = 1;
}

void DebugWindow::SetDebugMode()
{
	if (trace_ || traceTrap_ || traceDma_ || traceInt_ || (numberOfBreakPoints_ > 0) || (numberOfTraps_ > 0) || (numberOfTregs_ > 0))
		updateDebugMenu(true);
	else
		updateDebugMenu(false);
}

void DebugWindow::SetChip8DebugMode()
{
	if (chip8Trace_|| traceChip8Int_ || (numberOfChip8BreakPoints_ > 0))
		updateChip8DebugMenu(true);
	else
		updateChip8DebugMenu(false);
}

void DebugWindow::onLog(wxCommandEvent& WXUNUSED(event))
{
//	wxSetWorkingDirectory (workingDir_);
	int num = 0;
	wxString fileName, number;

	fileName = wxFileSelector( "Select the log file to save",
                               debugDir_, "trace.log",
                               "log",
                               wxString::Format
                              (
                                   "Dump File (*.log)|*.log|All files (%s)|%s",
                                   wxFileSelectorDefaultWildcardStr,
                                   wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_SAVE|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );

	if (!fileName || fileName.empty())
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	wxString name = FullPath.GetName();
	wxString path = FullPath.GetPath();
	wxString ext = FullPath.GetExt();

	while(wxFile::Exists(fileName))
	{
		num++;
		number.Printf("%d", num);
		fileName = path + pathSeparator_ + name + "." + number + "." + ext;
	}
	traceWindowPointer->SaveFile(fileName);
}

void DebugWindow::onDebugDisLog(wxCommandEvent& WXUNUSED(event))
{
//	wxSetWorkingDirectory (workingDir_);
	int num = 0;
	wxString fileName, number;

	fileName = wxFileSelector( "Select the log file to save",
                               debugDir_, "disassembler.log",
                               "log",
                               wxString::Format
                              (
                                   "Dump File (*.log)|*.log|All files (%s)|%s",
                                   wxFileSelectorDefaultWildcardStr,
                                   wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_SAVE|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );

	if (!fileName || fileName.empty())
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	wxString name = FullPath.GetName();
	wxString path = FullPath.GetPath();
	wxString ext = FullPath.GetExt();

	while(wxFile::Exists(fileName))
	{
		num++;
		number.Printf("%d", num);
		fileName = path + pathSeparator_ + name + "." + number + "." + ext;
	}
	disassemblerWindowPointer->SaveFile(fileName);
}

void DebugWindow::onDebugDisChip8(wxCommandEvent& WXUNUSED(event))
{
	enableDebugGui(true);
}

void DebugWindow::onClear(wxCommandEvent& WXUNUSED(event))
{
	traceWindowPointer->Clear();
}

void DebugWindow::onTrace(wxCommandEvent& WXUNUSED(event))
{
	trace_ = !trace_;
	if (computerRunning_)
	{
		p_Computer->setTraceStatus(trace_);
		enableDebugGui(true);
	}
	SetDebugMode();
}

void DebugWindow::onTraceDma(wxCommandEvent& WXUNUSED(event))
{
	traceDma_ = !traceDma_;
	if (computerRunning_)
	{
		p_Computer->setDmaTraceStatus(traceDma_);
		enableDebugGui(true);
	}
	SetDebugMode();
}

void DebugWindow::onTraceInt(wxCommandEvent& WXUNUSED(event))
{
	traceInt_ = !traceInt_;
	if (computerRunning_)
	{
		p_Computer->setIntTraceStatus(traceInt_);
		enableDebugGui(true);
	}
	SetDebugMode();
}

void DebugWindow::onChip8TraceInt(wxCommandEvent& WXUNUSED(event))
{
	traceChip8Int_ = !traceChip8Int_;
	if (computerRunning_)
	{
		p_Computer->setChip8IntTraceStatus(traceChip8Int_);
		enableChip8DebugGui(true);
	}
	SetChip8DebugMode();
}

void DebugWindow::onTraceTrap(wxCommandEvent& WXUNUSED(event))
{
	traceTrap_ = !traceTrap_;
	if (traceTrap_)
		XRCCTRL(*this, "TrapWindowText", wxStaticText)->SetLabel("Instr. Trace:");
	else
		XRCCTRL(*this, "TrapWindowText", wxStaticText)->SetLabel("Instr. Traps:");
	enableDebugGui(true);
	SetDebugMode();
}

void DebugWindow::onInt(wxCommandEvent& WXUNUSED(event))
{
	p_Computer->interrupt();
}

void DebugWindow::onBreakPointSet(wxCommandEvent&WXUNUSED(event))
{
	if (numberOfBreakPoints_ == 64)
	{
		(void)wxMessageBox( "Maximum number of Break Points set\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	long breakPointAddress = get16BitValue("BreakPointAddress");
	if (breakPointAddress == -1)
	{
		(void)wxMessageBox( "No Break Point value specified\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	breakPoints_[numberOfBreakPoints_] = breakPointAddress;
	breakPointsSelected_[numberOfBreakPoints_] = true;
	addBreakPoint();
	SetDebugMode();
}

void DebugWindow::onTregSet(wxCommandEvent&WXUNUSED(event))
{
	wxString printBuffer, strValue;

	char reg = XRCCTRL(*this,"TregRegister",wxChoice)->GetCurrentSelection();

	if ((reg >= TREG_D) && (reg <= TREG_RF))
	{
		long tregValue = get16BitValue("TregValue");;
		if (tregValue == -1)
		{
			(void)wxMessageBox( "No Register Value value specified\n",
										"Emma 02", wxICON_ERROR | wxOK );
			return;
		}

		if (numberOfTregs_ == 64)
		{
			(void)wxMessageBox( "Maximum number of Register Traps set\n",
										"Emma 02", wxICON_ERROR | wxOK );
			return;
		}

		if ((reg >= TREG_R0) && (reg <= TREG_RF))
		{
			if (tregValue > 0xffff)
			{
				(void)wxMessageBox( "Please specify value of 16 bit max\n",
											"Emma 02", wxICON_ERROR | wxOK );
				return;
			}
		}
		else
		{
			if (reg <= TREG_T)
			{
				if (tregValue > 0xff)
				{
					(void)wxMessageBox( "Please specify value of 8 bit max\n",
												"Emma 02", wxICON_ERROR | wxOK );
					return;
				}
			}
			else
			{
				if (tregValue > 1)
				{
					(void)wxMessageBox( "Please specify value of 0 or 1\n",
												"Emma 02", wxICON_ERROR | wxOK );
					return;
				}
			}
		}

		tregs_[numberOfTregs_][0] = reg;
		tregs_[numberOfTregs_][1] = tregValue;
		tregsSelected_[numberOfTregs_] = true;
		addTreg();
		SetDebugMode();
	}
	else
	{
		(void)wxMessageBox( "No register specified\n",
									"Emma 02", wxICON_ERROR | wxOK );
	}

}

void DebugWindow::onTrapCommand(wxCommandEvent&event)
{
	int command = event.GetSelection();
	wxString value = XRCCTRL(*this, "TrapValue", wxTextCtrl)->GetValue();

	if (opCode2[command] == 0x30)
	{
		XRCCTRL(*this, "TrapValue", wxTextCtrl)->Enable(numberOfBytes[command] > 1);
		if (numberOfBytes[command] == 2)
		{
			XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetMaxLength(2);
			if (value != "")
				XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetValue(value.Right(2));
		}
		if (numberOfBytes[command] == 3)
			XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetMaxLength(4);
	}
	else
	{
		XRCCTRL(*this, "TrapValue", wxTextCtrl)->Enable(numberOfBytes[command] > 2);
		if (numberOfBytes[command] == 3)
		{
			XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetMaxLength(2);
			if (value != "")
				XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetValue(value.Right(2));
		}
		if (numberOfBytes[command] == 4)
			XRCCTRL(*this, "TrapValue", wxTextCtrl)->SetMaxLength(4);
	}

	XRCCTRL(*this, "TrapRegister", wxChoice)->Enable(useRegister[command]);
}

void DebugWindow::onTrapSet(wxCommandEvent&WXUNUSED(event))
{
	if (numberOfTraps_ == 64)
	{
		(void)wxMessageBox( "Maximum number of Command Traps set\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	int command = XRCCTRL(*this,"TrapCommand",wxChoice)->GetCurrentSelection();
	wxString strValue = XRCCTRL(*this,"TrapValue",wxTextCtrl)->GetValue(), errorCode;
	long trapValue;

	if ((command >= 0) && (command <= 112))
	{
		traps_[numberOfTraps_][5] = 0xff;
		traps_[numberOfTraps_][0] = numberOfBytes[command];
		traps_[numberOfTraps_][1] = opCode[command];

		if (opCode[command] == 0x68)
		{
			if (cpuType_ >= minCpuType[command])
			{
				traps_[numberOfTraps_][2] = opCode2[command];
				if (useRegister[command])
				{
					int registerValue = XRCCTRL(*this,"TrapRegister",wxChoice)->GetCurrentSelection()-1;
					if ((registerValue >= 0) && (registerValue <= 0xf))
					{
						traps_[numberOfTraps_][2] |= registerValue;
					}
					else
					{
						traps_[numberOfTraps_][5] = 0xf0;
					}
				}
				if (traps_[numberOfTraps_][0] == 3)
				{
					strValue = strValue.MakeUpper();
					if (strValue == "X")
					{
						traps_[numberOfTraps_][0] = 2;
					}
					else
					{
						if (!strValue.ToLong(&trapValue, 16))
						{
							(void)wxMessageBox( "Please specify value in hexadecimal\n",
														"Emma 02", wxICON_ERROR | wxOK );
							return;
						}
						if (trapValue > 0xff)
						{
							(void)wxMessageBox( "Please specify value of 8 bit max\n",
														"Emma 02", wxICON_ERROR | wxOK );
							return;
						}
						else
						{
							traps_[numberOfTraps_][3] = trapValue;
						}
					}
				}
				if (traps_[numberOfTraps_][0] == 4)
				{
					strValue = strValue.MakeUpper();
					if (strValue == "X")
					{
						traps_[numberOfTraps_][0] = 2;
					}
					else
					{
						if (!strValue.ToLong(&trapValue, 16))
						{
							(void)wxMessageBox( "Please specify value in hexadecimal\n",
														"Emma 02", wxICON_ERROR | wxOK );
							return;
						}
						if (trapValue > 0xffff)
						{
							(void)wxMessageBox( "Please specify value of 16 bit max\n",
														"Emma 02", wxICON_ERROR | wxOK );
							return;
						}
						else
						{
							traps_[numberOfTraps_][3] = (trapValue >> 8) & 0xff;
							traps_[numberOfTraps_][4] = trapValue & 0xff;
						}
					}
				}
			}
			else
			{
				if (cpuType_ == CPU1802)
					errorCode = "Not supported on CDP1802\n";
				if (cpuType_ == CPU1804)
					errorCode = "Not supported on CDP1804\n";
				(void)wxMessageBox( errorCode,
											"Emma 02", wxICON_ERROR | wxOK );
				return;
			}
		}
		else
		{
			if (useRegister[command])
			{
				int registerValue = XRCCTRL(*this,"TrapRegister",wxChoice)->GetCurrentSelection()-1;
				if ((registerValue >= 0) && (registerValue <= 0xf))
				{
					traps_[numberOfTraps_][1] |= registerValue;
				}
				else
				{
					traps_[numberOfTraps_][5] = 0xf0;
				}
			}
			if ((opCode[command] == 0x61) || (opCode[command] == 0x69))
			{
				strValue = strValue.MakeUpper();
				traps_[numberOfTraps_][0] = 1;
				if (strValue == "X")
				{
					traps_[numberOfTraps_][5] = 0xf8;
				}
				else
				{
					if (!strValue.ToLong(&trapValue))
					{
						(void)wxMessageBox( "Please specify value 1 to 7\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
					if ((trapValue >= 1) && (trapValue <= 7))
					{
						traps_[numberOfTraps_][1] = traps_[numberOfTraps_][1] + trapValue - 1;
					}
					else
					{
						(void)wxMessageBox( "Please specify value 1 to 7\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
				}
			}
			if (traps_[numberOfTraps_][0] == 2)
			{
				strValue = strValue.MakeUpper();
				if (strValue == "X")
				{
					traps_[numberOfTraps_][0] = 1;
				}
				else
				{
					if (!strValue.ToLong(&trapValue, 16))
					{
						(void)wxMessageBox( "Please specify value in hexadecimal\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
					if (trapValue > 0xff)
					{
						(void)wxMessageBox( "Please specify value of 8 bit max\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
					else
					{
						traps_[numberOfTraps_][2] = trapValue;
					}
				}
			}
			if (traps_[numberOfTraps_][0] == 3)
			{
				strValue = strValue.MakeUpper();
				if (strValue == "X")
				{
					traps_[numberOfTraps_][0] = 1;
				}
				else
				{
					if (!strValue.ToLong(&trapValue, 16))
					{
						(void)wxMessageBox( "Please specify value in hexadecimal\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
					if (trapValue > 0xffff)
					{
						(void)wxMessageBox( "Please specify value of 16 bit max\n",
													"Emma 02", wxICON_ERROR | wxOK );
						return;
					}
					else
					{
						traps_[numberOfTraps_][2] = (trapValue >> 8) & 0xff;
						traps_[numberOfTraps_][3] = trapValue & 0xff;
					}
				}
			}
		}
	}
	else
	{
		(void)wxMessageBox( "No command specified\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}
	trapsSelected_[numberOfTraps_] = true;
	addTrap();
	SetDebugMode();
}

void DebugWindow::D(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("D");
	if (value == -1)  return;

	p_Computer->setAccumulator(value);
}

void DebugWindow::P(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("P");
	if (value == -1)  return;


	p_Computer->setProgramCounter(value);
}

void DebugWindow::X(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("X");
	if (value == -1)  return;

	p_Computer->setDataPointer(value);
}

void DebugWindow::T(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("T");
	if (value == -1)  return;

	p_Computer->setRegisterT(value);
}

void DebugWindow::DF(wxCommandEvent&WXUNUSED(event))
{
	long value = getBitValue("DF");
	if (value == -1)  return;

	p_Computer->setDataFlag(value);
}

void DebugWindow::Q(wxCommandEvent&WXUNUSED(event))
{
	long value = getBitValue("Q");
	if (value == -1)  return;

	p_Computer->setFlipFlopQ(value);
}

void DebugWindow::IE(wxCommandEvent&WXUNUSED(event))
{
	long value = getBitValue("IE");
	if (value == -1)  return;

	p_Computer->setInterruptEnable(value);
}

void DebugWindow::chip8I(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("Chip8I");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(CHIP8_I, value);
}

void DebugWindow::Vx(wxCommandEvent&event)
{
	wxString buttonName = wxWindow::FindWindowById(event.GetId())->GetName();
	long value = get16BitValue(buttonName);
	if (value == -1)  return;

	wxString buttonNumber = buttonName.Last();
	long number;
	if (buttonNumber.ToLong(&number, 16))
		p_Computer->writeMem(p_Computer->getChip8baseVar() + number, value, false);
}

void DebugWindow::R0(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R0");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(0, value);
}

void DebugWindow::R1(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R1");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(1, value);
}

void DebugWindow::R2(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R2");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(2, value);
}

void DebugWindow::R3(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R3");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(3, value);
}

void DebugWindow::R4(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R4");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(4, value);
}

void DebugWindow::R5(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R5");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(5, value);
}

void DebugWindow::R6(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R6");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(6, value);
}

void DebugWindow::R7(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R7");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(7, value);
}

void DebugWindow::R8(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R8");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(8, value);
}

void DebugWindow::R9(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R9");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(9, value);
}

void DebugWindow::RA(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R10");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(10, value);
}

void DebugWindow::RB(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R11");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(11, value);
}

void DebugWindow::RC(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R12");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(12, value);
}

void DebugWindow::RD(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R13");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(13, value);
}

void DebugWindow::RE(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R14");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(14, value);
}

void DebugWindow::RF(wxCommandEvent&WXUNUSED(event))
{
	long value = get16BitValue("R15");
	if (value == -1)  return;

	p_Computer->setScratchpadRegister(15, value);
}

void DebugWindow::O1(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("O1");
	if (value == -1)  return;

	p_Computer->out(1, 0, value);
}

void DebugWindow::O2(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("O2");
	if (value == -1)  return;

	p_Computer->out(2, 0, value);
}

void DebugWindow::O3(wxCommandEvent&WXUNUSED(event))
{
	long value = get8BitValue("O3");
	if (value == -1)  return;

	p_Computer->out(3, 0, value);
}

void DebugWindow::O4(wxCommandEvent&WXUNUSED(event))
{
	long value;

	if (runningComputer_ == COMX || runningComputer_ == CIDELSA || runningComputer_ ==  TMC600 || runningComputer_ == PECOM)
	{
		value = get16BitValue("O4");
		if (value == -1)  return;
		p_Computer->out(4, value, 0);
	}
	else
	{
		value = get8BitValue("O4");
		if (value == -1)  return;
		p_Computer->out(4, 0, value);
	}
}

void DebugWindow::O5(wxCommandEvent&WXUNUSED(event))
{
	long value;

	if (runningComputer_ == COMX || runningComputer_ == CIDELSA || runningComputer_ ==  TMC600 || runningComputer_ == PECOM)
	{
		value = get16BitValue("O5");
		if (value == -1)  return;
		p_Computer->out(5, value, 0);
	}
	else
	{
		value = get8BitValue("O5");
		if (value == -1)  return;
		p_Computer->out(5, 0, value);
	}
}

void DebugWindow::O6(wxCommandEvent&WXUNUSED(event))
{
	long value;

	if (runningComputer_ == COMX || runningComputer_ == CIDELSA || runningComputer_ ==  TMC600 || runningComputer_ == PECOM)
	{
		value = get16BitValue("O6");
		if (value == -1)  return;
		p_Computer->out(6, value, 0);
	}
	else
	{
		value = get8BitValue("O6");
		if (value == -1)  return;
		p_Computer->out(6, 0, value);
	}
}

void DebugWindow::O7(wxCommandEvent&WXUNUSED(event))
{
	long value;

	if (runningComputer_ == COMX || runningComputer_ == CIDELSA || runningComputer_ ==  TMC600 || runningComputer_ == PECOM)
	{
		value = get16BitValue("O7");
		if (value == -1)  return;
		p_Computer->out(7, value, 0);
	}
	else
	{
		value = get8BitValue("O7");
		if (value == -1)  return;
		p_Computer->out(7, 0, value);
	}
}

void DebugWindow::onProtectedMode(wxCommandEvent&event)
{
	protectedMode_ = event.IsChecked();
	enableDebugGui(true);
}

void DebugWindow::onDebugDisplayPageSpinUp(wxSpinEvent&WXUNUSED(event))
{
	debugDisplayPageSpinUp();
}

void DebugWindow::debugDisplayPageSpinUp()
{
	long address = get16BitValue("DebugDisplayPage");
	if (address == -1)  return;

	address += 0x100;

	Word ramMask = getAddressMask();

	if (memoryDisplay_ == V_6847_RAM)
	{
		while (address >= ((ramMask+1)*2)) 
			address -=  ((ramMask + 1)*2);
	}
	else
	{
		while (address > ramMask)  
			address -=  (ramMask + 1);
	}

	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber((unsigned int)address);

	memoryDisplay();
}

void DebugWindow::onDebugDisplayPageSpinDown(wxSpinEvent&WXUNUSED(event))
{
	long address = get16BitValue("DebugDisplayPage");
	if (address == -1)  return;

	address -= 0x100;

	Word ramMask = getAddressMask();

	if (memoryDisplay_ == V_6847_RAM)
	{
		while (address < 0)  
			address +=  ((ramMask + 1)*2);
	}
	else
	{
		while (address < 0)  
			address +=  (ramMask + 1);
	}

	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber((unsigned int)address);

	memoryDisplay();
}

void DebugWindow::onDebugDisplayPage(wxCommandEvent&WXUNUSED(event))
{
	if (xmlLoaded_)
		p_Main->updateMemoryTab();
//	memoryDisplay();
}

void DebugWindow::DebugDisplayPage()
{
	if (!computerRunning_)
	{
		if (xmlLoaded_)
			XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("No emulation running");
		return;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	Word ramMask = getAddressMask();
	while (start > ramMask)
		start -=  (ramMask + 1);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	switch (runningComputer_)
	{
		case COMX:
			XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->changeNumber(p_Comx->getComxExpansionSlot()+1);
			if (p_Comx->isRamCardActive())
				XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->changeNumber(p_Comx->getComxExpansionRamBank());
			if (p_Comx->isEpromBoardLoaded())
				XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->changeNumber(p_Comx->getComxExpansionEpromBank());
		break;

		case ELF:
		case ELFII:
		case SUPERELF:
			if (elfConfiguration[runningComputer_].useEms)
				XRCCTRL(*this, "DebugEmsPage", HexEdit)->changeNumber(p_Computer->getEmsPage());
		break;
	}

	wxString idReference, value;

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)(start+x)&0xf);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);
		switch (p_Computer->getMemoryType(start/256))
		{
			case COMXEXPBOX:
				XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0x80, 0x80, 0xff));
			break;

			case EMSMEMORY:
				XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0xc8, 0xb4, 0x3e));
			break;

			case PAGER:
				if (((start>>12)&0xf) == portExtender_)
					XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0xa9, 0x3e, 0xac));
			break;
		}

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", p_Computer->readMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);
			if (p_Computer->getMemoryType(start/256) == COMXEXPBOX)
			{
					if (p_Computer->getExpansionMemoryType(p_Comx->getComxExpansionSlot(), (start&0x1fff)/256) == RAMBANK)
						XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(wxColour(0x45, 0xac, 0x22));
					if (p_Computer->getExpansionMemoryType(p_Comx->getComxExpansionSlot(), (start&0x1fff)/256) == EPROMBANK)
						XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(wxColour(0x80, 0x00, 0x40));
			}

			XRCCTRL(*this, idReference, MemEdit)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)
				start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::ShowCharacters(Word address, int y)
{
	wxString idReference;
	idReference.Printf("CHAR%01X", y);

	int t;
	char bits [9];

	dcLine.SelectObject(*lineBmp[y]);
	dcLine.DrawRectangle(0, 0, 128, 16);
	wxFont exactFont(7, wxFONTFAMILY_TELETYPE, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);

	for (int j=0; j<16; j++)
	{
		if ((runningComputer_ == COMX) || (runningComputer_ == TMC600) || (runningComputer_ == PECOM))
		{
			for (int i=0; i<9; i++)
			{
				if (runningComputer_ == COMX)
					t = p_Comx->readCramDirect((debugReadMem(address+j)&0x7f)*16+i);
				else if (runningComputer_ == TMC600)
					t = p_Tmc600->readCramDirect((debugReadMem(address+j)&0xff)*16+i);
				else
					t = p_Pecom->readCramDirect((debugReadMem(address+j)&0x7f)*16+i);
				bits[i] = (t & 0x1) << 5;
				bits[i] |= (t & 0x2) << 3;
				bits[i] |= (t & 0x4) << 1;
				bits[i] |= (t & 0x8) >> 1;
				bits[i] |= (t & 0x10) >> 3;
				bits[i] |= (t & 0x20) >> 5;
			}
			wxBitmap character(bits, 6, 9, 1);
			dcChar.SelectObject(character);
			dcLine.Blit(j*8+1, 5, 6, 9, &dcChar, 0, 0, wxAND);
			dcChar.SelectObject(wxNullBitmap);
		}
		else
		{
			wxString character;
			Byte byteValue = debugReadMem(address+j)&0x7f;
			character.Printf("%c", byteValue);
			dcLine.SetFont(exactFont);
/*			int offset = 1;
			if (byteValue == 0x4d || byteValue == 0x57)
				offset = 0;
			if (byteValue == 0x49)
				offset = 3;
			if (byteValue == 0x54)
				offset = 2;*/
			dcLine.DrawText(character, (j*8), 1);
		}
	}

	dcLine.SelectObject(wxNullBitmap);
	XRCCTRL(*this, idReference, wxStaticBitmap)->SetForegroundColour(*wxBLACK);
	XRCCTRL(*this, idReference, wxStaticBitmap)->SetBitmap(*lineBmp[y]);
}

void DebugWindow::DebugDisplayMap()
{
	if (!computerRunning_)
	{
		if (xmlLoaded_)
			XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("No emulation running");
		return;
	}

	wxString idReference, idReference2, value;
	bool textGreen, textOrange;

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");

	switch (runningComputer_)
	{
		case COMX:
			XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->changeNumber(p_Comx->getComxExpansionSlot()+1);
			if (p_Comx->isRamCardActive())
				XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->changeNumber(p_Comx->getComxExpansionRamBank());
			if (p_Comx->isEpromBoardLoaded())
				XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->changeNumber(p_Comx->getComxExpansionEpromBank());
		break;

		case ELF:
		case ELFII:
		case SUPERELF:
			if (elfConfiguration[runningComputer_].useEms)
				XRCCTRL(*this, "DebugEmsPage", HexEdit)->changeNumber(p_Computer->getEmsPage());
		break;
	}

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", x);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		value.Printf("%04X", y*4096);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		for (int x=0; x<16; x++)
		{
			textGreen = false;
			textOrange = false;
			switch (p_Computer->getMemoryType((y<<4)+x))
			{
				case UNDEFINED:
					value.Printf (" ");
				break;

				case RAM:
					value.Printf (".");
				break;

				case MAPPEDRAM:
					value.Printf ("M.");
				break;

				case VP570RAM:
					value.Printf ("E.");
				break;

				case COLOURRAM:
					value.Printf ("C.");
				break;

				case COMXEXPROM:
				case ROM:
					value.Printf ("R");
				break;

				case CARTRIDGEROM:
				case CRAM1870:
					value.Printf ("CR");
				break;

				case PRAM1870:
					value.Printf ("PR");
				break;

				case COMXEXPBOX:
					XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0x80, 0x80, 0xff));
					switch (p_Computer->getExpansionMemoryType(p_Comx->getComxExpansionSlot(), (y&1)*16+x))
					{
						case MC6845RAM:
							value.Printf ("M5");
						break;

						case MC6845REGISTERS:
							value.Printf ("MR");
						break;

						case RAMBANK:
							textGreen = true;
							switch (p_Computer->getBankMemoryType(p_Comx->getComxExpansionRamBank(), (y&1)*16+x))
							{
								case RAM:
									value.Printf (".");
								break;

								case ROM:
									value.Printf ("R");
								break;

								case UNDEFINED:
									value.Printf (" ");
								break;

								default:
									value.Printf ("xx");
								break;
							}
						break;

						case EPROMBANK:
							textOrange = true;
							switch (p_Computer->getEpromBankMemoryType(p_Comx->getComxExpansionEpromBank(), (y&1)*16+x))
							{
								case RAM:
									value.Printf (".");
								break;

								case ROM:
									value.Printf ("R");
								break;

								case UNDEFINED:
									value.Printf (" ");
								break;

								default:
									value.Printf ("xx");
								break;
							}
						break;

						case RAM:
							value.Printf (".");
						break;

						case ROM:
							value.Printf ("R");
						break;

						case UNDEFINED:
							value.Printf (" ");
						break;

						default:
							value.Printf ("xx");
						break;
					}
				break;

				case COPYFLOPROM:
					value.Printf ("CF");
				break;

				case COPYCOMXEXPROM:
					value.Printf ("CE");
				break;

				case EMSMEMORY:
					XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0xc8, 0xb4, 0x3e));

					switch (p_Computer->getEmsMemoryType((y&3)*16+x))
					{
						case RAM:
							value.Printf (".");
						break;

						case ROM:
							value.Printf ("R");
						break;

						case UNDEFINED:
							value.Printf (" ");
						break;

						default:
							value.Printf ("xx");
						break;
					}
				break;

				case PAGER:
					if (y == portExtender_)
						XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(wxColour(0xa9, 0x3e, 0xac));
					switch (p_Computer->getPagerMemoryType(y*16+x))
					{
						case RAM:
							value.Printf (".");
						break;

						case ROM:
							value.Printf ("R");
						break;

						case UNDEFINED:
							value.Printf (" ");
						break;

						default:
							value.Printf ("xx");
						break;
					}
				break;

				case MC6847RAM:
					value.Printf ("M7");
				break;

				case MC6845RAM:
					value.Printf ("M5");
				break;

				case MC6845REGISTERS:
					value.Printf ("MR");
				break;

				default:
					value.Printf ("xx");
				break;
			}
			idReference2.Printf("MEM%01X%01X", y, x);
			XRCCTRL(*this, idReference2, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference2, MemEdit)->ChangeValue(value);
			XRCCTRL(*this, idReference2, MemEdit)->saveNumber(-1);
			if (textGreen)
				XRCCTRL(*this, idReference2, MemEdit)->SetForegroundColour(wxColour(0x45, 0xac, 0x22));
			else
			{
				if (textOrange)
					XRCCTRL(*this, idReference2, MemEdit)->SetForegroundColour(wxColour(0x80, 0x00, 0x40));
				else
					XRCCTRL(*this, idReference2, MemEdit)->SetForegroundColour(*wxBLACK);
			}
		}

		idReference.Printf("CHAR%01X", y);
		wxBitmap line(128, 16, 24);
		wxMemoryDC dcLine;

		dcLine.SelectObject(line);
#if defined(__WXGTK__)
	dcLine.SetPen(wxPen(wxColour(0xfb, 0xf8, 0xf1)));
	dcLine.SetBrush(wxBrush(wxColour(0xfb, 0xf8, 0xf1)));
#else
	dcLine.SetPen(*wxWHITE_PEN);
	dcLine.SetBrush(*wxWHITE_BRUSH);
#endif
		dcLine.DrawRectangle(0, 0, 128, 16);

		dcLine.SelectObject(wxNullBitmap);
		XRCCTRL(*this, idReference, wxStaticBitmap)->SetBitmap(line);
	}
}

void DebugWindow::DebugDisplay1870VideoRam()
{
	if (!(runningComputer_ == COMX || runningComputer_ == CIDELSA || runningComputer_ ==  TMC600 || runningComputer_ == PECOM))
	{
		if (xmlLoaded_)
			XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("CDP 1870 not running");
		return;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)(start+x)&0xf);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", debugReadMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplay1864ColorRam()
{
	if (!(runningComputer_ == TMC2000 || runningComputer_ == VIP || runningComputer_ ==  ETI))
	{
		if (xmlLoaded_)
			XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("1864 Color RAM not used");
		return;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", debugReadMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplay6845CharRom()
{
	switch (runningComputer_)
	{
		case COMX:
		break;

		case ELF:
		case ELFII:
		case SUPERELF:
			if (!(elfConfiguration[runningComputer_].use6845 || elfConfiguration[runningComputer_].useS100))  
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6845 not running");
				return;
			}
		break;

		default:
			if (xmlLoaded_)
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6845 not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", debugReadMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplay8275CharRom()
{
	switch (runningComputer_)
	{
		case ELF:
		case ELFII:
		case SUPERELF:
		case ELF2K:
			if (!elfConfiguration[runningComputer_].use8275)  
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Intel 8275 not running");
				return;
			}
		break;

		default:
			XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Intel 8275 not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", debugReadMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplay6847CharRom()
{
	switch (runningComputer_)
	{
		case ELF:
		case ELFII:
		case SUPERELF:
			if (!elfConfiguration[runningComputer_].use6847)
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6847 not running");
				return;
			}
		break;

		default:
			if (xmlLoaded_)
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6847 not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	if (start < 0x200)
		XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Showing: Internal Rom");
	else
		XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Showing: External Rom");

	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);
			value.Printf("%02X", debugReadMem(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplay6847VideoRam()
{
	switch (runningComputer_)
	{
		case ELF:
		case ELFII:
		case SUPERELF:
			if (!elfConfiguration[runningComputer_].use6847)
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6847 not running");
				return;
			}
		break;

		default:
			if (xmlLoaded_)
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("MC6847 not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start >= ((ramMask+1)*2)) 
		start -=  ((ramMask+1)*2);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	if (start <= ramMask)
		XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Showing: Bit 0 to 7");
	else
		XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("Showing: Bit 8 to 11");

	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			if (start <= ramMask)
			{
				XRCCTRL(*this, idReference, MemEdit)->saveNumber(-1);
				XRCCTRL(*this, idReference, MemEdit)->changeNumber2X(debugReadMem(start));
			}
			else
			{
				XRCCTRL(*this, idReference, MemEdit)->saveNumber(-1);
				XRCCTRL(*this, idReference, MemEdit)->changeNumber1X(debugReadMem(start));
			}

			start++;
			while (start >= ((ramMask+1)*2))  
				start -=  ((ramMask+1)*2);
		}
	}
}

void DebugWindow::DebugDisplayTmsRam()
{
	switch (runningComputer_)
	{
		case ELF:
		case ELFII:
		case SUPERELF:
			if (!elfConfiguration[runningComputer_].useTMS9918)
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("TMS 9918 not running");
				return;
			}
		break;

		default:
			if (xmlLoaded_)
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("TMS 9918 not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);
			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");

			switch(runningComputer_)
			{
				case ELF:
					value.Printf("%02X", p_Elf->getTmsMemory(start));
				break;

				case ELFII:
					value.Printf("%02X", p_Elf2->getTmsMemory(start));
				break;

				case SUPERELF:
					value.Printf("%02X", p_Super->getTmsMemory(start));
				break;
			}
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::DebugDisplayVtRam()
{
	switch (runningComputer_)
	{
		case ELF:
		case ELFII:
		case ELF2K:
		case COSMICOS:
		case MEMBER:
		case SUPERELF:
			if (!elfConfiguration[runningComputer_].vtType != VTNONE)
			{
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("VT not running");
				return;
			}
		break;

		default:
			if (xmlLoaded_)
				XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("VT not running");
			return;
		break;
	}

	long start = get16BitValue("DebugDisplayPage");
	if (start == -1)  return;
	XRCCTRL(*this, "DebugDisplayPage", HexEdit)->saveNumber(start);

	wxString idReference, value;

	Word ramMask = getAddressMask();
	while (start > ramMask)  
		start -=  (ramMask + 1);

	memoryStart_ = (unsigned int)start;
	p_Computer->setDebugMemoryStart(start);

	XRCCTRL(*this, "MEM_Message", wxStaticText)->SetLabel("");
	//XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(memoryStart_);

	for (int x=0; x<16; x++)
	{
		idReference.Printf("TOP_HEADER%01X", x);
		value.Printf("  %01X", (unsigned int)((start+x)&0xf));
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);
	}
	for (int y=0; y<16; y++)
	{
		idReference.Printf("MEM_HEADER%01X", y);
		XRCCTRL(*this, idReference, wxStaticText)->SetForegroundColour(*wxBLACK);

		value.Printf("%04X", (unsigned int)start);
		XRCCTRL(*this, idReference, wxStaticText)->SetLabel(value);

		ShowCharacters(start, y);
		for (int x=0; x<16; x++)
		{
			idReference.Printf("MEM%01X%01X", y, x);

			value.Printf("%02X", p_Vt100->getVtMemory(start));

			XRCCTRL(*this, idReference, MemEdit)->SetForegroundColour(*wxBLACK);

			XRCCTRL(*this, idReference, wxTextCtrl)->ChangeValue("");
			XRCCTRL(*this, idReference, MemEdit)->ChangeValue(value);

			start++;
			while (start > ramMask)  
			start -=  (ramMask + 1);
		}
	}
}

void DebugWindow::onEditMemory(wxCommandEvent&event)
{
	long address, value, id;
	wxString strValue, idReference, buttonNumber;

	if (!computerRunning_)
	{
		(void)wxMessageBox( "Memory edit not possible as no machine is running\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	idReference = wxWindow::FindWindowById(event.GetId())->GetName();
	buttonNumber = idReference.Right(2);
	if (!buttonNumber.ToLong(&id, 16))
		return;

	switch (memoryDisplay_)
	{
		case CPU_TYPE:
			strValue = XRCCTRL(*this, idReference, wxTextCtrl)->GetValue();
			strValue = strValue.MakeUpper();
			if (strValue != " ")
			{
				strValue.Trim(false);
				strValue.Trim(true);
			}

			if (strValue == "R")
				setMemoryType(id, ROM);
			else if (strValue == ".")
				setMemoryType(id, RAM);
			else if (strValue == "M.")
				setMemoryType(id, MAPPEDRAM);
			else if (strValue == "E.")
				setMemoryType(id, VP570RAM);
			else if (strValue == "C.")
				setMemoryType(id, COLOURRAM);
			else if (strValue == " ")
				setMemoryType(id, UNDEFINED);
			else if (strValue == "")
				setMemoryType(id, UNDEFINED);
			else if (strValue == "CR")
				setMemoryType(id, CRAM1870);
			else if (strValue == "PR")
				setMemoryType(id, PRAM1870);
			else if (strValue == "M7")
				setMemoryType(id, MC6847RAM);
			else if (strValue == "M5")
				setMemoryType(id, MC6845RAM);
			else if (strValue == "MR")
				setMemoryType(id, MC6845REGISTERS);
			else if (strValue == "CF")
				setMemoryType(id, COPYFLOPROM);
			else if (strValue == "CE")
				setMemoryType(id, COPYCOMXEXPROM);
			else if (strValue == "P")
				setMemoryType(id, UNDEFINED);
			else if (strValue == "C")
				setMemoryType(id, UNDEFINED);
			else if (strValue == "E")
				setMemoryType(id, UNDEFINED);
			else if (strValue == "M")
				setMemoryType(id, UNDEFINED);
			else
			{
				(void)wxMessageBox( 	"Please use one of the following codes:\n"
										"space = UNDEFINED\n"
										". = RAM\n"
										"R = ROM\n"
										"M. = Mapped RAM\n"
										"E. = VP570 Expansion RAM\n"
										"PR = 1870 Page RAM\n"
										"CR = 1870 Character RAM\n"
										"M7 = MC6847 Video RAM\n"
										"M5 = MC6845 Video RAM\n"
										"MR = MC6845 Register\n"
										"CE = COMX Expansion ROM copy\n"
										"CF = COMX Floppy disk ROM copy\n"
										"C. = Victory or Vip Colour RAM access\n"
										"\nNote: some options are only allowed\n"
										"in specific cases.\n",
											"Emma 02", wxICON_ERROR | wxOK );
				return;
			}

			XRCCTRL(*this, "DebugDisplayPage", HexEdit)->changeNumber(id*256);
		break;

		case CPU_MEMORY:
		case CDP_1870_C:
		case CDP_1870_P:
		case TMS_MEMORY:
		case VT_RAM:
		case CDP_1864:
		case V_6845:
		case I_8275:
		case V_6847:
		case V_6847_RAM:
			address = get16BitValue("DebugDisplayPage");
			if (address == -1)  return;

			value = get8BitValue(idReference);
			if (value == -1)  return;

			address += id;
			debugWriteMem(address, value);
			XRCCTRL(*this, idReference, MemEdit)->saveNumber(value);

			ShowCharacters(address-(id&0xf), (int)(id/16));
			
#if defined(__WXMSW__)
			wxString strValue = XRCCTRL(*this, idReference, MemEdit)->GetValue();
			if (strValue.Len() == 2)
			{
				if (id == 255)
				{
					debugDisplayPageSpinUp();
					XRCCTRL(*this, "MEM00", MemEdit)->SetFocus();
					XRCCTRL(*this, "MEM00", MemEdit)->SelectAll();
				}
				else	
				{
					wxKeyEvent keyEvent(wxEVT_KEY_DOWN); 
					keyEvent.m_rawCode = WXK_TAB; 
					XRCCTRL(*this, idReference, MemEdit)->EmulateKeyPress(keyEvent);
				}
			}
#endif
		break;
	}
}

void DebugWindow::setMemoryType(int id, int setType)
{
	switch (runningComputer_)
	{
		case COMX:
			if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED) || (setType == CRAM1870) || (setType == PRAM1870)|| (setType == COMXEXPBOX) || (setType == MC6845RAM) || (setType == MC6845REGISTERS) || (setType == COPYFLOPROM) || (setType == COPYCOMXEXPROM))
			{
				if (((setType == MC6845RAM) || (setType == MC6845REGISTERS)) && !p_Comx->isColumnRomLoaded())
				{
					(void)wxMessageBox( "No 80 column card configured\n",
												"Emma 02", wxICON_ERROR | wxOK );
					return;
				}
				if (p_Computer->getMemoryType(id) == COMXEXPBOX)
				{
					if (p_Computer->getExpansionMemoryType(p_Comx->getComxExpansionSlot(), id&0x1f) == RAMBANK)
					{
						if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
							p_Computer->defineBankMemoryType(p_Comx->getComxExpansionRamBank(), (id&0x1f)*256, setType);
						else
						{
							(void)wxMessageBox( "Only RAM (.), ROM (R) or UNDEFINED (space) allowed in 32K RAM Card\n",
														"Emma 02", wxICON_ERROR | wxOK );
						}
					}
					else
					{
						if (p_Computer->getExpansionMemoryType(p_Comx->getComxExpansionSlot(), id&0x1f) == EPROMBANK)
						{
							if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
								p_Computer->defineEpromBankMemoryType(p_Comx->getComxExpansionEpromBank(), (id&0x1f)*256, setType);
							else
							{
								(void)wxMessageBox( "Only RAM (.), ROM (R) or UNDEFINED (space) allowed in 32K RAM Card\n",
															"Emma 02", wxICON_ERROR | wxOK );
							}
						}
						else
						{
							if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED) || (setType == MC6845RAM) || (setType == MC6845REGISTERS))
								p_Computer->defineExpansionMemoryType(p_Comx->getComxExpansionSlot(), (id&0x1f)*256, setType);
							else
							{
								(void)wxMessageBox( "Only RAM (.), ROM (R), MC6845 (MR/M5) or UNDEFINED (space) allowed in COMX Expansion Slot\n",
															"Emma 02", wxICON_ERROR | wxOK );
							}
						}
					}
				}
				else
					p_Computer->defineMemoryType(id*256, setType);
			}
			else
			{
				(void)wxMessageBox( "Only RAM (.), ROM (R), CDP1870 (PR/CR), MC6845 (MR/M5), ROM Copy (CF/CE) or UNDEFINED (space) allowed in COMX emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case ELF:
		case ELFII:
		case SUPERELF:
			if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED) || (setType == MAPPEDRAM) || (setType == MC6847RAM) || (setType == MC6845RAM) || (setType == MC6845REGISTERS) )
			{
				if (!(elfConfiguration[runningComputer_].use6845 || elfConfiguration[runningComputer_].useS100) && ((setType == MC6845RAM) || (setType == MC6845REGISTERS)))
				{
					(void)wxMessageBox( "No MC6845 configured\n",
												"Emma 02", wxICON_ERROR | wxOK );
					return;
				}
				if (!elfConfiguration[runningComputer_].use6847 && (setType == MC6847RAM))
				{
					(void)wxMessageBox( "No MC6847 configured\n",
												"Emma 02", wxICON_ERROR | wxOK );
					return;
				}
				if (p_Computer->getMemoryType(id) == EMSMEMORY)
				{
					if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
						p_Computer->defineEmsMemoryType((id&0x3f)*256, setType);
					else
					{
						(void)wxMessageBox( "Only RAM (.), ROM (R), or UNDEFINED (space) allowed in EMS Memory bank\n",
													"Emma 02", wxICON_ERROR | wxOK );
					}
				}
				else if (p_Computer->getMemoryType(id) == PAGER)
				{
					if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
						p_Computer->definePagerMemoryType(id*256, setType);
					else
					{
						(void)wxMessageBox( "Only RAM (.), ROM (R), or UNDEFINED (space) allowed in Pager Memory\n",
													"Emma 02", wxICON_ERROR | wxOK );
					}
				}
				else
					p_Computer->defineMemoryType(id*256, setType);
			}
			else
			{
				(void)wxMessageBox( "Only RAM (.), Mapped RAM (M.), ROM (R), MC6845 (M5/MR), MC6847 (M7) or UNDEFINED (space) allowed in Elf emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case CIDELSA:
		case TMC600:
		case PECOM:
			if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED) || (setType == CRAM1870)|| (setType == PRAM1870))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), ROM (R), CDP1870 (PR/CR) or UNDEFINED (space) allowed in "+computerInfo[runningComputer_].name+" emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case VIP:
			if ((setType == RAM) || (setType == ROM) || (setType == COLOURRAM) || (setType == UNDEFINED) || (setType == MAPPEDRAM) || (setType == VP570RAM))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), ROM (R), Mapped RAM (M.), Colour RAM (C.), VP570 Expansion RAM (VP) or UNDEFINED (space) allowed in Cosmac VIP emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case ELF2K:
		case TMC2000:
		case TMC1800:
		case NANO:
		case MEMBER:
		case ETI:
			if ((setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), ROM (R), or UNDEFINED (space) allowed in "+computerInfo[runningComputer_].name+" emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case COSMICOS:
			if ((setType == MAPPEDRAM) || (setType == RAM) || (setType == ROM) || (setType == UNDEFINED))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), ROM (R), Main RAM (M.) or UNDEFINED (space) allowed in "+computerInfo[runningComputer_].name+" emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case VICTORY:
			if (setType == CRAM1870)
				setType = CARTRIDGEROM;
			if ((setType == RAM) || (setType == MAPPEDRAM) || (setType == COLOURRAM) || (setType == ROM) || (setType == CARTRIDGEROM) || (setType == UNDEFINED))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), Mapped RAM (M.), Colour RAM (C.), ROM (R), Cartridge ROM (CR) or UNDEFINED (space) allowed in "+computerInfo[runningComputer_].name+" emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;

		case VISICOM:
		case STUDIO:
			if (setType == CRAM1870)
				setType = CARTRIDGEROM;
			if ((setType == RAM) || (setType == MAPPEDRAM) || (setType == ROM) || (setType == CARTRIDGEROM) || (setType == UNDEFINED))
				p_Computer->defineMemoryType(id*256, setType);
			else
			{
				(void)wxMessageBox( "Only RAM (.), Mapped RAM (M.), ROM (R), Cartridge ROM (CR) or UNDEFINED (space) allowed in "+computerInfo[runningComputer_].name+" emulation\n",
											"Emma 02", wxICON_ERROR | wxOK );
			}
		break;
	}
}


void DebugWindow::memoryDisplay()
{
//	XRCCTRL(*this, "DebugDisplayPage", wxTextCtrl)->SetForegroundColour(*wxBLACK);
	switch (memoryDisplay_)
	{
		case CPU_MEMORY:
			DebugDisplayPage();
		break;

		case CPU_TYPE:
			DebugDisplayMap();
		break;

		case CDP_1870_C:
			DebugDisplay1870VideoRam();
		break;

		case CDP_1870_P:
			DebugDisplay1870VideoRam();
		break;

		case TMS_MEMORY:
			DebugDisplayTmsRam();
		break;

		case VT_RAM:
			DebugDisplayVtRam();
		break;

		case CDP_1864:
			DebugDisplay1864ColorRam();
		break;

		case I_8275:
			DebugDisplay8275CharRom();
		break;

		case V_6845:
			DebugDisplay6845CharRom();
		break;

		case V_6847:
			DebugDisplay6847CharRom();
		break;

		case V_6847_RAM:
			DebugDisplay6847VideoRam();
		break;
	}
}

Word DebugWindow::getAddressMask()
{
	switch (memoryDisplay_)
	{
		case CPU_MEMORY:
			return 0xffff;
		break;

		case CPU_TYPE:
			return 0xffff;
		break;

		case CDP_1870_C:
			switch (runningComputer_)
			{
				case COMX:
					return p_Comx->getCharacterMemoryMask();
				break;
				case CIDELSA:
					return p_Cidelsa->getCharacterMemoryMask();
				break;
				case TMC600:
					return p_Tmc600->getCharacterMemoryMask();
				break;
				case PECOM:
					return p_Pecom->getCharacterMemoryMask();
				break;
				default:
					return 0x7ff;
				break;
			}
		break;

		case CDP_1870_P:
			switch (runningComputer_)
			{
				case COMX:
					return p_Comx->getPageMemoryMask();
				break;
				case CIDELSA:
					return p_Cidelsa->getPageMemoryMask();
				break;
				case TMC600:
					return p_Tmc600->getPageMemoryMask();
				break;
				case PECOM:
					return p_Pecom->getPageMemoryMask();
				break;
				default:
					return 0x3ff;
				break;
			}
		break;

		case TMS_MEMORY:
			return 0x3fff;
		break;

		case CDP_1864:
			switch (runningComputer_)
			{
				case ETI:
				case VIP:
					return 0xff;
				break;
				default:
					return 0x3ff;
				break;
			}
		break;

		case I_8275:
			return 0x1fff;
		break;

		case VT_RAM:
		case V_6845:
		case V_6847:
			return 0x7ff;
		break;

		case V_6847_RAM:
			switch(runningComputer_)
			{
				case ELF:
					return p_Elf->get6847RamMask();
				break;

				case ELFII:
					return p_Elf2->get6847RamMask();
				break;

				case SUPERELF:
					return p_Super->get6847RamMask();
				break;

				default:
					return 0;
				break;
			}
		break;

		default:
			return 0xffff;
		break;
	}
}

void DebugWindow::onDebugMemType(wxCommandEvent&event)
{
	memoryDisplay_ = event.GetSelection();

	memoryDisplay();

	XRCCTRL(*this, "DebugDisplayPage", wxTextCtrl)->Enable(memoryDisplay_ != CPU_TYPE);
#if defined(__WXMSW__)
	XRCCTRL(*this, "DebugDisplayPageSpinButton", wxSpinButton)->Enable(memoryDisplay_ != CPU_TYPE);
#endif
	XRCCTRL(*this, "DebugSave", wxButton)->Enable(memoryDisplay_ != CPU_TYPE);
	XRCCTRL(*this, "DebugCopy", wxButton)->Enable(memoryDisplay_ != CPU_TYPE);
}

void DebugWindow::onDebugExpansionSlot(wxCommandEvent&WXUNUSED(event))
{
	wxString value = XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->GetValue();

	if (!computerRunning_) return;

	long selection;
	if (!value.ToLong(&selection))
		return;

	if (p_Comx->getComxExpansionSlot() == selection-1)  return;
	XRCCTRL(*this, "DebugExpansionSlot", SlotEdit)->saveNumber(selection);

	int slot = 1 << selection;
	int bank = p_Computer->getOutValue(1) & 0xe0;
	slot = bank | slot;

	p_Comx->out(1, 0, slot);
}

void DebugWindow::onDebugExpansionRam(wxCommandEvent&WXUNUSED(event))
{
	wxString value = XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->GetValue();

	if (!computerRunning_) return;

	long selection;
	if (!value.ToLong(&selection))
		return;

	if (p_Comx->getComxExpansionRamBank() == selection)  return;
	XRCCTRL(*this, "DebugExpansionRam", SlotEdit)->saveNumber(selection);

	int bank = selection << 5;
	int slot = p_Computer->getOutValue(1) & 0x1f;
	slot = bank | slot;

	p_Comx->out(1, 0, slot);
}

void DebugWindow::onDebugExpansionEprom(wxCommandEvent&WXUNUSED(event))
{
	wxString value = XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->GetValue();

	if (!computerRunning_) return;

	long selection;
	if (!value.ToLong(&selection))
		return;

	if (p_Comx->getComxExpansionEpromBank() == selection)  return;
	XRCCTRL(*this, "DebugExpansionEprom", SlotEdit)->saveNumber(selection);

	int bank = selection << 5;
	int slot = p_Computer->getOutValue(1) & 0x1f;
	slot = bank | slot;

	p_Comx->out(1, 0, slot);
}

void DebugWindow::onDebugEmsPage(wxCommandEvent&WXUNUSED(event))
{
	if (!elfConfiguration[runningComputer_].useEms)
		return;

	wxString value = XRCCTRL(*this, "DebugEmsPage", HexEdit)->GetValue();

	if (!computerRunning_) return;

	long page;
	if (!value.ToLong(&page, 16))
		return;

	XRCCTRL(*this, "DebugEmsPage", HexEdit)->saveNumber(page);

	if (page > 0x1f)
		page = 0x1f;

	p_Computer->setEmsPage(page);
}

void DebugWindow::onDebugPager(wxCommandEvent&WXUNUSED(event))
{
	if (!elfConfiguration[runningComputer_].usePager)
		return;

	wxString value = XRCCTRL(*this, "DebugPager", HexEdit)->GetValue();

	if (!computerRunning_) return;

	long page;
	if (!value.ToLong(&page, 16))
		return;

	XRCCTRL(*this, "DebugPager", HexEdit)->saveNumber(page);

	int selectOutput = getConfigItem(computerInfo[runningComputer_].gui+"/PortExtenderSelectOutput", 5l);
	p_Computer->out(selectOutput, 0, portExtender_);

	int writeOutput = getConfigItem(computerInfo[runningComputer_].gui+"PortExtenderWriteOutput", 6l);
	p_Computer->out(writeOutput, 0, page);
}

void DebugWindow::onDebugPortExtender(wxCommandEvent&WXUNUSED(event))
{
	if (!elfConfiguration[runningComputer_].usePager)
		return;

	wxString value = XRCCTRL(*this, "DebugPortExtender", HexEdit)->GetValue();

	if (!computerRunning_) return;

	long portExt;
	if (!value.ToLong(&portExt, 16))
		return;

	portExtender_ = portExt;

	XRCCTRL(*this, "DebugPager", HexEdit)->changeNumber(p_Computer->getPager(portExtender_));

	memoryDisplay();
}

void DebugWindow::onDebugCopy(wxCommandEvent&WXUNUSED(event))
{
	long start = get16BitValue("DebugCopyStart");
	if (start == -1)  return;

	if (start > getAddressMask())
	{
		(void)wxMessageBox( "Please specify a start value within specified memory\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	long end = get16BitValue("DebugCopyEnd");
	if (end == -1)  return;

	if (end > getAddressMask())
	{
		(void)wxMessageBox( "Please specify a end value within specified memory\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	long destination = get16BitValue("DebugCopyTo");
	if (destination == -1)  return;

	if (destination > getAddressMask())
	{
		(void)wxMessageBox( "Please specify a destination value within specified memory\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	while(start <= end)
	{
		debugWriteMem(destination++, debugReadMem(start++));
	}
}

void DebugWindow::onDebugCopyStart(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugCopyStart");
}

void DebugWindow::onDebugCopyEnd(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugCopyEnd");
}

void DebugWindow::onDebugCopyTo(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugCopyTo");
}

void DebugWindow::onDebugAssemblerAddress(wxCommandEvent&WXUNUSED(event))
{
	long address = get16BitValue("DebugAssemblerAddress");
	if (address == -1)  return;

	debugAddress_ =	address;
}

void DebugWindow::onDebugDisStart(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugDisStart");
}

void DebugWindow::onDebugDisEnd(wxCommandEvent&WXUNUSED(event))
{
	get16BitValue("DebugDisEnd");
}


Byte DebugWindow::debugReadMem(Word address)
{
	switch (memoryDisplay_)
	{
		case CPU_MEMORY:
			return p_Computer->readMem(address);
		break;

		case CDP_1870_C:
			switch (runningComputer_)
			{
				case COMX:
					return p_Comx->readCramDirect(address);
				break;
				case CIDELSA:
					return p_Cidelsa->readCramDirect(address);
				break;
				case TMC600:
					return p_Tmc600->readCramDirect(address);
				break;
				case PECOM:
					return p_Pecom->readCramDirect(address);
				break;
				default:
					return 0;
				break;
			}
		break;

		case CDP_1870_P:
			switch (runningComputer_)
			{
				case COMX:
					return p_Comx->readPramDirect(address);
				break;
				case CIDELSA:
					return p_Cidelsa->readPramDirect(address);
				break;
				case TMC600:
					return p_Tmc600->readPramDirect(address);
				break;
				case PECOM:
					return p_Pecom->readPramDirect(address);
				break;
				default:
					return 0;
				break;
			}
		break;

		case TMS_MEMORY:
			switch(runningComputer_)
			{
				case ELF:
					return p_Elf->getTmsMemory(address);
				break;

				case ELFII:
					return p_Elf2->getTmsMemory(address);
				break;

				case SUPERELF:
					return p_Super->getTmsMemory(address);
				break;

				default:
					return 0;
				break;
			}
		break;

		case VT_RAM:
			if ((computerRunning_ && (p_Vt100 != NULL)))
				return p_Vt100->getVtMemory(address);
			else
				return 0;
		break;

		case CDP_1864:
			switch(runningComputer_)
			{
				case TMC600:
					return p_Tmc2000->read1864ColorDirect(address);
				break;
				case ETI:
					return p_Eti->read1864ColorDirect(address);
				break;
				case VIP:
					return p_Vip->read1864ColorDirect(address);
				break;
				default:
					return 0;
				break;
			}
		break;

		case I_8275:
			switch(runningComputer_)
			{
				case ELF:
					return p_Elf->read8275CharRom(address);
				break;

				case ELFII:
					return p_Elf2->read8275CharRom(address);
				break;

				case SUPERELF:
					return p_Super->read8275CharRom(address);
				break;

				case ELF2K:
					return p_Elf2K->read8275CharRom(address);
				break;

				default:
					return 0;
				break;
			}
		break;

		case V_6845:
			switch(runningComputer_)
			{
				case COMX:
					return p_Comx->read6845CharRom(address);
				break;

				case ELF:
					return p_Elf->read6845CharRom(address);
				break;

				case ELFII:
					return p_Elf2->read6845CharRom(address);
				break;

				case SUPERELF:
					return p_Super->read6845CharRom(address);
				break;

				default:
					return 0;
				break;
			}
		break;

		case V_6847:
			switch(runningComputer_)
			{
				case ELF:
					return p_Elf->read6847CharRom(address);
				break;

				case ELFII:
					return p_Elf2->read6847CharRom(address);
				break;

				case SUPERELF:
					return p_Super->read6847CharRom(address);
				break;

				default:
					return 0;
				break;
			}
		break;

		case V_6847_RAM:
			switch(runningComputer_)
			{
				case ELF:
					return p_Elf->readDirect6847(address);
				break;

				case ELFII:
					return p_Elf2->readDirect6847(address);
				break;

				case SUPERELF:
					return p_Super->readDirect6847(address);
				break;

				default:
					return 0;
				break;
			}
		break;

		default:
			return 0;
		break;
	}
}

void DebugWindow::debugWriteMem(Word address, Byte value)
{
	switch (memoryDisplay_)
	{
		case CPU_MEMORY:
			p_Computer->writeMem(address, value, false);
		break;

		case CDP_1870_C:
			switch (runningComputer_)
			{
				case COMX:
					p_Comx->writeCramDirect(address, value);
				break;
				case CIDELSA:
					p_Cidelsa->writeCramDirect(address, value);
				break;
				case TMC600:
					p_Tmc600->writeCramDirect(address, value);
				break;
				case PECOM:
					p_Pecom->writeCramDirect(address, value);
				break;
			}
		break;

		case CDP_1870_P:
			switch (runningComputer_)
			{
				case COMX:
					p_Comx->writePramDirect(address, value);
				break;
				case CIDELSA:
					p_Cidelsa->writePramDirect(address, value);
				break;
				case TMC600:
					p_Tmc600->writePramDirect(address, value);
				break;
				case PECOM:
					p_Pecom->writePramDirect(address, value);
				break;
			}
		break;

		case TMS_MEMORY:
			switch(runningComputer_)
			{
				case ELF:
					p_Elf->setTmsMemory(address, value);
				break;

				case ELFII:
					p_Elf2->setTmsMemory(address, value);
				break;

				case SUPERELF:
					p_Super->setTmsMemory(address, value);
				break;
			}
		break;

		case VT_RAM:
			if (computerRunning_ && (p_Vt100 != NULL))
				p_Vt100->setVtMemory(address, value);
		break;

		case CDP_1864:
			switch (runningComputer_)
			{
				case ETI:
					p_Eti->write1864ColorDirect(address, value);
				break;
				case TMC2000:
					p_Tmc2000->write1864ColorDirect(address, value);
				break;
				case VIP:
					p_Vip->write1864ColorDirect(address, value);
				break;
			}
		break;

		case I_8275:
			switch(runningComputer_)
			{
				case ELF:
					p_Elf->write8275CharRom(address, value);
				break;

				case ELFII:
					p_Elf2->write8275CharRom(address, value);
				break;

				case SUPERELF:
					p_Super->write8275CharRom(address, value);
				break;

				case ELF2K:
					p_Elf2K->write8275CharRom(address, value);
				break;
			}
		break;

		case V_6845:
			switch(runningComputer_)
			{
				case COMX:
					p_Comx->write6845CharRom(address, value);
				break;

				case ELF:
					p_Elf->write6845CharRom(address, value);
				break;

				case ELFII:
					p_Elf2->write6845CharRom(address, value);
				break;

				case SUPERELF:
					p_Super->write6845CharRom(address, value);
				break;
			}
		break;

		case V_6847:
			switch(runningComputer_)
			{
				case ELF:
					p_Elf->write6847CharRom(address, value);
				break;

				case ELFII:
					p_Elf2->write6847CharRom(address, value);
				break;

				case SUPERELF:
					p_Super->write6847CharRom(address, value);
				break;
			}
		break;

		case V_6847_RAM:
			switch(runningComputer_)
			{
				case ELF:
					p_Elf->writeDirect6847(address, value);
				break;

				case ELFII:
					p_Elf2->writeDirect6847(address, value);
				break;

				case SUPERELF:
					p_Super->writeDirect6847(address, value);
				break;
			}
		break;
	}
}

void DebugWindow::setSwName(wxString swName)
{
	swName_ = swName;
	if (swName != "")  
		swName_ = ", Software: " + swName_;
	updateTitle();
}

void DebugWindow::updateTitle()
{
	wxString title;

	title = swName_;
	if (debugMode_)
	{
		title = swName_ + ", Debug Mode";
		if (chip8DebugMode_)
			title = swName_ + ", Chip-8 and 1802 Debug Mode";
	}
	else
	{
		if (chip8DebugMode_)
			title = swName_ + ", Chip-8 Debug Mode";
	}

	switch (runningComputer_)
	{
		case COMX:
			if (p_Comx->getSteps()==0)
				title = title + " ** PAUSED **";
			p_Comx->SetTitle("COMX-35" + title);
			p_Comx->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case CIDELSA:
			if (p_Cidelsa->getSteps()==0)
				title = title + " ** PAUSED **";
			p_Cidelsa->SetTitle("Cidelsa" + title);
			p_Cidelsa->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case TMC600:
			if (p_Tmc600->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Tmc600->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Tmc600->SetTitle("Telmac" + title);
			p_Tmc600->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case PECOM:
			if (p_Pecom->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Pecom->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Pecom->SetTitle("Pecom 64" + title);
			p_Pecom->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case STUDIO:
			if (p_Studio2->getSteps()==0)
				title = title + " ** PAUSED **";
			p_Studio2->SetTitle("Studio II" + title);
			p_Studio2->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case VISICOM:
			if (p_Visicom->getSteps()==0)
				title = title + " ** PAUSED **";
			p_Visicom->SetTitle("Visicom COM-100" + title);
			p_Visicom->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case VICTORY:
			if (p_Victory->getSteps()==0)
				title = title + " ** PAUSED **";
			p_Victory->SetTitle("Victory MPT-02" + title);
			p_Victory->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case VIP:
			if (p_Vip->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Vip->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Vip->SetTitle("Cosmac VIP" + title);
			p_Vip->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case TMC2000:
			if (p_Tmc2000->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Tmc2000->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Tmc2000->SetTitle("Telmac 2000" + title);
			p_Tmc2000->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case TMC1800:
			if (p_Tmc1800->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Tmc1800->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Tmc1800->SetTitle("Telmac 1800" + title);
			p_Tmc1800->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case ETI:
			if (p_Eti->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Eti->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Eti->SetTitle("ETI 660" + title);
			p_Eti->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case NANO:
			if (p_Nano->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Nano->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Nano->SetTitle("Telmac Nano" + title);
			p_Nano->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case ELF2K:
			if (p_Elf2K->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Elf2K->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Elf2K->SetTitle("Elf 2000" + title);
			p_Elf2K->updateTitle(title);
			p_Elf2K->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case COSMICOS:
			if (p_Cosmicos->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Cosmicos->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Cosmicos->SetTitle("Cosmicos" + title);
			p_Cosmicos->updateTitle(title);
			p_Cosmicos->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case MEMBER:
			if (p_Membership->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Membership->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Membership->SetTitle("Membership Card" + title);
			p_Membership->updateTitle(title);
			p_Membership->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case ELF:
			if (p_Elf->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Elf->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Elf->SetTitle("Elf" + title);
			p_Elf->updateTitle(title);
			p_Elf->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case ELFII:
			if (p_Elf2->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Elf2->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Elf2->SetTitle("Elf II" + title);
			p_Elf2->updateTitle(title);
			p_Elf2->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;

		case SUPERELF:
			if (p_Super->getSteps()==0)
				title = title + " ** PAUSED **";
			if (p_Super->getClear()==0)
				title = title + " ** CPU STOPPED **";
			p_Super->SetTitle("Super Elf" + title);
			p_Super->updateTitle(title);
			p_Super->setDebugMode(debugMode_, chip8DebugMode_, trace_, traceDma_, traceInt_, traceChip8Int_);
		break;
	}
}

void DebugWindow::updateDebugMenu(bool debugMode)
{
	debugMode_ = debugMode;
	XRCCTRL(*this,"DebugMode", wxCheckBox)->SetValue(debugMode_);
	if (computerRunning_)
	{
		if (!debugMode_ && !chip8DebugMode_)
			p_Computer->setPercentageClock(1);
		else
			p_Computer->setPercentageClock(percentageClock_);
	}
	XRCCTRL(*this,"PercentageClock", wxSlider)->Enable(debugMode_);
	XRCCTRL(*this,"PercentageClockText", wxStaticText)->Enable(debugMode_);
	updateTitle();
}

void DebugWindow::onDebugMode(wxCommandEvent&event)
{
	debugMode_ = event.IsChecked();
	updateDebugMenu(debugMode_);
}

void DebugWindow::onFxDebugMode()
{
	if (!mode_.gui)
		return;
	debugMode_ = !debugMode_;
 	updateDebugMenu(debugMode_);
}

void DebugWindow::updateChip8DebugMenu(bool debugMode)
{
	chip8DebugMode_ = debugMode;
	XRCCTRL(*this,"Chip8DebugMode", wxCheckBox)->SetValue(chip8DebugMode_);
	updateTitle();
}

void DebugWindow::onChip8DebugMode(wxCommandEvent&event)
{
	chip8DebugMode_ = event.IsChecked();
	if (computerRunning_)
	{
		if (!debugMode_ && !chip8DebugMode_)
			p_Computer->setPercentageClock(1);
		else
			p_Computer->setPercentageClock(percentageClock_);
	}
	updateTitle();
	enableChip8DebugGui(true);
}

void DebugWindow::onChip8PauseButton(wxCommandEvent&WXUNUSED(event))
{
	if (chip8Steps_ < 0)
	{
		chip8Steps_ = 1;
		XRCCTRL(*this, "Chip8PauseButton", wxBitmapButton)->SetBitmapLabel(pauseOnBitmap);
		XRCCTRL(*this,"Chip8PauseButton", wxBitmapButton)->Enable(false);
		updateChip8DebugMenu(true);
	}
	else
	{
		chip8Steps_ = -1;
		p_Computer->setSteps(-1);
		setChip8PauseState();
	}
}

void DebugWindow::setChip8PauseState()
{
	if (chip8Steps_ == 0)
	{
		XRCCTRL(*this, "Chip8PauseButton", wxBitmapButton)->SetBitmapLabel(pauseOnBitmap);
		XRCCTRL(*this,"Chip8PauseButton", wxBitmapButton)->Enable(true);
		XRCCTRL(*this,"Chip8StepButton", wxBitmapButton)->Enable(true);
	}
	else
	{
		if (chip8Steps_ > 0)
		{
			XRCCTRL(*this, "Chip8PauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);
			XRCCTRL(*this,"Chip8StepButton", wxBitmapButton)->Enable(true);
		}
		else
		{
			XRCCTRL(*this, "Chip8PauseButton", wxBitmapButton)->SetBitmapLabel(pauseOffBitmap);
			XRCCTRL(*this,"Chip8StepButton", wxBitmapButton)->Enable(false);
		}
	}
	updateTitle();
}

void DebugWindow::onChip8StepButton(wxCommandEvent&WXUNUSED(event))
{
	wxString stepsStr = XRCCTRL(*this,"Chip8NumberOfSteps",wxTextCtrl)->GetValue();
	if (!stepsStr.ToLong(&chip8Steps_))
		chip8Steps_ = 1;

	chip8Steps_++;
	setChip8PauseState();
	p_Computer->setSteps(-1);
	performChip8Step_ = true;
}

void DebugWindow::chip8ShowInstruction(wxString buffer)
{
	if (!chip8DebugMode_)  return;
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiEnter();
#endif
	chip8TraceWindowPointer->AppendText(buffer);
	if (!additionalChip8Details_)
		chip8TraceWindowPointer->AppendText("\n");
#if defined(__WXGTK__)
	if (!wxIsMainThread())
		wxMutexGuiLeave();
#endif
}

void DebugWindow::onChip8Clear(wxCommandEvent& WXUNUSED(event))
{
	chip8TraceWindowPointer->Clear();
}

void DebugWindow::chip8Trace(Word address)
{	
	chip8ShowInstruction(chip8Disassemble(address, true, false));
}

wxString DebugWindow::chip8Disassemble(Word dis_address, bool includeDetails, bool showOpcode)
{
	Byte chip8_opcode1 = p_Computer->readMem(dis_address);
	Byte chip8_opcode2 = p_Computer->readMem(dis_address + 1);
	wxString buffer, detailsBuffer, addressStr;

	buffer = "";
	detailsBuffer = "";

	//Instruction value
	int vX = chip8_opcode1&0xf;
	int vY = (chip8_opcode2>>4)&0xf;
	Word address = ((chip8_opcode1 & 0xf) << 8) + chip8_opcode2;
	Byte nibble = chip8_opcode2&0xf;
	Byte value = chip8_opcode2;

	//Real time value
	Word addressX = p_Computer->getChip8baseVar() + vX;
	Word addressY = p_Computer->getChip8baseVar() + vY;
	Byte valueX = p_Computer->readMem(addressX);
	Byte valueY = p_Computer->readMem(addressY);
	Word valueI = p_Computer->getScratchpadRegister(CHIP8_I);

	//Calculation variables
	int resultValue;
	wxString tempStr1;
	long tempLong1, tempLong2;

	if (showOpcode)
		addressStr.Printf("%04X: %02X%02X  ", dis_address, chip8_opcode1, chip8_opcode2);
	else
		addressStr.Printf("%03X", dis_address&0xfff);

	switch (chip8_opcode1 & 0xf0)
	{
		case 0:
			if (p_Computer->getChip8Type() == CHIP8X && chip8_opcode1 == 0x02 && chip8_opcode2 == 0xA0)
				buffer.Printf(" CLR   BACK");
			else
				switch (chip8_opcode2)
				{
					case 0xe0:
						buffer.Printf(" CLS");
					break;

					case 0xee:
						buffer.Printf(" RET");
					break;

					default:
						buffer.Printf(" SYS   %03X", address);
					break;
				}
		break;

		case 0x10:
			buffer.Printf(" JP    %03X", address );
		break;

		case 0x20:
			buffer.Printf(" CALL  %03X", address );
		break;

		case 0x30:
			buffer.Printf(" SE    V%01X, %02X", vX, value);
		break;

		case 0x40:
			buffer.Printf(" SNE   V%01X, %02X", vX, value);
		break;

		case 0x50:
			switch (chip8_opcode2 & 0xf)
			{
				case 0x0:
					buffer.Printf(" SE    V%01X, V%01X", vX, vY);
				break;

				case 0x1:
					if (p_Computer->getChip8Type() == CHIP8X)
					{
						tempStr1.Printf("%01X", (valueX>>4)&0x7);   
						tempStr1.ToLong(&tempLong1, 8); 
						tempStr1.Printf("%01X", (valueY>>4)&0x7);   
						tempStr1.ToLong(&tempLong2, 8); 
						resultValue = ((tempLong1 + tempLong2) & 0x7) << 4;
						tempStr1.Printf("%01X", valueX&0x7);   
						tempStr1.ToLong(&tempLong1, 8); 
						tempStr1.Printf("%01X", valueY&0x7);   
						tempStr1.ToLong(&tempLong2, 8); 
						resultValue = resultValue + ((tempLong1 + tempLong2)&0x7);

						buffer.Printf(" ADD-8 V%01X, V%01X", vX, vY);
						detailsBuffer.Printf("V%01X=%02X", vX, (resultValue)&0xff);
					}
					else
						buffer.Printf(" Illegal instruction");
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0x60:
			buffer.Printf(" LD    V%01X, %02X", vX, value);
		break;

		case 0x70:
			buffer.Printf(" ADD   V%01X, %02X", vX, value);
			detailsBuffer.Printf("V%01X=%02X", vX, (valueX + value)&0xff);
		break;

		case 0x80:
			switch (chip8_opcode2 & 0xf)
			{
				case 0x0:
					buffer.Printf(" LD    V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueY);
				break;

				case 0x1:
					buffer.Printf(" OR    V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX | valueY);
				break;

				case 0x2:
					buffer.Printf(" AND   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX & valueY);
				break;

				case 0x3:
					buffer.Printf(" XOR   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX ^ valueY);
				break;

				case 0x4:
					buffer.Printf(" ADD   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueX + valueY)&0xff);
				break;

				case 0x5:
					buffer.Printf(" SUB   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueX - valueY)&0xff);
				break;

				case 0x6:
					if (vX == vY)
						buffer.Printf(" SHR   V%01X", vX);
					else
						buffer.Printf(" SHR   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueY>>1);
				break;

				case 0x7:
					buffer.Printf(" SUBN  V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueY - valueX)&0xff);
				break;

				case 0xE:
					if (vX == vY)
						buffer.Printf(" SHL   V%01X", vX);
					else
						buffer.Printf(" SHL   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueY<<1)&0xff);
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0x90:
			if ((chip8_opcode2 & 0xf) == 0)
				buffer.Printf(" SNE   V%01X, V%01X", vX, vY);
			else
				buffer.Printf(" Illegal instruction");
		break;

		case 0xA0:
			buffer.Printf(" LD    I, %03X", address );
		break;

		case 0xB0:
			if (p_Computer->getChip8Type() == CHIP8X)
			{
				if ((chip8_opcode2 & 0xf) == 0)
					buffer.Printf(" CLR-L V%01X/V%01X, V%01X", vX, (vX+1)&0xf, vY);
				else
					buffer.Printf(" CLR-H V%01X/V%01X, V%01X, %01X", vX, (vX+1)&0xf, vY, nibble);
			}
			else
			{
				buffer.Printf(" JP    V0, %03X", address);
				detailsBuffer.Printf("JP %03X", (address + p_Computer->readMem(p_Computer->getChip8baseVar()))&0xfff);
			}
		break;

		case 0xC0:
			if (additionalChip8Details_)
			{
				addressStr = "";
				detailsBuffer.Printf("V%01X=%02X", vX, valueX);
				additionalChip8Details_ = false;
			}
			else
			{
				buffer.Printf(" RND   V%01X, %02X", vX, value);
				if (includeDetails)
					additionalChip8Details_ = true;
			}
		break;

		case 0xD0:
			buffer.Printf(" DRW   V%01X, V%01X, %01X", vX, vY, nibble);
		break;

		case 0xE0:
			switch (chip8_opcode2)
			{
				case 0x9E:
					buffer.Printf(" SKP   V%01X", vX);
				break;

				case 0xA1:
					buffer.Printf(" SKNP  V%01X", vX);
				break;

				case 0xF2:
					if (p_Computer->getChip8Type() == CHIP8X)
						buffer.Printf(" SKP2  V%01X", vX);
					else
						buffer.Printf(" Illegal instruction");
				break;

				case 0xF5:
					if (p_Computer->getChip8Type() == CHIP8X)
						buffer.Printf(" SKNP2 V%01X", vX);
					else
						buffer.Printf(" Illegal instruction");
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0xF0:
			switch (chip8_opcode2)
			{
				case 0x0:
					if (runningComputer_ == ETI)
					{
						buffer.Printf(" LD    P, V%01X", vX);
						detailsBuffer.Printf("P=%02X", valueX);
					}
					else
						buffer.Printf(" Illegal instruction");
				break;

				case 0x07:
					if (additionalChip8Details_)
					{
						addressStr = "";
						detailsBuffer.Printf("V%01X=%02X", vX, valueX);
						additionalChip8Details_ = false;
					}
					else
					{
						buffer.Printf(" LD    V%01X, DT", vX);
						if (includeDetails)
							additionalChip8Details_ = true;
					}
				break;

				case 0x0A:
					if (additionalChip8Details_)
					{
						addressStr = "";
						detailsBuffer.Printf("V%01X=%02X", vX, valueX);
						additionalChip8Details_ = false;
					}
					else
					{
						buffer.Printf(" LD    V%01X, K", vX);
						if (includeDetails)
							additionalChip8Details_ = true;
					}
				break;

				case 0x15:
					buffer.Printf(" LD    DT, V%01X", vX);
					detailsBuffer.Printf("DT=%02X", valueX);
				break;

				case 0x18:
					buffer.Printf(" LD    ST, V%01X", vX);
					detailsBuffer.Printf("ST=%d ms", valueX*20);
				break;

				case 0x1E:
					buffer.Printf(" ADD   I, V%01X", vX);
					detailsBuffer.Printf("I=%03X", (valueI+valueX)&0xfff);
				break;

				case 0x29:
					buffer.Printf(" LD    F, V%01X", vX);
					detailsBuffer.Printf("I=%03X", chip8Sprite[valueX&0xf]);
				break;

				case 0x33:
					buffer.Printf(" LD    B, V%01X", vX);
					detailsBuffer.Printf("[%03X->%03X]='%03d'", valueI, (valueY+2)&0xfff, valueX);
				break;

				case 0x55:
					buffer.Printf(" LD    [I], V%01X", vX);
				break;

				case 0x65:
					buffer.Printf(" LD    V%01X, [I]", vX);
				break;

				case 0xF8:
					if (p_Computer->getChip8Type() == CHIP8X)
						buffer.Printf(" OUT-3 V%01X", vX);
					else
						buffer.Printf(" Illegal instruction");
				break;

				case 0xFB:
					if (p_Computer->getChip8Type() == CHIP8X)
					{
						if (additionalChip8Details_)
						{
							addressStr = "";
							detailsBuffer.Printf("V%01X=%02X", vX, p_Computer->getInValue(1));
							additionalChip8Details_ = false;
						}
						else
						{
							buffer.Printf(" INP-1 V%01X", vX);
							if (includeDetails)
								additionalChip8Details_ = true;
						}
					}
					else
						buffer.Printf(" Illegal instruction");
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;
	}	
	while (buffer.Len() < 21 && addressStr != "")
		buffer += " ";
	buffer = addressStr + buffer;
	if (includeDetails)
		buffer = buffer + detailsBuffer;
	return buffer;
}

void DebugWindow::st2Trace(Word address)
{	
	chip8ShowInstruction(st2Disassemble(address, true, false));
}

wxString DebugWindow::st2Disassemble(Word dis_address, bool includeDetails, bool showOpcode)
{	
	Byte chip8_opcode1 = p_Computer->readMem(dis_address);
	Byte chip8_opcode2 = p_Computer->readMem(dis_address + 1);
	wxString buffer, detailsBuffer, addressStr;

	buffer = "";
	detailsBuffer = "";

	//Instruction value
	int vX = chip8_opcode1&0xf;
	int vY = (chip8_opcode2>>4)&0xf;
	Word address = ((chip8_opcode1 & 0xf) << 8) + chip8_opcode2;
//	Byte nibble = chip8_opcode2&0xf;
	Byte value = chip8_opcode2;

	//Real time value
	Word addressX = p_Computer->getChip8baseVar() + vX;
	Word addressY = p_Computer->getChip8baseVar() + vY;
	Byte valueX = p_Computer->readMem(addressX);
	Byte valueY = p_Computer->readMem(addressY);
	Word valueI = p_Computer->getScratchpadRegister(CHIP8_I);

	//Calculation variables
//	int resultValue;
	wxString tempStr1;
//	long tempLong1, tempLong2;

	if (showOpcode)
		addressStr.Printf("%04X: %02X%02X  ", dis_address, chip8_opcode1, chip8_opcode2);
	else
		addressStr.Printf("%03X", dis_address&0xfff);

	switch (chip8_opcode1 & 0xf0)
	{
		case 0:
			switch (chip8_opcode2)
			{
				case 0x66:
					buffer.Printf(" VIDEO ON");
				break;

				default:
					if (chip8_opcode1 == 0x2 && chip8_opcode2 == 0xF2)
					{
						buffer.Printf(" LD    [->I], 0");
						detailsBuffer.Printf("[%03X->%03X]=0", valueI&0xf00, valueI);
					}
					else
					{
						buffer.Printf(" SYS   %03X", address );
					}
				break;
			}
		break;

		case 0x10:
			buffer.Printf(" JP    %03X", address );
		break;

		case 0x20:
			buffer.Printf(" CALL  %03X", address );
		break;

		case 0x30:
			buffer.Printf(" JNZ   V%01X, %02X", vX, value);
		break;

		case 0x40:
			buffer.Printf(" JZ    V%01X, %02X", vX, value);
		break;

		case 0x50:
			buffer.Printf(" SNE   V%01X, %02X", vX, value);
		break;

		case 0x60:
			buffer.Printf(" LD    V%01X, %02X", vX, value);
		break;

		case 0x70:
			if (chip8_opcode1 == 0x70)
				buffer.Printf(" DJNZ  V0, %02X", value);
			else
			{
				buffer.Printf(" ADD   V%01X, %02X", vX, value);
				detailsBuffer.Printf("V%01X=%02X", vX, (valueX + value)&0xff);
			}
		break;

		case 0x80:
			switch (chip8_opcode2 & 0xf)
			{
				case 0x0:
					buffer.Printf(" LD    V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueY);
				break;

				case 0x1:
					buffer.Printf(" OR    V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX | valueY);
				break;

				case 0x2:
					buffer.Printf(" AND   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX & valueY);
				break;

				case 0x3:
					buffer.Printf(" XOR   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueX ^ valueY);
				break;

				case 0x4:
					buffer.Printf(" ADD   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueX + valueY)&0xff);
				break;

				case 0x5:
					buffer.Printf(" SUB   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueX - valueY)&0xff);
				break;

				case 0x6:
					if (vX == vY)
						buffer.Printf(" SHR   V%01X", vX);
					else
						buffer.Printf(" SHR   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, valueY>>1);
				break;

				case 0x7:
					buffer.Printf(" SUBN  V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueY - valueX)&0xff);
				break;

				case 0xE:
					if (vX == vY)
						buffer.Printf(" SHL   V%01X", vX);
					else
						buffer.Printf(" SHL   V%01X, V%01X", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, (valueY<<1)&0xff);
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0x90:
			switch (chip8_opcode2 & 0xf)
			{
				case 0x0:
					buffer.Printf(" SNE   V%01X, V%01X", vX, vY);
				break;

				case 0x1:
					buffer.Printf(" LD    V%01X, V%01X", vY, vX);
					detailsBuffer.Printf("V%01X=%02X", vY, valueX);
				break;

				case 0x2:
					buffer.Printf(" LD    V%01X, [V%01X]", vX, vY);
					detailsBuffer.Printf("V%01X=%02X", vX, p_Computer->readMem(0x800+valueY));
				break;

				case 0x4:
					buffer.Printf(" LD    [V%01X], V%01X", vY, vX);
					detailsBuffer.Printf("[8%02X]=%02X", valueY, valueX);
				break;

				case 0x8:
					buffer.Printf(" LD    B, [V%01X], V%01X", vY, vX);
					detailsBuffer.Printf("[%03X->%03X]='%03d'", (valueY+0x800)&0xfff, (valueY+0x802)&0xfff, valueX);
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0xA0:
			buffer.Printf(" LD    I, %03X", address);
		break;

		case 0xB0:
			buffer.Printf(" LD    [I], %02X, +%01X", value, vX);
			detailsBuffer.Printf("[%03X]=%02X, I=%03X", valueI, value, (valueI+vX)&0xfff);
		break;

		case 0xC0:
			if (chip8_opcode1 == 0xc0)
			{
				if (showOpcode)
					addressStr.Printf("%03X: %02X    ", dis_address, chip8_opcode1);
				buffer.Printf(" RET");
			}
			else
			{
				if (additionalChip8Details_)
				{
					addressStr = "";
					detailsBuffer.Printf("V%01X=%02X", vX, valueX);
					additionalChip8Details_ = false;
				}
				else
				{
					buffer.Printf(" RND   V%01X, %02X", vX, value);
					if (includeDetails)
						additionalChip8Details_ = true;
				}
			}
		break;

		case 0xD0:
			if (chip8_opcode1 == 0xDF)
				buffer.Printf(" JKP   VB, %02X", value);
			else
				buffer.Printf(" JKP   %01X, %02X", vX, value);
		break;

		case 0xE0:
			if (showOpcode && (chip8_opcode1 != 0xe8))
				addressStr.Printf("%03X: %02X    ", dis_address, chip8_opcode1);
			switch (chip8_opcode1)
			{
				case 0xE0:
					buffer.Printf(" SP    CLEAR");
				break;

				case 0xE1:
					buffer.Printf(" SP    MOVE");
				break;

				case 0xE2:
					buffer.Printf(" SP    VC, MOVE");
				break;

				case 0xE4:
					buffer.Printf(" SP    XOR, [I]");
				break;

				case 0xE8:
					buffer.Printf(" SP    DRW, JC, %02X", value);
				break;

				default:
					buffer.Printf(" Illegal instruction");
				break;
			}
		break;

		case 0xF0:
			if (vX == 0)
			{
				buffer.Printf(" SYS   2%02X", chip8_opcode2);
			}
			else
			{
				switch (chip8_opcode2)
				{
					case 0xa6:
						buffer.Printf(" LDM   V%01X, [I]", vX);
						detailsBuffer.Printf("V%01X=%02X", vX, p_Computer->readMem(valueI));
					break;

					case 0xa9:
						buffer.Printf(" LDM   [I], V%01X", vX);
						detailsBuffer.Printf("[%03X]=%02X", valueI, valueX);
					break;

					case 0xac:
						buffer.Printf(" LDMA   V%01X, [I]", vX);
						detailsBuffer.Printf("V%01X=%02X, I=%03X", vX, p_Computer->readMem(valueI), (valueI+1)&0xfff);
					break;

					case 0xaf:
						buffer.Printf(" LDMA  [I], V%01X", vX);
						detailsBuffer.Printf("[%03X]=%02X, I=%03X", valueI, value, (valueI+1)&0xff);
					break;

					case 0xb3:
						buffer.Printf(" SLSB  V%01X", vX);
						detailsBuffer.Printf("I=%03X", (valueI&0xf00) | valueX);
					break;

					case 0xb6:
						buffer.Printf(" AFOR  V%01X", vX);
						detailsBuffer.Printf("V%01X=%02X", (valueX&0xf) | (valueI&0xff));
					break;

					default:
						buffer.Printf(" Illegal instruction");
					break;
				}
			}
		break;
	}	
	while (buffer.Len() < 21 && addressStr != "")
		buffer += " ";
	buffer = addressStr + buffer;
	if (includeDetails)
		buffer = buffer + detailsBuffer;
	return buffer;
}

void DebugWindow::onChip8Trace(wxCommandEvent& WXUNUSED(event))
{
	additionalChip8Details_ = false;
	chip8Trace_ = !chip8Trace_;
	SetChip8DebugMode();
	enableChip8DebugGui(true);
}

void DebugWindow::onChip8ProtectedMode(wxCommandEvent&event)
{
	chip8ProtectedMode_= event.IsChecked();
	enableChip8DebugGui(true);
}

void DebugWindow::onChip8Log(wxCommandEvent& WXUNUSED(event))
{
//	wxSetWorkingDirectory (workingDir_);
	int num = 0;
	wxString fileName, number;

	fileName = wxFileSelector( "Select the log file to save",
                               debugDir_, "chip8_trace.log",
                               "log",
                               wxString::Format
                              (
                                   "Dump File (*.log)|*.log|All files (%s)|%s",
                                   wxFileSelectorDefaultWildcardStr,
                                   wxFileSelectorDefaultWildcardStr
                               ),
                               wxFD_SAVE|wxFD_CHANGE_DIR|wxFD_PREVIEW,
                               this
                              );

	if (!fileName || fileName.empty())
		return;

	wxFileName FullPath = wxFileName(fileName, wxPATH_NATIVE);
	wxString name = FullPath.GetName();
	wxString path = FullPath.GetPath();
	wxString ext = FullPath.GetExt();

	while(wxFile::Exists(fileName))
	{
		num++;
		number.Printf("%d", num);
		fileName = path + pathSeparator_ + name + "." + number + "." + ext;
	}
	chip8TraceWindowPointer->SaveFile(fileName);
}

void DebugWindow::deleteChip8BreakPoint(wxListEvent&event)
{
	long selectedItem = event.GetIndex();

	if (selectedItem > -1)
	{
		for (int i=selectedItem; i<numberOfChip8BreakPoints_-1; i++)
			chip8BreakPoints_[i] = chip8BreakPoints_[i+1];
		numberOfChip8BreakPoints_--;
		SetChip8DebugMode();
	}
}

void DebugWindow::editChip8BreakPoint(wxListEvent&event)
{
	wxString bpStr = event.GetText();
	long selectedItem = event.GetIndex();

	if (bpStr == "")
	{
		chip8BreakPointWindowPointer->DeleteItem(selectedItem);
		event.Veto();
		return;
	}

	wxString strValue = extractWord (&bpStr);
	long value;

	if (!strValue.ToLong(&value, 16))
	{
		(void)wxMessageBox( "Please specify value in hexadecimal\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}

	if (value > 0xfff)
	{
		(void)wxMessageBox( "Please specify value of 12 bit max\n",
									"Emma 02", wxICON_ERROR | wxOK );
		event.Veto();
		return;
	}

	chip8BreakPoints_[selectedItem] = value;
}

void DebugWindow::addChip8BreakPoint()
{
	wxString printBuffer;

	printBuffer.Printf("%03X", chip8BreakPoints_[numberOfChip8BreakPoints_]);
	chip8BreakPointWindowPointer->InsertItem(numberOfChip8BreakPoints_, printBuffer);
	numberOfChip8BreakPoints_++;
}

void DebugWindow::onChip8BreakPointAddress(wxCommandEvent&WXUNUSED(event))
{
	get12BitValue("Chip8BreakPointAddress");
}

void DebugWindow::onChip8BreakPointSet(wxCommandEvent&WXUNUSED(event))
{
	if (numberOfChip8BreakPoints_ == 64)
	{
		(void)wxMessageBox( "Maximum number of Break Points set\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	long chip8BreakPointAddress = get12BitValue("Chip8BreakPointAddress");
	if (chip8BreakPointAddress == -1)
	{
		(void)wxMessageBox( "No Break Point value specified\n",
									"Emma 02", wxICON_ERROR | wxOK );
		return;
	}

	chip8BreakPoints_[numberOfChip8BreakPoints_] = chip8BreakPointAddress;
	chip8BreakPointsSelected_[numberOfChip8BreakPoints_] = true;
	addChip8BreakPoint();
	SetChip8DebugMode();
}

void DebugWindow::onPercentageClock(wxScrollEvent&event)
{
	double percentage = event.GetPosition();
	XRCCTRL(*this,"Chip8PercentageClock", wxSlider)->SetValue((int)percentage);

	if (percentage < 100)
		percentage /= 50;

	percentage = pow (percentage, 2);
	percentageClock_ = (double)percentage / pow ((double)100,2);

	if (computerRunning_)
		p_Computer->setPercentageClock(percentageClock_);
}

void DebugWindow::onChip8PercentageClock(wxScrollEvent&event)
{
	double percentage = event.GetPosition();
	XRCCTRL(*this,"PercentageClock", wxSlider)->SetValue((int)percentage);

	if (percentage < 100)
		percentage /= 5;

	percentage = pow (percentage, 2);
	percentageClock_ = (double)percentage / pow ((double)100,2);

	if (computerRunning_)
		p_Computer->setPercentageClock(percentageClock_);
}
